#pragma once
#include "stdafx.h"
#include "FbxSdkManager.h"
#include "FbxIOPluginRegistry.h"
#include "FbxMemoryAllocator.h"
#include "FbxError.h"
#include "FbxDataType.h"
#include "FbxXRefManager.h"
#include "FbxString.h"
#include "FbxLibrary.h"
#include "FbxScene.h"
#include "FbxIOSettings.h"

namespace Skill
{
	namespace FbxSDK
	{	
		void FbxSdkManager::CollectManagedMemory()
		{
			this->_IOPluginRegistry = nullptr;
			this->_SystemLibraries = nullptr;
			this->_UserLibraries = nullptr;
			this->_XRefManager = nullptr;
			this->_RootLibrary = nullptr;
			this->_KError = nullptr;			
		}		
		bool FbxSdkManager::SetMemoryAllocator(FbxMemoryAllocator^ memoryAllocator)
		{
			return KFbxSdkManager::SetMemoryAllocator(memoryAllocator->_Ref());
		}
		FbxSdkManager^ FbxSdkManager::Create()
		{
			KFbxSdkManager* km = KFbxSdkManager::Create();
			FbxSdkManager^ m = nullptr;
			if(km)
			{
				m = gcnew FbxSdkManager(km);
				m->_Free = true;
			}
			return m;
		}
		void FbxSdkManager::Destroy()
		{
			CollectManagedMemory();
			_Ref()->Destroy();
			_Free = false;
			_disposed = true;
		}

		REF_PROPERTY_GET_DEFINATION_FROM_VALUE(FbxSdkManager,GetError(),FbxError,KError);

		FbxSdkManager::Error FbxSdkManager::LastErrorID::get()
		{
			return (Error)_Ref()->GetLastErrorID();
		}		
		String^ FbxSdkManager::LastErrorString::get()
		{
			return gcnew String(_Ref()->GetLastErrorString());
		}			
		FbxDataType^ FbxSdkManager::CreateFbxDataType(String^ name,FbxType type)
		{			
			STRINGTO_CONSTCHAR_ANSI(n,name);
			FbxDataType^ b = gcnew FbxDataType(&_Ref()->CreateFbxDataType(n,(EFbxType)type));
			FREECHARPOINTER(n);
			return b;
		}
		VALUE_PROPERTY_GET_DEFINATION(FbxSdkManager,GetFbxDataTypeCount(),int,FbxDataTypeCount);				
		FbxDataType^ FbxSdkManager::GetFbxDataType(int index)
		{
			return gcnew FbxDataType(&_Ref()->GetFbxDataType(index));
		}


		REF_PROPERTY_GET_DEFINATION_FROM_REF(FbxSdkManager,KFbxLibrary,GetRootLibrary(),FbxLibrary,RootLibrary);
		REF_PROPERTY_GET_DEFINATION_FROM_REF(FbxSdkManager,KFbxLibrary,GetSystemLibraries(),FbxLibrary,SystemLibraries);
		REF_PROPERTY_GET_DEFINATION_FROM_REF(FbxSdkManager,KFbxLibrary,GetUserLibraries(),FbxLibrary,UserLibraries);		

		bool FbxSdkManager::SetLocale(String^ locale)
		{			
			STRINGTO_CONSTCHAR_ANSI(l,locale);
			bool b = _Ref()->SetLocale(l);
			FREECHARPOINTER(l);
			return b;
		}
		String^ FbxSdkManager::Localize(String^ ID, String^ Default)
		{			
			STRINGTO_CONSTCHAR_ANSI(id,ID);			
			STRINGTO_CONSTCHAR_ANSI(d,Default);

			String^ result = gcnew String(_Ref()->Localize(id,d));
			FREECHARPOINTER(id);
			FREECHARPOINTER(d);
			return result;

		}
		String^ FbxSdkManager::Localize(String^ ID)
		{			
			STRINGTO_CONSTCHAR_ANSI(id,ID);	
			String^  result = gcnew String(_Ref()->Localize(id));
			FREECHARPOINTER(id);
			return result;			
		}
		REF_PROPERTY_GET_DEFINATION_FROM_VALUE(FbxSdkManager,GetXRefManager(),FbxXRefManager,XRefManager);

		REF_PROPERTY_GET_DEFINATION_FROM_REF(FbxSdkManager,KFbxIOPluginRegistry,GetIOPluginRegistry(),FbxIOPluginRegistry,IOPluginRegistry);
		bool FbxSdkManager::LoadPluginsDirectory (String^ filename,String^ extensions)
		{			
			STRINGTOCHAR_ANSI(f,filename);			
			STRINGTOCHAR_ANSI(e,extensions);
			bool b = _Ref()->LoadPluginsDirectory (f,e);
			FREECHARPOINTER(e);
			FREECHARPOINTER(f);
			return b;
		}
		bool FbxSdkManager::LoadPlugin (String^ filename)
		{			
			STRINGTOCHAR_ANSI(f,filename);
			bool b = _Ref()->LoadPlugin(f);
			FREECHARPOINTER(f);
			return b;
		}
		bool FbxSdkManager::UnloadPlugins()
		{
			return _Ref()->UnloadPlugins();
		}

		void FbxSdkManager::RegisterObject(FbxPlug^ plug)
		{
			_Ref()->RegisterObject(plug->_Ref());
		}
		void FbxSdkManager::UnregisterObject(FbxPlug^ plug)
		{
			_Ref()->UnregisterObject(plug->_Ref());
		}

		void FbxSdkManager::FillIOSettingsForReadersRegistered(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->FillIOSettingsForReadersRegistered(*ios->_Ref());
		}

		void FbxSdkManager::FillIOSettingsForWritersRegistered(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->FillIOSettingsForWritersRegistered(*ios->_Ref());
		}

		void FbxSdkManager::FillCommonIOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios, bool import)
		{
			_Ref()->FillCommonIOSettings(*ios->_Ref(),import);
		}

		void FbxSdkManager::Create_Common_Import_IOSettings_Groups(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Create_Common_Import_IOSettings_Groups(*ios->_Ref());
		}

		void FbxSdkManager::Create_Common_Export_IOSettings_Groups(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Create_Common_Export_IOSettings_Groups(*ios->_Ref());
		}

		void FbxSdkManager::Add_Common_Import_IOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Add_Common_Import_IOSettings(*ios->_Ref());
		}

		void FbxSdkManager::Add_Common_Export_IOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Add_Common_Export_IOSettings(*ios->_Ref());
		}

		void FbxSdkManager::Add_Common_RW_Import_IOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Add_Common_RW_Import_IOSettings(*ios->_Ref());
		}

		void FbxSdkManager::Add_Common_RW_Export_IOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Add_Common_RW_Export_IOSettings(*ios->_Ref());
		}

		void FbxSdkManager::Fill_Motion_Base_ReaderIOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Fill_Motion_Base_ReaderIOSettings(*ios->_Ref());
		}

		void FbxSdkManager::Fill_Motion_Base_WriterIOSettings(Skill::FbxSDK::IO::FbxIOSettings^ ios)
		{
			_Ref()->Fill_Motion_Base_WriterIOSettings(*ios->_Ref());
		}

		String^ FbxSdkManager::PrefixName(String^ prefix,String^ name)
		{
			STRINGTO_CONSTCHAR_ANSI(pr,prefix);
			STRINGTO_CONSTCHAR_ANSI(n,name);

			KString kstr = KFbxSdkManager::PrefixName(pr,n);

			FREECHARPOINTER(pr);
			FREECHARPOINTER(n);

			CONVERT_KSTRING_TO_STRING(kstr,str);
			return str;
		}

			#ifndef DOXYGEN_SHOULD_SKIP_THIS
			  
				  bool FbxSdkManager::CanDestroyFbxSrcObject(FbxObject^ obj, FbxObject^ srcObj, bool recursive, bool dependents)
				  {
					  return _Ref()->CanDestroyFbxSrcObject(obj->_Ref(), srcObj->_Ref(),recursive,dependents);
				  }

				  void FbxSdkManager::CreateMissingBindPoses(FbxScene^ scene)
				  {
					  _Ref()->CreateMissingBindPoses(scene->_Ref());
				  }
				  int  FbxSdkManager::GetBindPoseCount(FbxScene^ scene)
				  {
					  return _Ref()->GetBindPoseCount(scene->_Ref());
				  }

				  //FbxPlug^ CreateClassFrom(FbxClassId^ , const char *pName, const KFbxObject* pFrom, const char* pFBXType=0, const char* pFBXSubType=0);  

			  #endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
	}
}