﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Skill.FbxSDK;
using System.Collections;

namespace ExportScene01
{
    class Common
    {
        static void Main(string[] args)
        {
            string SAMPLE_FILENAME = "ExportScene01.fbx";

            FbxSdkManager SdkManager = null;
            FbxScene Scene = null;
            bool Result = false;

            Common com = new Common();

            // Prepare the FBX SDK.
            com.InitializeSdkObjects(out SdkManager, out Scene);

            // Create the scene.
            Result = com.CreateScene(SdkManager, Scene);

            if (Result == false)
            {
                Console.Write("\n\nAn error occured while creating the scene...\n");
                com.DestroySdkObjects(SdkManager);
                return;
            }

            // Save the scene.

            // The example can take an output file name as an argument.
            if (args.Length != 0)
            {
                Result = com.SaveScene(SdkManager, Scene, args[1], (int)Skill.FbxSDK.IO.FileFormat.FbxBinary, false);
            }
            // A default output file name is given otherwise.
            else
            {
                Result = com.SaveScene(SdkManager, Scene, SAMPLE_FILENAME, (int)Skill.FbxSDK.IO.FileFormat.FbxBinary, false);
            }

            if (Result == false)
            {
                Console.Write("\n\nAn error occured while saving the scene...\n");
                com.DestroySdkObjects(SdkManager);
                return;
            }

            // Destroy all objects created by the FBX SDK.
            com.DestroySdkObjects(SdkManager);

            return;
        }
        private bool CreateScene(FbxSdkManager SdkManager, FbxScene Scene)
        {
            // create scene info
            FbxDocumentInfo sceneInfo = FbxDocumentInfo.Create(SdkManager, "SceneInfo");
            sceneInfo.Title = "Example scene";
            sceneInfo.Subject = "Illustrates the creation and animation of a deformed cylinder.";
            sceneInfo.Author = "ExportScene01.exe sample program.";
            sceneInfo.Revision = "rev. 1.0";
            sceneInfo.Keywords = "deformed cylinder";
            sceneInfo.Comment = "no particular comments required.";

            // we need to add the sceneInfo before calling AddThumbNailToScene because
            // that function is asking the scene for the sceneInfo.
            Scene.SceneInfo = sceneInfo;

            AddThumbnailToScene(SdkManager, Scene);

            FbxNode Patch = CreatePatch(SdkManager, "Patch");
            FbxNode SkeletonRoot = CreateSkeleton(SdkManager, "Skeleton");

            LinkPatchToSkeleton(SdkManager, Patch, SkeletonRoot);
            StoreBindPose(SdkManager, Scene, Patch, SkeletonRoot);
            StoreRestPose(SdkManager, Scene, SkeletonRoot);
            AnimateSkeleton(SdkManager, Scene, SkeletonRoot);

            // Build the node tree.
            FbxNode RootNode = Scene.RootNode;
            RootNode.AddChild(Patch);
            RootNode.AddChild(SkeletonRoot);

            // Identify current take when file is loaded.
            Scene.SetCurrentTake("Bend on 2 sides");

            return true;
        }
        // Create a cylinder centered on the Z axis. 
        private FbxNode CreatePatch(FbxSdkManager SdkManager, String Name)
        {
            FbxPatch Patch = FbxPatch.Create(SdkManager, Name);

            // Set patch properties.
            Patch.InitControlPoints(4, FbxPatch.PatchType.Bspline, 7, FbxPatch.PatchType.Bspline);
            Patch.SetStep(4, 4);
            Patch.SetClosed(true, false);

            FbxVector4[] Vector4 = new FbxVector4[Patch.ControlPointsCount];
            int i;

            for (i = 0; i < 7; i++)
            {
                double Radius = 15.0;
                double SegmentLength = 20.0;

                Vector4[4 * i + 0] = new FbxVector4(Radius, 0.0, (i - 3) * SegmentLength);
                Vector4[4 * i + 1] = new FbxVector4(0.0, -Radius, (i - 3) * SegmentLength);
                Vector4[4 * i + 2] = new FbxVector4(-Radius, 0.0, (i - 3) * SegmentLength);
                Vector4[4 * i + 3] = new FbxVector4(0.0, Radius, (i - 3) * SegmentLength);
            }

            Patch.ControlPoints = Vector4;
            FbxNode Node = FbxNode.Create(SdkManager, Name);

            // Rotate the cylinder along the X axis so the axis
            // of the cylinder is the same as the bone axis (Y axis)
            FbxVector4 R = new FbxVector4(90.0, 0.0, 0.0);
            Node.SetDefaultR(R);

            Node.NodeAttribute = Patch;

            return Node;
        }
        private FbxNode CreateSkeleton(FbxSdkManager SdkManager, string Name)
        {
            // Create skeleton root. 
            String RootName = Name;
            RootName += "Root";
            FbxSkeleton SkeletonRootAttribute = FbxSkeleton.Create(SdkManager, Name);
            SkeletonRootAttribute.Skeleton_Type = FbxSkeleton.SkeletonType.Root;
            FbxNode SkeletonRoot = FbxNode.Create(SdkManager, RootName);
            SkeletonRoot.NodeAttribute = SkeletonRootAttribute;
            SkeletonRoot.SetDefaultT(new FbxVector4(0.0, -40.0, 0.0));

            // Create skeleton first limb node. 
            String LimbNodeName1 = Name;
            LimbNodeName1 += "LimbNode1";
            FbxSkeleton SkeletonLimbNodeAttribute1 = FbxSkeleton.Create(SdkManager, LimbNodeName1);
            SkeletonLimbNodeAttribute1.Skeleton_Type = FbxSkeleton.SkeletonType.LimbNode;
            SkeletonLimbNodeAttribute1.Size = 1.0;
            FbxNode SkeletonLimbNode1 = FbxNode.Create(SdkManager, LimbNodeName1);
            SkeletonLimbNode1.NodeAttribute = SkeletonLimbNodeAttribute1;
            SkeletonLimbNode1.SetDefaultT(new FbxVector4(0.0, 40.0, 0.0));

            // Create skeleton second limb node. 
            String LimbNodeName2 = Name;
            LimbNodeName2 += "LimbNode2";
            FbxSkeleton SkeletonLimbNodeAttribute2 = FbxSkeleton.Create(SdkManager, LimbNodeName2);
            SkeletonLimbNodeAttribute2.Skeleton_Type = FbxSkeleton.SkeletonType.LimbNode;
            SkeletonLimbNodeAttribute2.Size = 1.0;
            FbxNode SkeletonLimbNode2 = FbxNode.Create(SdkManager, LimbNodeName2);
            SkeletonLimbNode2.NodeAttribute = SkeletonLimbNodeAttribute2;
            SkeletonLimbNode2.SetDefaultT(new FbxVector4(0.0, 40.0, 0.0));

            // Build skeleton node hierarchy. 
            SkeletonRoot.AddChild(SkeletonLimbNode1);
            SkeletonLimbNode1.AddChild(SkeletonLimbNode2);

            return SkeletonRoot;
        }
        private void AnimateSkeleton(FbxSdkManager SdkManager, FbxScene Scene, FbxNode SkeletonRoot)
        {
            String TakeName;
            FbxCurve Curve = null;
            FbxTime Time = new FbxTime();
            int KeyIndex = 0;

            FbxNode root = SkeletonRoot;
            FbxNode LimbNode1 = SkeletonRoot.GetChild(0);

            // First take.	
            TakeName = "Bend on 2 sides";
            Scene.CreateTake(TakeName);

            root.CreateTakeNode(TakeName);
            root.SetCurrentTakeNode(TakeName);
            root.LclRotation.GetKFCurveNode(true, TakeName);

            Curve = root.LclRotation.GetKFCurve(FbxCurve.FBXCURVE_R_Z, TakeName);
            if (Curve != null)
            {
                Curve.KeyModifyBegin();

                Time.SecondDouble = 0.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 1.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 45.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 2.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, -45.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 3.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);
                Curve.KeyModifyEnd();
            }


            LimbNode1.CreateTakeNode(TakeName);
            LimbNode1.SetCurrentTakeNode(TakeName);
            LimbNode1.LclRotation.GetKFCurveNode(true, TakeName);

            Curve = LimbNode1.LclRotation.GetKFCurve(FbxCurve.FBXCURVE_R_Z, TakeName);
            if (Curve != null)
            {
                Curve.KeyModifyBegin();

                Time.SecondDouble = 0.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 1.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, -90.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 2.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 90.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 3.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Curve.KeyModifyEnd();
            }

            AddThumbnailToTake(SdkManager, Scene, TakeName, 0);

            // Second take.
            TakeName = "Bend and turn around";
            Scene.CreateTake(TakeName);

            root.CreateTakeNode(TakeName);
            root.SetCurrentTakeNode(TakeName);
            root.LclRotation.GetKFCurveNode(true, TakeName);

            Curve = root.LclRotation.GetKFCurve(FbxCurve.FBXCURVE_R_Y, TakeName);
            if (Curve != null)
            {
                Curve.KeyModifyBegin();

                Time.SecondDouble = 0.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 2.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 720.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Curve.KeyModifyEnd();
            }

            LimbNode1.CreateTakeNode(TakeName);
            LimbNode1.SetCurrentTakeNode(TakeName);
            LimbNode1.LclRotation.GetKFCurveNode(true, TakeName);

            Curve = LimbNode1.LclRotation.GetKFCurve(FbxCurve.FBXCURVE_R_Z, TakeName);
            if (Curve != null)
            {
                Curve.KeyModifyBegin();

                Time.SecondDouble = 0.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 1.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 90.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Time.SecondDouble = 2.0;
                KeyIndex = Curve.KeyAdd(Time);
                Curve.KeySetValue(KeyIndex, 0.0f);
                Curve.KeySetInterpolation(KeyIndex, FbxCurveKey.KeyInterpolation.Cubic);

                Curve.KeyModifyEnd();
            }

            AddThumbnailToTake(SdkManager, Scene, TakeName, 1);
        }
        private void LinkPatchToSkeleton(FbxSdkManager SdkManager, FbxNode Patch, FbxNode SkeletonRoot)
        {

            int i, j;
            FbxXMatrix XMatrix;

            FbxNode Root = SkeletonRoot;
            FbxNode LimbNode1 = SkeletonRoot.GetChild(0);
            FbxNode LimbNode2 = LimbNode1.GetChild(0);

            // Bottom section of cylinder is clustered to skeleton root.
            FbxCluster ClusterToRoot = FbxCluster.Create(SdkManager, "");
            ClusterToRoot.Link = Root;
            ClusterToRoot.Mode = FbxCluster.LinkMode.Total1;
            for (i = 0; i < 4; ++i)
                for (j = 0; j < 4; ++j)
                    ClusterToRoot.AddControlPointIndex(4 * i + j, 1.0 - 0.25 * i);

            // Center section of cylinder is clustered to skeleton limb node.
            FbxCluster ClusterToLimbNode1 = FbxCluster.Create(SdkManager, "");
            ClusterToLimbNode1.Link = LimbNode1;
            ClusterToLimbNode1.Mode = FbxCluster.LinkMode.Total1;

            for (i = 1; i < 6; ++i)
                for (j = 0; j < 4; ++j)
                    ClusterToLimbNode1.AddControlPointIndex(4 * i + j, (i == 1 || i == 5 ? 0.25 : 0.50));


            // Top section of cylinder is clustered to skeleton limb.

            FbxCluster ClusterToLimbNode2 = FbxCluster.Create(SdkManager, "");
            ClusterToLimbNode2.Link = LimbNode2;
            ClusterToLimbNode2.Mode = FbxCluster.LinkMode.Total1;

            for (i = 3; i < 7; ++i)
                for (j = 0; j < 4; ++j)
                    ClusterToLimbNode2.AddControlPointIndex(4 * i + j, 0.25 * (i - 2));

            // Now we have the Patch and the skeleton correctly positioned,
            // set the Transform and TransformLink matrix accordingly.
            XMatrix = Patch.GetGlobalFromDefaultTake(FbxNode.PivotSet.SourceSet);

            ClusterToRoot.TransformMatrix = XMatrix;
            ClusterToLimbNode1.TransformMatrix = XMatrix;
            ClusterToLimbNode2.TransformMatrix = XMatrix;



            XMatrix = Root.GetGlobalFromDefaultTake(FbxNode.PivotSet.SourceSet);
            ClusterToRoot.TransformLinkMatrix = XMatrix;


            XMatrix = LimbNode1.GetGlobalFromDefaultTake(FbxNode.PivotSet.SourceSet);
            ClusterToLimbNode1.TransformLinkMatrix = XMatrix;


            XMatrix = LimbNode2.GetGlobalFromDefaultTake(FbxNode.PivotSet.SourceSet);
            ClusterToLimbNode2.TransformLinkMatrix = XMatrix;


            // Add the clusters to the patch by creating a skin and adding those clusters to that skin.
            // After add that skin.

            FbxGeometry PatchAttribute = (FbxGeometry)Patch.NodeAttribute;
            FbxSkin Skin = FbxSkin.Create(SdkManager, "");
            Skin.AddCluster(ClusterToRoot);
            Skin.AddCluster(ClusterToLimbNode1);
            Skin.AddCluster(ClusterToLimbNode2);
            PatchAttribute.AddDeformer(Skin);
        }
        private void AddThumbnailToTake(FbxSdkManager SdkManager, FbxScene Scene, string TakeName, int ThumbnailIndex)
        {

            FbxTakeInfo TakeInfo = new FbxTakeInfo(); ;
            FbxThumbnail Thumbnail = FbxThumbnail.Create(SdkManager, "");

            Thumbnail.Data_Format = FbxThumbnail.DataFormat.RGB_24;
            Thumbnail.Size = FbxThumbnail.ImageSize.E64x64;
            Thumbnail.SetThumbnailImage(ThumbnailIndex == 0 ? TakeOneThumbnail : TakeTwoThumbnail);

            TakeInfo.Name = new FbxString(TakeName);
            TakeInfo.Description = new FbxString("Take at index ") + ThumbnailIndex;
            TakeInfo.TakeThumbnail = Thumbnail;


            Scene.SetTakeInfo(TakeInfo);
        }
        private void AddThumbnailToScene(FbxSdkManager SdkManager, FbxScene Scene)
        {
            FbxThumbnail thumbnail = FbxThumbnail.Create(SdkManager, "");

            thumbnail.Data_Format = FbxThumbnail.DataFormat.RGB_24;
            thumbnail.Size = FbxThumbnail.ImageSize.E64x64;
            thumbnail.SetThumbnailImage(SceneThumbnail);

            if (Scene.SceneInfo != null)
            {
                Scene.SceneInfo.SceneThumbnail = thumbnail;
            }
        }
        private void StoreRestPose(FbxSdkManager SdkManager, FbxScene Scene, FbxNode SkeletonRoot)
        {

            // This example show an arbitrary rest pose asignment.
            // This rest pose will set the bone rotation to the same value 
            // as time 1 second in the first take of animation, but the 
            // position of the bone will be set elsewhere in the scene.
            String NodeName;
            FbxNode KFbxNode;
            FbxMatrix TransformMatrix = new FbxMatrix();
            FbxVector4 lT = new FbxVector4();
            FbxVector4 lR = new FbxVector4();
            FbxVector4 lS = new FbxVector4(1.0, 1.0, 1.0);


            // Create the rest pose
            FbxPose Pose = FbxPose.Create(SdkManager, "A Bind Pose");

            // Set the skeleton root node to the global position (10, 10, 10)
            // and global rotation of 45deg along the Z axis.
            lT.Set(10.0, 10.0, 10.0);
            lR.Set(0.0, 0.0, 45.0);

            ///////////TransformMatrix.SetTRS(lT, lR, lS);

            // Add the skeleton root node to the pose
            KFbxNode = SkeletonRoot;
            Pose.Add(KFbxNode, TransformMatrix, false /*it's a global matrix*/, false);

            // Set the lLimbNode1 node to the local position of (0, 40, 0)
            // and local rotation of -90deg along the Z axis. This show that
            // you can mix local and global coordinates in a rest pose.
            lT.Set(0.0, 40.0, 0.0);
            lR.Set(0.0, 0.0, -90.0);

            TransformMatrix.SetTRS(lT, lR, lS);

            // Add the skeleton second node to the pose
            KFbxNode = KFbxNode.GetChild(0);
            Pose.Add(KFbxNode, TransformMatrix, true /*it's a local matrix*/, false);

            // Set the lLimbNode2 node to the local position of (0, 40, 0)
            // and local rotation of 45deg along the Z axis.
            lT.Set(0.0, 40.0, 0.0);
            lR.Set(0.0, 0.0, 45.0);

            TransformMatrix.SetTRS(lT, lR, lS);

            // Add the skeleton second node to the pose
            KFbxNode = KFbxNode.GetChild(0);
            NodeName = KFbxNode.Name;
            Pose.Add(KFbxNode, TransformMatrix, true /*it's a local matrix*/, false);

            // Now add the pose to the scene
            Scene.AddPose(Pose);
        }
        private void StoreBindPose(FbxSdkManager SdkManager, FbxScene Scene, FbxNode Patch, FbxNode SkeletonRoot)
        {
            // In the bind pose, we must store all the link's global matrix at the time of the bind.
            // Plus, we must store all the parent(s) global matrix of a link, even if they are not
            // themselves deforming any model.

            // In this example, since there is only one model deformed, we don't neet walk thru 
            // the scene
            //

            // Now list the all the link involve in the patch deformation	
            List<FbxNode> ClusteredFbxNodes = new List<FbxNode>();

            //KArrayTemplate<KFbxNode*> lClusteredFbxNodes;
            int i, j;

            if (Patch != null && Patch.NodeAttribute != null)
            {
                int SkinCount = 0;
                int ClusterCount = 0;
                switch (Patch.NodeAttribute.AttribType)
                {
                    case FbxNodeAttribute.AttributeType.Mesh:
                    case FbxNodeAttribute.AttributeType.Nurb:
                    case FbxNodeAttribute.AttributeType.Patch:


                        SkinCount = ((FbxGeometry)Patch.NodeAttribute).GetDeformerCount(FbxDeformer.DeformerType.Skin);
                        //Go through all the skins and count them
                        //then go through each skin and get their cluster count
                        for (i = 0; i < SkinCount; ++i)
                        {
                            // FbxGeometry g = ((FbxGeometry)Patch.NodeAttribute);
                            FbxSkin Skin = (FbxSkin)((FbxGeometry)Patch.NodeAttribute).GetDeformer(i, FbxDeformer.DeformerType.Skin);
                            // FbxSkin Skin = ((FbxGeometry)Patch.NodeAttribute).GetDeformer(i, FbxDeformer.DeformerType.Skin)  as FbxSkin;
                            ClusterCount += Skin.ClusterCount;
                        }
                        break;
                }
                //if we found some clusters we must add the node
                if (ClusterCount != 0)
                {
                    //Again, go through all the skins get each cluster link and add them
                    for (i = 0; i < SkinCount; ++i)
                    {
                        // FbxGeometry g = ((FbxGeometry)Patch.NodeAttribute);
                        FbxSkin Skin = (FbxSkin)((FbxGeometry)Patch.NodeAttribute).GetDeformer(i, FbxDeformer.DeformerType.Skin);
                        // FbxSkin Skin = g.GetDeformer(i, FbxDeformer.DeformerType.Skin) as FbxSkin;
                        ClusterCount = Skin.ClusterCount;
                        for (j = 0; j < ClusterCount; ++j)
                        {
                            FbxNode ClusterNode = Skin.GetCluster(j).Link;
                            AddNodeRecursively(ClusteredFbxNodes, ClusterNode);
                        }

                    }

                    // Add the patch to the pose
                    ClusteredFbxNodes.Add(Patch);
                }
            }

            // Now create a bind pose with the link list
            if (ClusteredFbxNodes.Count() != 0)
            {
                // A pose must be named. Arbitrarily use the name of the patch node.
                FbxPose Pose = FbxPose.Create(SdkManager, Patch.Name);

                for (i = 0; i < ClusteredFbxNodes.Count; i++)
                {
                    FbxNode KFbxNode = ClusteredFbxNodes[i];
                    FbxMatrix BindMatrix = new FbxMatrix(KFbxNode.GetGlobalFromDefaultTake(FbxNode.PivotSet.SourceSet));

                    Pose.Add(KFbxNode, BindMatrix);
                }

                // Add the pose to the scene
                Scene.AddPose(Pose);
            }
        }


        private void AddNodeRecursively(List<FbxNode> NodeArray, FbxNode Node)
        {
            if (Node != null)
            {
                AddNodeRecursively(NodeArray, Node.Parent);

                if (!NodeArray.Contains(Node, new FbxEqualityComparer<FbxNode>()))
                {
                    // Node not in the list, add it
                    NodeArray.Add(Node);
                }
            }
        }
        #region Common
        public bool InitializeSdkObjects(out FbxSdkManager sdkManager, out FbxScene scene)
        {
            // Create the FBX SDK memory manager object.
            // The SDK Manager allocates and frees memory
            // for almost all the classes in the SDK.
            sdkManager = FbxSdkManager.Create();

            if (sdkManager == null)
            {
                Console.Write("Unable to create the FBX SDK manager");
                sdkManager = null;
                scene = null;
                return false;
            }

            // Create the scene object. The scene will hold the data
            // in the file to be imported.
            scene = FbxScene.Create(sdkManager, "");
            if (scene == null)
            {
                Console.Write("Unable to create the Scene");
                sdkManager = null;
                scene = null;
                return false;
            }
            return true;
        }

        public void DestroySdkObjects(FbxSdkManager sdkManager)
        {
            // Delete the FBX SDK manager.
            // All the objects that
            // (1) have been allocated by the memory manager, AND that
            // (2) have not been explicitly destroyed
            // will be automatically destroyed.                        
            if (sdkManager != null)
                sdkManager.Destroy();

        }

        public bool LoadScene(FbxSdkManager sdkManager, FbxScene scene, string importFileName, string password)
        {
            Version fileVersion;
            Version sdkVersion;
            int fileFormat = -1;
            int i, takeCount;
            bool status = true;

            // Use memory manager to create an object
            // to store the options for the import of FBX files only.
            Skill.FbxSDK.IO.FbxStreamOptionsFbxReader importOptions =
                Skill.FbxSDK.IO.FbxStreamOptionsFbxReader.Create(sdkManager, "");

            // Get the version number of the FBX files generated by the
            // version of FBX SDK that you are using.
            sdkVersion = Skill.FbxSDK.IO.FbxIO.CurrentVersion;

            // Create an importer.
            Skill.FbxSDK.IO.FbxImporter importer = Skill.FbxSDK.IO.FbxImporter.Create(sdkManager, "");

            //Detect the file format of the file to be imported            
            if (!sdkManager.IOPluginRegistry.DetectFileFormat(importFileName, ref fileFormat))
            {
                // Unrecognizable file format.
                // Try to fall back to SDK's native file format (an FBX binary file).
                fileFormat =
                    sdkManager.IOPluginRegistry.NativeReaderFormat;
            }
            importer.FileFormat = fileFormat;

            // Initialize the importer by providing a filename.
            bool importStatus = importer.Initialize(importFileName);

            // Get the version number of the FBX file format.
            fileVersion = importer.FileVersion;

            if (!importStatus)  // Problem with the file to be imported
            {
                Console.Write("Call to FbxImporter.Initialize() failed.");
                Console.Write(string.Format("Error returned: {0}", importer.LastErrorString));

                if (importer.LastErrorID ==
                        Skill.FbxSDK.IO.FbxIO.Error.FileVersionNotSupportedYet ||
                    importer.LastErrorID ==
                        Skill.FbxSDK.IO.FbxIO.Error.FileVersionNotSupportedAnymore)
                {
                    Console.Write(string.Format("FBX version number for this FBX SDK is {0}.{1}.{2}",
                               sdkVersion.Major, sdkVersion.Minor, sdkVersion.Revision));
                    Console.Write(string.Format("FBX version number for file {0} is {1}.{2}.{3}",
                               importFileName, fileVersion.Major, fileVersion.Minor, fileVersion.Revision));
                }
                return false;
            }

            Console.Write(string.Format("FBX version number for this FBX SDK is {0}.{1}.{2}",
                               sdkVersion.Major, sdkVersion.Minor, sdkVersion.Revision));

            if (importer.IsFBX)
            {
                Console.Write(string.Format("FBX version number for file {0} is {1}.{2}.{3}",
                               importFileName, fileVersion.Major, fileVersion.Minor, fileVersion.Revision));

                // In FBX, a scene can have one or more "takes". A take is a
                // container for animation data.
                // You can access a file's take information without
                // the overhead of loading the entire file into the scene.

                Console.Write("Take Information");

                takeCount = importer.TakeCount;

                Console.Write(string.Format("    Number of takes: {0}", takeCount));
                Console.Write(string.Format("    Current take: \"{0}\"",
                               importer.CurrentTakeName));

                for (i = 0; i < takeCount; i++)
                {
                    FbxTakeInfo takeInfo = importer.GetTakeInfo(i);

                    Console.Write(string.Format("    Take {0}", i));
                    Console.Write(string.Format("         Name: \"{0}\"", takeInfo.Name.ToString()));
                    Console.Write(string.Format("         Description: \"{0}\"",
                                        takeInfo.Description.ToString()));

                    // Change the value of the import name if the take should
                    // be imported under a different name.
                    Console.Write(string.Format("         Import Name: \"{0}\"", takeInfo.ImportName.ToString()));

                    // Set the value of the import state to false
                    // if the take should be not be imported.
                    Console.Write(string.Format("         Import State: {0}", takeInfo.Select ? "true" : "false"));
                }

                // Import options determine what kind of data is to be imported.
                // The default is true, but here we set the options explictly.                

                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.MATERIAL, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.TEXTURE, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.LINK, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.SHAPE, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.GOBO, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.ANIMATION, true);
                importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.GLOBAL_SETTINGS, true);
            }

            // Import the scene.
            status = importer.Import(scene, importOptions);

            if (status == false &&     // The import file may have a password
               importer.LastErrorID == Skill.FbxSDK.IO.FbxIO.Error.PasswordError)
            {
                Console.Write("Please enter password: ");

                if (password != null)
                {
                    importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.PASSWORD, password);
                    importOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.PASSWORD_ENABLE, true);
                    status = importer.Import(scene, importOptions);

                    if (status == false && importer.LastErrorID == Skill.FbxSDK.IO.FbxIO.Error.PasswordError)
                    {
                        Console.Write("Incorrect password: file not imported.");
                    }
                }
            }

            // Destroy the importer.
            if (importOptions != null)
            {
                importOptions.Destroy();
                importOptions = null;
            }

            // Destroy the importer
            importer.Destroy();
            return status;
        }


        public bool SaveScene(FbxSdkManager sdkManager, FbxScene scene, string exportFileName, int writeFileFormat, bool embedMedia)
        {
            bool status = true;

            // Create an exporter.
            Skill.FbxSDK.IO.FbxExporter exporter = Skill.FbxSDK.IO.FbxExporter.Create(sdkManager, "");

            // Initialize the exporter by providing a filename.
            if (exporter.Initialize(exportFileName) == false)
            {
                Console.Write("Call to FbxExporter.Initialize() failed.");
                Console.Write(string.Format("Error returned: {0}", exporter.LastErrorString));
                return false;
            }

            Version version = Skill.FbxSDK.IO.FbxIO.CurrentVersion;
            Console.Write(string.Format("FBX version number for this FBX SDK is {0}.{1}.{2}",
                      version.Major, version.Minor, version.Revision));


            if (writeFileFormat < 0 ||
                writeFileFormat >=
                    sdkManager.IOPluginRegistry.WriterFormatCount)
            {
                // Write in fall back format if pEmbedMedia is true
                writeFileFormat =
                    sdkManager.IOPluginRegistry.NativeWriterFormat;

                if (!embedMedia)
                {
                    //Try to export in ASCII if possible
                    int formatIndex, formatCount =
                        sdkManager.IOPluginRegistry.
                            WriterFormatCount;

                    for (formatIndex = 0; formatIndex < formatCount; formatIndex++)
                    {
                        if (sdkManager.IOPluginRegistry.
                            WriterIsFBX(formatIndex))
                        {
                            string desc = sdkManager.IOPluginRegistry.
                                GetWriterFormatDescription(formatIndex);
                            if (desc.Contains("ascii"))
                            {
                                writeFileFormat = formatIndex;
                                break;
                            }
                        }
                    }
                }
            }

            // Set the file format
            exporter.FileFormat = writeFileFormat;

            Skill.FbxSDK.IO.FbxStreamOptionsFbxWriter exportOptions =
                Skill.FbxSDK.IO.FbxStreamOptionsFbxWriter.Create(sdkManager, "");

            if (sdkManager.IOPluginRegistry.WriterIsFBX(writeFileFormat))
            {
                // Export options determine what kind of data is to be imported.
                // The default (except for the option eEXPORT_TEXTURE_AS_EMBEDDED)
                // is true, but here we set the options explictly.
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.MATERIAL, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.TEXTURE, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.EMBEDDED, embedMedia);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.LINK, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.SHAPE, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.GOBO, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.ANIMATION, true);
                exportOptions.SetOption(Skill.FbxSDK.IO.FbxStreamOptionsFbx.GLOBAL_SETTINGS, true);
            }

            // Export the scene.
            status = exporter.Export(scene, exportOptions);

            if (exportOptions != null)
            {
                exportOptions.Destroy();
                exportOptions = null;
            }

            // Destroy the exporter.
            exporter.Destroy();

            return status;
        }

        #endregion



        #region Tumbnail
        readonly byte[] SceneThumbnail = {
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe8, 0xe7, 0xe7, 0xc3, 
	0xc2, 0xc2, 0xdb, 0xda, 0xda, 0xf3, 0xf3, 0xf3, 0xf9, 0xf9, 0xf9, 0xf7, 0xf7, 0xf7, 0xfe, 0xfe, 
	0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xa6, 0xa6, 0xa6, 0x37, 0x35, 0x36, 0x1c, 
	0x18, 0x19, 0x25, 0x22, 0x23, 0xb1, 0xb0, 0xb1, 0xf4, 0xf4, 0xf4, 0xeb, 0xeb, 0xeb, 0xf1, 0xf1, 
	0xf1, 0xfd, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 
	0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xfc, 0xfc, 0xfc, 0xef, 0xef, 0xef, 0x7c, 0x79, 0x7a, 0x2d, 0x29, 0x2a, 0x1c, 0x18, 0x19, 0x1f, 
	0x1d, 0x1f, 0x1c, 0x1a, 0x1c, 0xa8, 0xa9, 0xab, 0xf1, 0xf2, 0xf4, 0xeb, 0xec, 0xed, 0xf0, 0xf0, 
	0xf0, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xf3, 0xf3, 0xf3, 0xd7, 0xd7, 0xd7, 0xcd, 0xcd, 0xcd, 0xe6, 0xe6, 0xe6, 0xf6, 0xf6, 0xf6, 
	0xf9, 0xf9, 0xf9, 0xf9, 0xf9, 0xf9, 0xfd, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xfa, 0xfa, 
	0xbc, 0xbc, 0xbc, 0x43, 0x43, 0x43, 0x24, 0x21, 0x21, 0x1f, 0x1b, 0x1c, 0x20, 0x1d, 0x1e, 0x41, 
	0x3f, 0x40, 0xa8, 0xa5, 0xa7, 0xde, 0xdf, 0xe1, 0xee, 0xef, 0xf0, 0xf1, 0xf1, 0xf2, 0xfc, 0xfc, 
	0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfd, 0xfd, 0x98, 0x97, 
	0x98, 0x39, 0x36, 0x37, 0x1d, 0x18, 0x19, 0x22, 0x1e, 0x1f, 0x1e, 0x1b, 0x1c, 0x5c, 0x5a, 0x5a, 
	0xbc, 0xbb, 0xbc, 0xf0, 0xf2, 0xf4, 0xea, 0xeb, 0xec, 0xf5, 0xf5, 0xf5, 0xfd, 0xfc, 0xfc, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xed, 0xed, 0xed, 0x8f, 0x8d, 0x8d, 
	0x22, 0x1e, 0x1f, 0x24, 0x1f, 0x20, 0x1e, 0x18, 0x1a, 0x2f, 0x2d, 0x2d, 0x81, 0x82, 0x83, 0xe1, 
	0xe2, 0xe4, 0xe7, 0xe8, 0xe9, 0xef, 0xef, 0xef, 0xf9, 0xf9, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xfa, 0xfa, 0x85, 0x84, 0x85, 0x31, 0x2d, 
	0x2e, 0x1c, 0x17, 0x18, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x21, 0x1f, 0x20, 0x13, 0x11, 0x11, 
	0x50, 0x4f, 0x50, 0xbb, 0xbd, 0xbf, 0xe9, 0xe9, 0xeb, 0xeb, 0xeb, 0xeb, 0xf5, 0xf5, 0xf5, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xd7, 0xd7, 0x61, 0x61, 0x61, 0x18, 0x15, 0x16, 
	0x22, 0x1e, 0x1f, 0x20, 0x1c, 0x1d, 0x55, 0x53, 0x53, 0xb7, 0xb8, 0xb7, 0xf2, 0xf4, 0xf5, 0xe9, 
	0xea, 0xec, 0xf4, 0xf5, 0xf6, 0xfc, 0xfc, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xfb, 0xcf, 0xcf, 0xcf, 
	0xbc, 0xbb, 0xbb, 0xde, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xf3, 0xf3, 0xf3, 0xf9, 0xf9, 0xf9, 0xfe, 
	0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xe1, 0xe0, 0xe0, 0x46, 0x43, 0x43, 0x19, 0x14, 
	0x15, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x1f, 0x1f, 0x1b, 0x19, 
	0x2c, 0x29, 0x29, 0x71, 0x71, 0x72, 0xea, 0xeb, 0xeb, 0xeb, 0xed, 0xec, 0xee, 0xf0, 0xef, 0xf9, 
	0xfa, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xf4, 0xf4, 0xf4, 0xb6, 0xb6, 0xb6, 0x45, 0x42, 0x43, 0x1b, 0x16, 0x18, 0x1d, 0x19, 0x1a, 
	0x2f, 0x2d, 0x2d, 0x83, 0x83, 0x83, 0xd8, 0xd9, 0xd9, 0xf0, 0xf2, 0xf1, 0xee, 0xef, 0xee, 0xfa, 
	0xfa, 0xfa, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xda, 0xd9, 0xd9, 0x41, 0x3d, 0x3e, 
	0x09, 0x05, 0x06, 0x74, 0x73, 0x73, 0xf2, 0xf2, 0xf2, 0xe9, 0xe9, 0xe9, 0xec, 0xec, 0xec, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xd5, 0xd4, 0xd4, 0x40, 0x3d, 0x3d, 0x18, 0x13, 
	0x15, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x1f, 0x20, 0x1c, 0x1b, 
	0x25, 0x22, 0x22, 0x64, 0x64, 0x65, 0xe9, 0xeb, 0xea, 0xeb, 0xed, 0xec, 0xed, 0xee, 0xed, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xec, 0xec, 
	0xec, 0x98, 0x98, 0x98, 0x28, 0x27, 0x27, 0x1f, 0x1c, 0x1d, 0x19, 0x14, 0x15, 0x49, 0x46, 0x47, 
	0xb2, 0xb2, 0xb2, 0xe4, 0xe6, 0xe6, 0xea, 0xed, 0xec, 0xf2, 0xf3, 0xf3, 0xfc, 0xfd, 0xfd, 0xfe, 
	0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xf5, 0xf5, 0xb3, 0xb2, 0xb2, 0x36, 0x32, 0x33, 
	0x08, 0x04, 0x05, 0x65, 0x63, 0x65, 0xdc, 0xdd, 0xdf, 0xea, 0xea, 0xec, 0xeb, 0xeb, 0xeb, 0xf8, 
	0xf8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xef, 0xef, 0x55, 0x55, 0x55, 0x1c, 0x1a, 
	0x1b, 0x21, 0x1d, 0x1e, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x1c, 0x17, 0x18, 
	0x39, 0x36, 0x36, 0x8a, 0x8a, 0x89, 0xe9, 0xeb, 0xea, 0xea, 0xec, 0xeb, 0xf0, 0xf2, 0xf1, 0xfc, 
	0xfc, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe2, 0xe1, 0x79, 0x77, 
	0x78, 0x1b, 0x16, 0x17, 0x23, 0x1f, 0x20, 0x18, 0x16, 0x16, 0x69, 0x69, 0x69, 0xcb, 0xcc, 0xcc, 
	0xee, 0xf0, 0xef, 0xe9, 0xec, 0xea, 0xf7, 0xf7, 0xf7, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xf5, 0xf5, 0xb3, 0xb2, 0xb2, 0x36, 0x32, 0x33, 
	0x08, 0x03, 0x04, 0x64, 0x62, 0x63, 0xdb, 0xdb, 0xdd, 0xea, 0xea, 0xec, 0xeb, 0xeb, 0xeb, 0xf8, 
	0xf8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xfd, 0xc2, 0xc2, 0xc3, 0x4e, 0x4b, 
	0x4c, 0x15, 0x11, 0x12, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x21, 0x1d, 0x1e, 0x1e, 0x19, 0x1b, 
	0x74, 0x71, 0x72, 0xe4, 0xe4, 0xe3, 0xe8, 0xea, 0xe9, 0xeb, 0xed, 0xec, 0xf7, 0xf8, 0xf8, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xd7, 0xd7, 0x61, 0x61, 0x60, 0x14, 0x12, 
	0x12, 0x21, 0x1d, 0x1e, 0x24, 0x21, 0x22, 0x87, 0x86, 0x86, 0xd8, 0xda, 0xda, 0xee, 0xf2, 0xf0, 
	0xec, 0xee, 0xed, 0xf9, 0xfa, 0xfa, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 0xf9, 0xf9, 0xbf, 0xbe, 0xbe, 0x37, 0x33, 0x34, 
	0x07, 0x03, 0x05, 0x5e, 0x5c, 0x5d, 0xd2, 0xd3, 0xd2, 0xeb, 0xeb, 0xeb, 0xe9, 0xeb, 0xea, 0xf6, 
	0xf8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xdf, 
	0xdf, 0x96, 0x95, 0x96, 0x56, 0x55, 0x56, 0x52, 0x50, 0x51, 0x6a, 0x69, 0x69, 0xb2, 0xb2, 0xb2, 
	0xe8, 0xe8, 0xe8, 0xef, 0xef, 0xef, 0xf4, 0xf4, 0xf4, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xf7, 0xf7, 0xf7, 0xcc, 0xcc, 0xcc, 0x4a, 0x47, 0x48, 0x1a, 0x16, 0x17, 0x1b, 0x17, 
	0x18, 0x38, 0x36, 0x37, 0xa6, 0xa6, 0xa6, 0xdf, 0xe1, 0xe0, 0xed, 0xef, 0xee, 0xef, 0xf1, 0xf0, 
	0xfd, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xd2, 0xd1, 0xd2, 0x3a, 0x37, 0x38, 
	0x09, 0x05, 0x06, 0x57, 0x54, 0x55, 0xc4, 0xc4, 0xc4, 0xea, 0xeb, 0xeb, 0xe8, 0xea, 0xe9, 0xf5, 
	0xf6, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xfd, 0xfe, 
	0xfe, 0xfd, 0xfe, 0xfe, 0xfd, 0xfe, 0xfe, 0xfd, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfd, 0xfd, 0xfd, 
	0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf9, 
	0xf9, 0xf9, 0xb2, 0xb2, 0xb2, 0x32, 0x32, 0x32, 0x20, 0x1d, 0x1e, 0x17, 0x13, 0x14, 0x53, 0x51, 
	0x51, 0xbd, 0xbe, 0xbe, 0xe7, 0xea, 0xe9, 0xea, 0xec, 0xeb, 0xf3, 0xf4, 0xf4, 0xfd, 0xfe, 0xfe, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe9, 0xe8, 0xe8, 0x46, 0x43, 0x44, 
	0x0e, 0x0a, 0x0b, 0x49, 0x47, 0x48, 0xae, 0xad, 0xb0, 0xeb, 0xeb, 0xed, 0xe8, 0xea, 0xe9, 0xf3, 
	0xf5, 0xf4, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xa1, 
	0xa0, 0xa0, 0x27, 0x23, 0x24, 0x23, 0x1f, 0x20, 0x17, 0x13, 0x14, 0x65, 0x63, 0x64, 0xc9, 0xca, 
	0xca, 0xef, 0xf1, 0xf0, 0xeb, 0xec, 0xeb, 0xf7, 0xf7, 0xf7, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf3, 0xf3, 0xf3, 0x63, 0x60, 0x60, 
	0x1c, 0x18, 0x19, 0x35, 0x33, 0x34, 0x92, 0x92, 0x94, 0xec, 0xed, 0xef, 0xe9, 0xeb, 0xea, 0xf1, 
	0xf2, 0xf2, 0xfc, 0xfc, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x88, 0x88, 0x88, 0x26, 
	0x24, 0x24, 0x1f, 0x1a, 0x1b, 0x21, 0x1d, 0x1e, 0x78, 0x78, 0x78, 0xce, 0xd1, 0xd0, 0xf0, 0xf3, 
	0xf2, 0xeb, 0xed, 0xec, 0xf8, 0xfb, 0xfa, 0xfe, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x8d, 0x8d, 0x8d, 
	0x31, 0x2e, 0x2f, 0x1d, 0x19, 0x1a, 0x71, 0x6e, 0x6f, 0xee, 0xee, 0xef, 0xea, 0xeb, 0xed, 0xed, 
	0xee, 0xf0, 0xf9, 0xf9, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf4, 0xf4, 0xf4, 0x7b, 0x78, 0x79, 0x2d, 0x29, 0x2a, 0x17, 
	0x13, 0x14, 0x32, 0x2e, 0x2f, 0x84, 0x83, 0x83, 0xd9, 0xdb, 0xda, 0xee, 0xf1, 0xf0, 0xee, 0xef, 
	0xee, 0xfb, 0xfb, 0xfb, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfb, 0xfb, 0xc4, 0xc3, 0xc3, 
	0x4a, 0x47, 0x48, 0x08, 0x05, 0x06, 0x51, 0x4f, 0x4f, 0xdd, 0xdd, 0xdd, 0xe8, 0xe9, 0xeb, 0xeb, 
	0xeb, 0xee, 0xf6, 0xf6, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xe4, 0xe4, 0xe4, 0x6f, 0x6f, 0x6f, 0x2e, 0x2b, 0x2b, 0x14, 0x10, 0x11, 0x39, 
	0x37, 0x38, 0x91, 0x91, 0x92, 0xe5, 0xe6, 0xe6, 0xea, 0xec, 0xeb, 0xef, 0xf1, 0xf0, 0xfa, 0xfb, 
	0xfb, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xfd, 0xfc, 0xfc, 
	0x67, 0x65, 0x66, 0x07, 0x04, 0x05, 0x3d, 0x3a, 0x3b, 0xaa, 0xab, 0xab, 0xe1, 0xe2, 0xe4, 0xeb, 
	0xec, 0xee, 0xf1, 0xf2, 0xf3, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 
	0xd9, 0xd8, 0xd9, 0x73, 0x70, 0x71, 0x24, 0x1f, 0x20, 0x1a, 0x14, 0x16, 0x3d, 0x3a, 0x3c, 0x9b, 
	0x9b, 0x9c, 0xf2, 0xf3, 0xf5, 0xe8, 0xe9, 0xeb, 0xf2, 0xf2, 0xf2, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0x9d, 0x9a, 0x9b, 0x33, 0x2e, 0x2f, 0x24, 0x21, 0x22, 0x64, 0x64, 0x64, 0xd8, 0xd8, 0xda, 0xf1, 
	0xf2, 0xf3, 0xee, 0xee, 0xef, 0xf9, 0xf9, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdc, 0xdb, 0xdc, 
	0x6f, 0x6d, 0x6e, 0x16, 0x13, 0x14, 0x24, 0x1f, 0x20, 0x3b, 0x39, 0x39, 0xab, 0xac, 0xab, 0xf3, 
	0xf5, 0xf5, 0xea, 0xeb, 0xed, 0xf2, 0xf2, 0xf4, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfe, 0xfe, 
	0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xe1, 0xe1, 0xe1, 0x71, 0x6f, 0x70, 0x0e, 0x09, 0x0a, 0x2d, 0x29, 0x2a, 0xba, 0xb9, 0xb9, 0xeb, 
	0xec, 0xec, 0xec, 0xec, 0xec, 0xf2, 0xf2, 0xf2, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xe4, 0xe5, 0xe5, 0x67, 0x67, 0x68, 
	0x14, 0x10, 0x12, 0x25, 0x20, 0x22, 0x41, 0x3f, 0x40, 0xbc, 0xbc, 0xbe, 0xed, 0xef, 0xf0, 0xed, 
	0xed, 0xee, 0xf4, 0xf4, 0xf4, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xb0, 0xae, 0xaf, 0x2b, 0x26, 0x27, 0x24, 0x20, 0x21, 0x63, 0x62, 0x62, 0xc6, 
	0xc6, 0xc6, 0xf4, 0xf4, 0xf4, 0xed, 0xed, 0xed, 0xf8, 0xf8, 0xf8, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xfd, 0xe6, 0xe5, 0xe6, 0x58, 0x56, 0x57, 0x1e, 0x1a, 0x1b, 
	0x1b, 0x17, 0x19, 0x4c, 0x4b, 0x4b, 0xcc, 0xce, 0xcd, 0xe6, 0xe8, 0xe9, 0xec, 0xec, 0xef, 0xf4, 
	0xf4, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xeb, 0xea, 0xea, 0x8b, 0x89, 0x89, 0x1f, 0x1b, 0x1c, 0x17, 0x15, 0x16, 0x8c, 
	0x8d, 0x8c, 0xe7, 0xe9, 0xe8, 0xeb, 0xed, 0xec, 0xec, 0xee, 0xed, 0xfa, 0xfb, 0xfa, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xd7, 0xd7, 0xd7, 0x56, 0x53, 0x54, 0x27, 0x24, 0x25, 0x13, 0x0f, 0x10, 
	0x5d, 0x5c, 0x5d, 0xcc, 0xce, 0xd0, 0xe9, 0xea, 0xec, 0xeb, 0xeb, 0xec, 0xf6, 0xf6, 0xf6, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd8, 0xd6, 0xd7, 0x55, 0x51, 0x52, 0x24, 0x1f, 0x21, 0x2c, 
	0x29, 0x2a, 0x94, 0x95, 0x95, 0xf4, 0xf7, 0xf5, 0xe8, 0xea, 0xe9, 0xf0, 0xf1, 0xf0, 0xff, 0xff, 
	0xff, 0xd3, 0xd3, 0xd3, 0x60, 0x5f, 0x5f, 0x22, 0x1f, 0x20, 0x15, 0x13, 0x14, 0x64, 0x64, 0x64, 
	0xc8, 0xc9, 0xc9, 0xf0, 0xf1, 0xf3, 0xea, 0xea, 0xec, 0xf7, 0xf6, 0xf7, 0xfd, 0xfd, 0xfd, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfb, 0xfc, 0xd4, 0xd3, 0xd4, 0x42, 0x40, 0x41, 0x0f, 
	0x0d, 0x0e, 0x3c, 0x3a, 0x3b, 0x96, 0x94, 0x95, 0xeb, 0xed, 0xec, 0xf0, 0xf3, 0xf2, 0xd0, 0xd1, 
	0xd0, 0x6f, 0x6d, 0x6d, 0x14, 0x0f, 0x10, 0x22, 0x1f, 0x20, 0x62, 0x62, 0x64, 0xc4, 0xc4, 0xc6, 
	0xf4, 0xf4, 0xf5, 0xec, 0xec, 0xec, 0xf8, 0xf8, 0xf8, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xb9, 0xb7, 0xb8, 0x49, 
	0x47, 0x48, 0x0d, 0x0a, 0x0b, 0x32, 0x2e, 0x2f, 0x82, 0x80, 0x81, 0xcc, 0xcc, 0xcc, 0x6a, 0x68, 
	0x69, 0x14, 0x10, 0x11, 0x29, 0x24, 0x25, 0x59, 0x55, 0x56, 0xd0, 0xcf, 0xd1, 0xf1, 0xf1, 0xf3, 
	0xed, 0xed, 0xef, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xaf, 
	0xb0, 0xb0, 0x44, 0x43, 0x43, 0x1b, 0x16, 0x17, 0x24, 0x20, 0x21, 0x2c, 0x28, 0x29, 0x24, 0x20, 
	0x21, 0x20, 0x1c, 0x1e, 0x5c, 0x5b, 0x5b, 0xd9, 0xdb, 0xda, 0xe8, 0xea, 0xeb, 0xec, 0xed, 0xef, 
	0xf6, 0xf7, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 
	0xff, 0xff, 0xbb, 0xba, 0xba, 0x48, 0x44, 0x45, 0x22, 0x1e, 0x1f, 0x22, 0x1e, 0x1f, 0x17, 0x12, 
	0x13, 0x45, 0x42, 0x44, 0xaa, 0xa9, 0xaa, 0xe9, 0xea, 0xea, 0xe9, 0xea, 0xeb, 0xf2, 0xf2, 0xf4, 
	0xfd, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 
	0xff, 0xff, 0xe4, 0xe4, 0xe5, 0x86, 0x85, 0x86, 0x1f, 0x1c, 0x1d, 0x1e, 0x1c, 0x1d, 0x21, 0x1f, 
	0x20, 0x1c, 0x1a, 0x1b, 0x2e, 0x2a, 0x2b, 0x6b, 0x67, 0x68, 0xc2, 0xc2, 0xc3, 0xe8, 0xea, 0xe9, 
	0xef, 0xf1, 0xf0, 0xf4, 0xf5, 0xf6, 0xfc, 0xfd, 0xfd, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 
	0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0x93, 0x93, 0x93, 0x1b, 0x19, 0x1a, 0x23, 0x20, 0x21, 0x3f, 0x3d, 0x3e, 0x42, 0x40, 
	0x41, 0x30, 0x2c, 0x2d, 0x21, 0x1c, 0x1d, 0x23, 0x1f, 0x20, 0x1f, 0x1e, 0x1e, 0x4b, 0x4a, 0x4b, 
	0x96, 0x96, 0x97, 0xd0, 0xd1, 0xd3, 0xea, 0xeb, 0xed, 0xf2, 0xf2, 0xf2, 0xf8, 0xf8, 0xf8, 0xfd, 
	0xfd, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xa0, 
	0x9d, 0x9d, 0x36, 0x32, 0x33, 0x1a, 0x16, 0x17, 0x31, 0x2f, 0x30, 0xb1, 0xb2, 0xb4, 0xed, 0xee, 
	0xf1, 0xcb, 0xcb, 0xcd, 0x79, 0x79, 0x78, 0x36, 0x35, 0x35, 0x23, 0x20, 0x21, 0x21, 0x1d, 0x1e, 
	0x20, 0x1c, 0x1d, 0x26, 0x22, 0x23, 0x41, 0x3e, 0x3f, 0x6b, 0x6a, 0x6b, 0x9b, 0x9a, 0x9a, 0xc4, 
	0xc3, 0xc3, 0xe4, 0xe4, 0xe4, 0xf3, 0xf3, 0xf3, 0xf8, 0xf8, 0xf8, 0xfa, 0xfb, 0xfb, 0xfc, 0xfd, 
	0xfd, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 
	0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xfd, 0xbf, 0xbe, 0xbe, 0x4d, 
	0x4a, 0x4b, 0x0a, 0x07, 0x08, 0x37, 0x37, 0x36, 0xa2, 0xa3, 0xa3, 0xdf, 0xe0, 0xe2, 0xed, 0xee, 
	0xf0, 0xf1, 0xf2, 0xf3, 0xfe, 0xfe, 0xfe, 0xf7, 0xf7, 0xf7, 0xc4, 0xc4, 0xc4, 0x81, 0x80, 0x80, 
	0x42, 0x40, 0x40, 0x23, 0x1e, 0x20, 0x22, 0x1e, 0x1f, 0x22, 0x1f, 0x20, 0x21, 0x1e, 0x1f, 0x20, 
	0x1d, 0x1e, 0x24, 0x21, 0x22, 0x33, 0x30, 0x31, 0x48, 0x47, 0x47, 0x5f, 0x5e, 0x5e, 0x73, 0x72, 
	0x72, 0x81, 0x81, 0x81, 0x8e, 0x8e, 0x8e, 0x8d, 0x8d, 0x8d, 0x8c, 0x8c, 0x8c, 0xb8, 0xb8, 0xb8, 
	0xf8, 0xf8, 0xf8, 0xf1, 0xf1, 0xf1, 0xf6, 0xf6, 0xf6, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xe1, 0xe1, 0xe1, 0x4d, 0x49, 0x4a, 0x17, 
	0x12, 0x13, 0x30, 0x2d, 0x2d, 0x7f, 0x7e, 0x7e, 0xea, 0xec, 0xeb, 0xeb, 0xec, 0xec, 0xf0, 0xf0, 
	0xf0, 0xfa, 0xfa, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xfe, 0xfe, 0xfe, 0xe8, 0xe8, 0xe8, 0xba, 0xba, 0xba, 0x91, 0x90, 0x91, 0x67, 0x66, 0x67, 0x44, 
	0x41, 0x42, 0x2b, 0x27, 0x28, 0x21, 0x1e, 0x1f, 0x20, 0x1d, 0x1e, 0x22, 0x1e, 0x1f, 0x23, 0x20, 
	0x21, 0x23, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x22, 0x1f, 0x20, 0x10, 0x0d, 0x0e, 0x5b, 0x5b, 0x5a, 
	0xe4, 0xe7, 0xe6, 0xe7, 0xe9, 0xe8, 0xeb, 0xeb, 0xeb, 0xf6, 0xf6, 0xf6, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0x65, 0x65, 0x65, 0x29, 0x26, 0x26, 0x15, 
	0x11, 0x12, 0x77, 0x74, 0x75, 0xe8, 0xe8, 0xe9, 0xea, 0xeb, 0xea, 0xec, 0xec, 0xec, 0xf8, 0xf8, 
	0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xf9, 0xf8, 0xf8, 0xe9, 0xe8, 0xe9, 0xd6, 0xd5, 0xd6, 0xc0, 0xbf, 0xc0, 0xae, 0xad, 
	0xae, 0xa2, 0xa1, 0xa2, 0x9c, 0x9b, 0x9c, 0x97, 0x97, 0x97, 0x9e, 0x9e, 0x9e, 0xc7, 0xc8, 0xc7, 
	0xf6, 0xf9, 0xf7, 0xf1, 0xf3, 0xf2, 0xf7, 0xf7, 0xf7, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 
	0xff, 0xff, 0xff, 0xed, 0xee, 0xee, 0x91, 0x91, 0x91, 0x20, 0x1b, 0x1d, 0x17, 0x12, 0x14, 0x65, 
	0x63, 0x64, 0xc6, 0xc7, 0xc8, 0xf0, 0xf1, 0xf3, 0xe9, 0xea, 0xeb, 0xf8, 0xf8, 0xf8, 0xfe, 0xfe, 
	0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xa4, 0xa4, 0xa5, 0x23, 0x21, 0x22, 0x23, 0x20, 0x21, 0x3e, 0x3b, 0x3c, 0xb5, 
	0xb3, 0xb4, 0xf7, 0xf7, 0xf8, 0xea, 0xeb, 0xed, 0xf4, 0xf5, 0xf5, 0xfd, 0xfd, 0xfe, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xba, 0xb9, 0xb9, 0x51, 0x4d, 0x4e, 0x14, 0x0f, 0x10, 0x2b, 0x27, 0x28, 0xaa, 0xab, 0xab, 0xe5, 
	0xe7, 0xe6, 0xed, 0xed, 0xed, 0xf0, 0xf0, 0xf0, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xfd, 0xeb, 0xeb, 0xeb, 
	0x60, 0x5d, 0x5e, 0x0e, 0x09, 0x0a, 0x31, 0x2d, 0x2e, 0x7a, 0x76, 0x77, 0xda, 0xdb, 0xda, 0xef, 
	0xf1, 0xf0, 0xef, 0xf0, 0xf0, 0xfb, 0xfa, 0xfb, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xf3, 0xf3, 0xf3, 0x77, 0x73, 0x74, 
	0x2c, 0x27, 0x28, 0x14, 0x11, 0x12, 0x55, 0x55, 0x55, 0xdb, 0xdd, 0xdc, 0xe8, 0xea, 0xe9, 0xeb, 
	0xed, 0xec, 0xf6, 0xf7, 0xf6, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf0, 0xf0, 0xa4, 0xa3, 0xa3, 0x33, 0x30, 0x31, 
	0x0f, 0x0a, 0x0c, 0x49, 0x46, 0x46, 0xac, 0xac, 0xac, 0xeb, 0xed, 0xec, 0xe9, 0xeb, 0xea, 0xf3, 
	0xf5, 0xf4, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xfa, 0xf9, 0xf9, 0xaa, 0xa8, 0xa9, 0x27, 0x22, 0x23, 0x1f, 0x1c, 0x1d, 
	0x1d, 0x1c, 0x1d, 0x82, 0x83, 0x82, 0xf3, 0xf5, 0xf4, 0xe9, 0xea, 0xe9, 0xef, 0xef, 0xef, 0xfa, 
	0xfa, 0xfa, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xb0, 0xb0, 0xb1, 0x33, 0x32, 0x33, 0x20, 0x1d, 0x1e, 0x18, 0x14, 0x15, 
	0x6b, 0x69, 0x69, 0xc8, 0xc9, 0xc8, 0xf0, 0xf2, 0xf1, 0xea, 0xec, 0xeb, 0xf8, 0xf9, 0xf9, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xca, 0xc9, 0xc9, 0x63, 0x61, 0x62, 0x3d, 0x3c, 0x3c, 0x6a, 0x6a, 0x6a, 
	0xca, 0xca, 0xca, 0xf5, 0xf5, 0xf5, 0xed, 0xed, 0xed, 0xf6, 0xf6, 0xf6, 0xfe, 0xfe, 0xfe, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xfe, 0xfd, 0xfe, 0xfd, 0xfd, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 
	0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7b, 0x7c, 0x7c, 0x7b, 0x7c, 0x7c, 
	0x7b, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7b, 0x7c, 0x7d, 0x7b, 0x7c, 0x78, 0x77, 0x78, 0x89, 0x89, 0x89, 0xad, 0xad, 0xad, 
	0x24, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 
	0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 
	0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 
	0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x20, 
	0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 
	0x23, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x20, 0x21, 0x22, 0x1f, 0x20, 0x22, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 
	0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 
	0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x20, 0x21, 0x23, 0x1f, 
	0x20, 0x21, 0x1d, 0x1e, 0x3d, 0x3b, 0x3c, 0x66, 0x63, 0x64, 0x68, 0x65, 0x66, 0x46, 0x43, 0x44, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x21, 0x1d, 0x1e, 0x25, 0x22, 0x23, 0x46, 0x44, 
	0x45, 0x97, 0x95, 0x96, 0x7e, 0x7c, 0x7d, 0x3b, 0x38, 0x39, 0x22, 0x1f, 0x20, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 
	0x1f, 0x20, 0x22, 0x1e, 0x1f, 0x22, 0x1e, 0x1f, 0x64, 0x60, 0x61, 0xb2, 0xaf, 0xb0, 0xaf, 0xad, 
	0xae, 0x43, 0x41, 0x42, 0x21, 0x1f, 0x20, 0x1d, 0x1b, 0x1c, 0x2b, 0x28, 0x29, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 
	0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x1f, 0x1b, 0x1c, 0x2e, 
	0x2a, 0x2b, 0x72, 0x6f, 0x70, 0xd8, 0xd6, 0xd7, 0xc4, 0xc3, 0xc4, 0x5e, 0x5c, 0x5c, 0x1e, 0x1a, 
	0x1b, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 
	0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x1c, 0x18, 0x19, 0x3a, 0x36, 0x38, 0x91, 0x8f, 0x90, 0xe3, 
	0xe3, 0xe3, 0xe1, 0xe0, 0xe0, 0x76, 0x74, 0x75, 0x2a, 0x26, 0x27, 0x1f, 0x1b, 0x1c, 0x24, 0x20, 
	0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1e, 0x1f, 0x25, 0x22, 0x22, 0x2d, 0x2c, 0x2c, 0x33, 
	0x30, 0x30, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 
	0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 
	0x35, 0x32, 0x33, 0x2b, 0x28, 0x29, 0x22, 0x20, 0x21, 0x2c, 0x2a, 0x2b, 0x33, 0x2f, 0x30, 0x35, 
	0x30, 0x31, 0x34, 0x30, 0x32, 0x34, 0x30, 0x32, 0x34, 0x30, 0x31, 0x34, 0x30, 0x31, 0x34, 0x31, 
	0x32, 0x31, 0x2e, 0x2f, 0x2f, 0x2b, 0x2c, 0x2e, 0x2a, 0x2b, 0x27, 0x23, 0x24, 0x22, 0x1e, 0x1f, 
	0x21, 0x1e, 0x1f, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x2e, 0x2a, 0x2b, 0x35, 
	0x32, 0x33, 0x33, 0x30, 0x31, 0x34, 0x30, 0x31, 0x2e, 0x2b, 0x2c, 0x26, 0x23, 0x24, 0x22, 0x1e, 
	0x1f, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x22, 0x1e, 0x1f, 0x1f, 0x1c, 0x1d, 
	0x26, 0x23, 0x24, 0x46, 0x43, 0x44, 0xa4, 0xa3, 0xa3, 0xf4, 0xf5, 0xf5, 0xed, 0xee, 0xee, 0x93, 
	0x93, 0x93, 0x30, 0x2f, 0x2f, 0x20, 0x1c, 0x1d, 0x22, 0x1e, 0x1f, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x16, 0x12, 0x13, 0x45, 0x42, 0x43, 0xc2, 0xc1, 0xc2, 0xf9, 
	0xf8, 0xf9, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 
	0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xff, 0xff, 0x96, 0x95, 0x95, 0x30, 0x2e, 0x2f, 0xa5, 0xa3, 0xa4, 0xf4, 0xf3, 0xf4, 0xff, 
	0xff, 0xff, 0xfe, 0xfe, 0xff, 0xfe, 0xfe, 0xfe, 0xff, 0xfe, 0xfe, 0xff, 0xfe, 0xff, 0xff, 0xfe, 
	0xff, 0xff, 0xfe, 0xfe, 0xfe, 0xfd, 0xfd, 0xfd, 0xfd, 0xfd, 0xfb, 0xfa, 0xfb, 0xed, 0xec, 0xed, 
	0xc1, 0xc0, 0xc2, 0x7a, 0x79, 0x7a, 0x2c, 0x2b, 0x2b, 0x1b, 0x18, 0x18, 0x4b, 0x47, 0x48, 0xac, 
	0xaa, 0xab, 0xfe, 0xfe, 0xfe, 0xfe, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xb9, 0xb8, 0xb8, 0x51, 0x4f, 
	0x4f, 0x1d, 0x19, 0x1a, 0x24, 0x20, 0x21, 0x21, 0x1c, 0x1d, 0x26, 0x22, 0x23, 0x5f, 0x5d, 0x5d, 
	0xc0, 0xc1, 0xc0, 0xf9, 0xf9, 0xf9, 0xff, 0xff, 0xff, 0xab, 0xac, 0xac, 0x43, 0x41, 0x42, 0x1e, 
	0x1a, 0x1b, 0x22, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x15, 0x11, 0x12, 0x45, 0x42, 0x43, 0xc2, 0xc2, 0xc2, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xd7, 0xd6, 0xd6, 0x74, 0x72, 0x73, 0x39, 0x35, 0x36, 0x3b, 0x39, 
	0x39, 0x3c, 0x3a, 0x3a, 0x3c, 0x3a, 0x3a, 0x3c, 0x3a, 0x3a, 0x3c, 0x3a, 0x3b, 0x3b, 0x39, 0x3a, 
	0x3d, 0x3c, 0x3d, 0x28, 0x25, 0x26, 0x3b, 0x37, 0x38, 0xa3, 0xa3, 0xa3, 0xf4, 0xf4, 0xf4, 0xff, 
	0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x8c, 0x8b, 0x8b, 0x3a, 0x37, 0x38, 0x39, 0x37, 0x38, 0x3b, 0x39, 
	0x3a, 0x3a, 0x38, 0x39, 0x3c, 0x3a, 0x3b, 0x4c, 0x4a, 0x4b, 0xa0, 0xa0, 0xa0, 0xea, 0xea, 0xea, 
	0xff, 0xff, 0xff, 0xf2, 0xf3, 0xf3, 0xa6, 0xa6, 0xa6, 0x3a, 0x37, 0x38, 0x17, 0x12, 0x13, 0x23, 
	0x20, 0x21, 0x52, 0x52, 0x52, 0xba, 0xb9, 0xba, 0xf5, 0xf4, 0xf5, 0xff, 0xff, 0xff, 0xf1, 0xf1, 
	0xf1, 0xb0, 0xb0, 0xb0, 0x4d, 0x4d, 0x4d, 0x74, 0x74, 0x74, 0xd0, 0xd0, 0xd0, 0xff, 0xff, 0xff, 
	0xf9, 0xf8, 0xf8, 0xce, 0xcd, 0xce, 0x5c, 0x58, 0x59, 0x23, 0x1e, 0x1f, 0x20, 0x1c, 0x1d, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x15, 0x11, 0x12, 0x45, 0x42, 0x43, 0xc2, 0xc2, 0xc2, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xde, 0xdf, 0xdf, 0x91, 0x90, 0x91, 0x64, 0x61, 0x62, 0x65, 0x63, 
	0x63, 0x65, 0x63, 0x64, 0x65, 0x63, 0x64, 0x65, 0x63, 0x64, 0x65, 0x63, 0x64, 0x67, 0x65, 0x66, 
	0x39, 0x37, 0x38, 0x13, 0x10, 0x11, 0x3b, 0x37, 0x38, 0xa4, 0xa3, 0xa4, 0xf4, 0xf4, 0xf4, 0xff, 
	0xff, 0xff, 0xea, 0xe9, 0xea, 0xa7, 0xa6, 0xa7, 0x69, 0x66, 0x67, 0x69, 0x67, 0x68, 0x6b, 0x69, 
	0x6a, 0x6a, 0x68, 0x69, 0x6e, 0x6c, 0x6d, 0x80, 0x7f, 0x80, 0xc7, 0xc6, 0xc7, 0xf8, 0xf8, 0xf8, 
	0xff, 0xff, 0xff, 0xc7, 0xc7, 0xc7, 0x54, 0x54, 0x54, 0x29, 0x25, 0x26, 0x1f, 0x1b, 0x1c, 0x23, 
	0x1f, 0x20, 0x21, 0x1d, 0x1f, 0x22, 0x1f, 0x20, 0x5b, 0x5a, 0x5a, 0xc2, 0xc2, 0xc2, 0xff, 0xff, 
	0xff, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xda, 0xd8, 0xd8, 
	0x74, 0x71, 0x72, 0x2b, 0x27, 0x28, 0x20, 0x1c, 0x1d, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x15, 0x12, 0x13, 0x44, 0x42, 0x43, 0xc2, 0xc2, 0xc2, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xfa, 0xfa, 0xfa, 0xee, 0xee, 0xee, 0xe9, 0xe9, 0xe9, 0xea, 0xea, 
	0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xea, 0xf1, 0xf1, 0xf1, 
	0x68, 0x67, 0x68, 0x0b, 0x09, 0x0a, 0x3a, 0x37, 0x38, 0xa4, 0xa3, 0xa3, 0xf4, 0xf4, 0xf4, 0xff, 
	0xff, 0xff, 0xfa, 0xfa, 0xfa, 0xec, 0xec, 0xec, 0xe1, 0xe1, 0xe1, 0xe3, 0xe2, 0xe2, 0xe4, 0xe3, 
	0xe3, 0xe2, 0xe2, 0xe2, 0xe3, 0xe3, 0xe3, 0xec, 0xeb, 0xeb, 0xf9, 0xfa, 0xfa, 0xff, 0xff, 0xff, 
	0xe0, 0xe1, 0xe0, 0xa1, 0xa1, 0xa1, 0x63, 0x62, 0x62, 0x2d, 0x2a, 0x2a, 0x1f, 0x1b, 0x1c, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x7d, 0x7a, 0x7b, 0xe4, 0xe2, 
	0xe3, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xfd, 0xff, 0xff, 0xff, 0xf6, 0xf6, 0xf6, 0xa3, 0xa2, 0xa2, 
	0x33, 0x30, 0x31, 0x24, 0x20, 0x21, 0x22, 0x1e, 0x1f, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x22, 0x1e, 0x1f, 0x2c, 0x28, 0x29, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x15, 0x12, 0x13, 0x44, 0x42, 0x43, 0xc2, 0xc2, 0xc2, 0xf9, 
	0xf9, 0xf9, 0xff, 0xff, 0xff, 0xd2, 0xd1, 0xd1, 0x63, 0x60, 0x61, 0x1d, 0x1a, 0x1b, 0x21, 0x1d, 
	0x1e, 0x21, 0x1e, 0x1f, 0x21, 0x1e, 0x1f, 0x21, 0x1e, 0x1f, 0x21, 0x1e, 0x1f, 0x21, 0x1e, 0x1f, 
	0x22, 0x1f, 0x20, 0x18, 0x15, 0x16, 0x3b, 0x38, 0x39, 0xa4, 0xa3, 0xa3, 0xf4, 0xf4, 0xf4, 0xff, 
	0xff, 0xff, 0xe0, 0xe0, 0xe0, 0x7c, 0x7b, 0x7b, 0x1c, 0x1b, 0x1b, 0x1d, 0x1a, 0x1b, 0x21, 0x1d, 
	0x1e, 0x22, 0x1e, 0x1f, 0x21, 0x1e, 0x1f, 0x22, 0x1f, 0x20, 0x2a, 0x27, 0x28, 0xa5, 0xa4, 0xa5, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfd, 0xfd, 0x90, 0x8e, 0x8f, 0x36, 0x32, 0x33, 0x16, 
	0x12, 0x13, 0x2c, 0x28, 0x29, 0x68, 0x64, 0x65, 0xd1, 0xd0, 0xd0, 0xff, 0xff, 0xff, 0xff, 0xff, 
	0xff, 0xec, 0xec, 0xec, 0xab, 0xab, 0xab, 0xdb, 0xdb, 0xdb, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 
	0xe7, 0xe6, 0xe7, 0x8e, 0x8e, 0x8e, 0x38, 0x34, 0x35, 0x1d, 0x18, 0x1a, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x22, 0x1e, 0x1f, 0x2c, 0x28, 0x29, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x15, 0x10, 0x12, 0x45, 0x42, 0x42, 0xc2, 0xc2, 0xc2, 0xf8, 
	0xf8, 0xf8, 0xff, 0xff, 0xff, 0xd2, 0xd2, 0xd2, 0x62, 0x62, 0x62, 0x1e, 0x1b, 0x1c, 0x22, 0x1e, 
	0x1f, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x19, 0x15, 0x16, 0x3b, 0x38, 0x39, 0xa3, 0xa2, 0xa3, 0xf3, 0xf3, 0xf3, 0xff, 
	0xff, 0xff, 0xeb, 0xeb, 0xeb, 0xaf, 0xaf, 0xaf, 0x77, 0x77, 0x77, 0x78, 0x77, 0x77, 0x7a, 0x78, 
	0x79, 0x7a, 0x78, 0x79, 0x7b, 0x79, 0x7a, 0x82, 0x81, 0x82, 0xa1, 0xa1, 0xa1, 0xe0, 0xe0, 0xe0, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xed, 0xec, 0xed, 0x6a, 0x69, 0x6a, 0x24, 0x22, 0x23, 0x5d, 
	0x5c, 0x5c, 0xcd, 0xcd, 0xcd, 0xfb, 0xfb, 0xfb, 0xff, 0xff, 0xff, 0xf5, 0xf5, 0xf5, 0xa6, 0xa3, 
	0xa4, 0x3f, 0x3b, 0x3c, 0x23, 0x1f, 0x20, 0x25, 0x23, 0x25, 0x81, 0x80, 0x81, 0xe2, 0xe1, 0xe1, 
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe6, 0xe6, 0xe6, 0x83, 0x83, 0x82, 0x2b, 0x29, 0x29, 0x21, 
	0x1c, 0x1d, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x18, 0x15, 0x16, 0x3e, 0x3c, 0x3c, 0x9f, 0x9f, 0x9f, 0xc8, 
	0xc8, 0xc8, 0xda, 0xda, 0xda, 0xaa, 0xab, 0xaa, 0x54, 0x53, 0x54, 0x1f, 0x1c, 0x1d, 0x22, 0x1e, 
	0x1f, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x1b, 0x17, 0x18, 0x35, 0x33, 0x34, 0x87, 0x87, 0x87, 0xc5, 0xc5, 0xc5, 0xd2, 
	0xd2, 0xd2, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcc, 0xcd, 0xcd, 0xcc, 0xcd, 
	0xcd, 0xcc, 0xce, 0xcd, 0xcc, 0xce, 0xcd, 0xca, 0xcb, 0xcb, 0xc4, 0xc4, 0xc4, 0xba, 0xb9, 0xba, 
	0xa0, 0x9e, 0x9f, 0x70, 0x6d, 0x6e, 0x34, 0x31, 0x32, 0x37, 0x35, 0x36, 0x8e, 0x8d, 0x8e, 0xce, 
	0xce, 0xce, 0xcf, 0xcf, 0xcf, 0xce, 0xce, 0xce, 0xa6, 0xa7, 0xa7, 0x50, 0x4f, 0x4f, 0x1a, 0x18, 
	0x18, 0x23, 0x1e, 0x1f, 0x23, 0x1f, 0x20, 0x22, 0x1f, 0x20, 0x19, 0x17, 0x17, 0x39, 0x38, 0x37, 
	0x89, 0x89, 0x89, 0xc7, 0xc7, 0xc7, 0xd0, 0xd0, 0xd0, 0xd3, 0xd3, 0xd3, 0xa6, 0xa5, 0xa5, 0x4f, 
	0x4e, 0x4e, 0x24, 0x23, 0x23, 0x20, 0x1d, 0x1d, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x22, 0x1e, 0x1f, 0x21, 
	0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x22, 0x1e, 0x1f, 0x22, 0x1e, 0x1f, 0x22, 
	0x1e, 0x1f, 0x22, 0x1e, 0x1f, 0x22, 0x1e, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 
	0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x22, 0x1e, 0x1f, 0x21, 
	0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x22, 0x1e, 0x1f, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x21, 0x1d, 0x1e, 0x22, 
	0x1e, 0x1f, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 
	0x1f, 0x24, 0x20, 0x1f, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 
	0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 
	0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 
	0x1f, 0x24, 0x20, 0x1f, 0x23, 0x1f, 0x20, 0x20, 0x1c, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x24, 0x20, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 
	0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 
	0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x20, 0x24, 
	0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 
	0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x23, 0x1f, 0x20, 0x22, 0x1e, 0x20, 0x22, 0x1e, 0x20, 
	0x22, 0x1e, 0x20, 0x22, 0x1e, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 
	0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x21, 
	0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x23, 0x1f, 0x20, 0x24, 
	0x20, 0x21, 0x24, 0x20, 0x21, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 
	0x20, 0x24, 0x20, 0x20, 0x24, 0x20, 0x20, 0x21, 0x1d, 0x1d, 0x2b, 0x27, 0x28, 0x41, 0x3d, 0x3e, 
	0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x26, 
	0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 
	0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 
	0x27, 0x23, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 
	0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 
	0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 
	0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 
	0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 
	0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x24, 0x26, 0x22, 0x24, 0x26, 0x22, 0x23, 
	0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x26, 0x22, 0x23, 0x27, 
	0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 
	0x24, 0x27, 0x23, 0x24, 0x27, 0x23, 0x24, 0x24, 0x20, 0x21, 0x2f, 0x2b, 0x2c, 0x44, 0x40, 0x41
};



        // Thumbnail for the first take.
        // 64x64 RGB pixels
        readonly byte[] TakeOneThumbnail = {
	0x3d, 0x3d, 0x43, 0x2f, 0x2f, 0x35, 0x2f, 0x2f, 0x35, 0x34, 0x34, 0x3a, 0x44, 0x44, 0x4e, 0x31, 
	0x31, 0x37, 0x2f, 0x2f, 0x35, 0x2f, 0x2f, 0x35, 0x32, 0x32, 0x38, 0x37, 0x37, 0x3e, 0x30, 0x30, 
	0x37, 0x30, 0x30, 0x36, 0x2d, 0x2d, 0x33, 0x37, 0x37, 0x3f, 0x41, 0x41, 0x4a, 0x30, 0x30, 0x37, 
	0x30, 0x30, 0x36, 0x2f, 0x2f, 0x36, 0x32, 0x32, 0x38, 0x37, 0x37, 0x3e, 0x30, 0x30, 0x37, 0x30, 
	0x30, 0x37, 0x2f, 0x2f, 0x36, 0x37, 0x37, 0x3e, 0x32, 0x32, 0x38, 0x30, 0x30, 0x37, 0x30, 0x30, 
	0x37, 0x2f, 0x2f, 0x35, 0x37, 0x37, 0x3e, 0x32, 0x32, 0x38, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 
	0x30, 0x30, 0x36, 0x3c, 0x3c, 0x45, 0x34, 0x34, 0x3b, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x30, 
	0x30, 0x36, 0x3b, 0x3b, 0x44, 0x34, 0x34, 0x3a, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x32, 0x32, 
	0x38, 0x36, 0x36, 0x3d, 0x30, 0x30, 0x36, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x37, 0x37, 0x3e, 
	0x32, 0x32, 0x38, 0x2f, 0x2f, 0x36, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x37, 0x37, 0x3e, 0x32, 
	0x32, 0x38, 0x2f, 0x2f, 0x35, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x3d, 0x3d, 0x46, 0x34, 0x34, 
	0x3a, 0x2f, 0x2f, 0x36, 0x30, 0x30, 0x37, 0x30, 0x30, 0x37, 0x41, 0x41, 0x4a, 0x43, 0x43, 0x4b, 
	0x36, 0x36, 0x3e, 0x3b, 0x3b, 0x46, 0x37, 0x37, 0x42, 0x2c, 0x2c, 0x35, 0x29, 0x29, 0x31, 0x2b, 
	0x2b, 0x33, 0x37, 0x37, 0x41, 0x3b, 0x3b, 0x46, 0x2c, 0x2c, 0x35, 0x2b, 0x2b, 0x34, 0x2b, 0x2b, 
	0x33, 0x30, 0x30, 0x3a, 0x43, 0x43, 0x51, 0x2e, 0x2e, 0x37, 0x2b, 0x2b, 0x33, 0x2c, 0x2c, 0x35, 
	0x2d, 0x2d, 0x36, 0x33, 0x33, 0x3d, 0x2c, 0x2c, 0x35, 0x2c, 0x2c, 0x34, 0x2d, 0x2d, 0x35, 0x2c, 
	0x2c, 0x34, 0x33, 0x33, 0x3c, 0x2d, 0x2d, 0x36, 0x2c, 0x2c, 0x35, 0x2d, 0x2d, 0x35, 0x2c, 0x2c, 
	0x35, 0x39, 0x39, 0x43, 0x2f, 0x2f, 0x38, 0x2c, 0x2c, 0x35, 0x2d, 0x2d, 0x35, 0x2d, 0x2d, 0x35, 
	0x2e, 0x2e, 0x37, 0x30, 0x30, 0x39, 0x2c, 0x2c, 0x35, 0x2d, 0x2d, 0x35, 0x2d, 0x2d, 0x35, 0x2d, 
	0x2d, 0x35, 0x35, 0x35, 0x40, 0x2f, 0x2f, 0x38, 0x2d, 0x2d, 0x35, 0x2d, 0x2d, 0x35, 0x2c, 0x2c, 
	0x35, 0x37, 0x37, 0x41, 0x2f, 0x2f, 0x39, 0x2d, 0x2d, 0x35, 0x2d, 0x2d, 0x35, 0x2c, 0x2c, 0x34, 
	0x2e, 0x2e, 0x37, 0x32, 0x32, 0x3c, 0x2d, 0x2d, 0x35, 0x2d, 0x2d, 0x35, 0x2c, 0x2c, 0x34, 0x30, 
	0x30, 0x39, 0x39, 0x39, 0x43, 0x2c, 0x2c, 0x35, 0x2d, 0x2d, 0x35, 0x2b, 0x2b, 0x33, 0x2e, 0x2e, 
	0x37, 0x33, 0x33, 0x3d, 0x2c, 0x2c, 0x34, 0x2c, 0x2c, 0x35, 0x2b, 0x2b, 0x33, 0x36, 0x36, 0x3f, 
	0x28, 0x28, 0x2e, 0x1f, 0x1f, 0x26, 0x1e, 0x1e, 0x24, 0x1c, 0x1c, 0x21, 0x20, 0x20, 0x27, 0x2e, 
	0x2e, 0x37, 0x1f, 0x1f, 0x25, 0x1f, 0x1f, 0x26, 0x1b, 0x1b, 0x20, 0x1d, 0x1d, 0x23, 0x2c, 0x2c, 
	0x35, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x20, 0x20, 0x20, 0x26, 
	0x29, 0x29, 0x31, 0x1d, 0x1d, 0x23, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1c, 0x1c, 0x21, 0x23, 
	0x23, 0x2a, 0x1f, 0x1f, 0x25, 0x1c, 0x1c, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1d, 0x1d, 
	0x23, 0x26, 0x26, 0x2d, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 
	0x1f, 0x1f, 0x25, 0x24, 0x24, 0x2b, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 
	0x1b, 0x21, 0x28, 0x28, 0x31, 0x1f, 0x1f, 0x25, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 
	0x21, 0x2c, 0x2c, 0x34, 0x21, 0x21, 0x28, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 
	0x1d, 0x1d, 0x23, 0x25, 0x25, 0x2d, 0x1c, 0x1c, 0x21, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1c, 
	0x1c, 0x22, 0x25, 0x25, 0x2c, 0x1e, 0x1e, 0x24, 0x1b, 0x1b, 0x21, 0x1b, 0x1b, 0x21, 0x1c, 0x1c, 
	0x21, 0x1d, 0x1d, 0x23, 0x22, 0x22, 0x29, 0x1f, 0x1f, 0x25, 0x1b, 0x1b, 0x21, 0x28, 0x28, 0x2e, 
	0x31, 0x31, 0x37, 0x22, 0x22, 0x29, 0x23, 0x23, 0x2a, 0x28, 0x28, 0x2f, 0x2a, 0x2a, 0x32, 0x29, 
	0x29, 0x31, 0x24, 0x24, 0x2a, 0x23, 0x23, 0x29, 0x24, 0x24, 0x2b, 0x2f, 0x2f, 0x39, 0x2d, 0x2d, 
	0x36, 0x24, 0x24, 0x2b, 0x23, 0x23, 0x29, 0x23, 0x23, 0x2a, 0x25, 0x25, 0x2c, 0x28, 0x28, 0x30, 
	0x28, 0x28, 0x2f, 0x23, 0x23, 0x2a, 0x24, 0x24, 0x2a, 0x23, 0x23, 0x2a, 0x27, 0x27, 0x2e, 0x2c, 
	0x2c, 0x34, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 0x23, 0x23, 0x2a, 0x29, 0x29, 
	0x31, 0x29, 0x29, 0x31, 0x23, 0x23, 0x2a, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 0x23, 0x23, 0x2a, 
	0x2d, 0x2d, 0x36, 0x28, 0x28, 0x30, 0x23, 0x23, 0x2a, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 0x23, 
	0x23, 0x2a, 0x2f, 0x2f, 0x37, 0x27, 0x27, 0x2e, 0x23, 0x23, 0x2a, 0x24, 0x24, 0x2a, 0x23, 0x23, 
	0x2a, 0x28, 0x28, 0x30, 0x2c, 0x2c, 0x35, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 0x24, 0x24, 0x2a, 
	0x24, 0x24, 0x2a, 0x2c, 0x2c, 0x34, 0x27, 0x27, 0x2e, 0x23, 0x23, 0x2a, 0x24, 0x24, 0x2a, 0x24, 
	0x24, 0x2a, 0x25, 0x25, 0x2c, 0x29, 0x29, 0x31, 0x24, 0x24, 0x2b, 0x23, 0x23, 0x2a, 0x24, 0x24, 
	0x2a, 0x23, 0x23, 0x2a, 0x27, 0x27, 0x2f, 0x2c, 0x2c, 0x35, 0x23, 0x23, 0x2a, 0x30, 0x30, 0x37, 
	0x28, 0x28, 0x2d, 0x22, 0x22, 0x28, 0x22, 0x22, 0x28, 0x21, 0x21, 0x27, 0x1a, 0x1a, 0x1e, 0x1a, 
	0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1f, 0x1f, 0x25, 0x20, 0x20, 0x26, 0x21, 0x21, 0x28, 0x19, 0x19, 
	0x1e, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x22, 0x28, 0x28, 0x2f, 0x1c, 0x1c, 0x22, 
	0x1a, 0x1a, 0x1f, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x22, 0x28, 0x28, 0x2f, 0x1c, 
	0x1c, 0x21, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x2a, 0x2a, 
	0x32, 0x1e, 0x1e, 0x24, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x22, 
	0x2d, 0x2d, 0x34, 0x1e, 0x1e, 0x24, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1a, 
	0x1a, 0x1f, 0x29, 0x29, 0x30, 0x1f, 0x1f, 0x24, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 
	0x1e, 0x19, 0x19, 0x1e, 0x23, 0x23, 0x2a, 0x1c, 0x1c, 0x22, 0x1a, 0x1a, 0x1f, 0x1b, 0x1b, 0x20, 
	0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x21, 0x27, 0x27, 0x2e, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 0x20, 0x1b, 
	0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x20, 0x20, 0x27, 0x23, 0x23, 0x2a, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 
	0x20, 0x1b, 0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x20, 0x20, 0x26, 0x23, 0x23, 0x2a, 0x2a, 0x2a, 0x2f, 
	0x41, 0x41, 0x4b, 0x35, 0x35, 0x3f, 0x2e, 0x2e, 0x37, 0x2d, 0x2d, 0x36, 0x2d, 0x2d, 0x36, 0x30, 
	0x30, 0x3a, 0x3a, 0x3a, 0x45, 0x39, 0x39, 0x45, 0x2d, 0x2d, 0x36, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 
	0x37, 0x2c, 0x2c, 0x35, 0x31, 0x31, 0x3b, 0x35, 0x35, 0x40, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 0x36, 
	0x2e, 0x2e, 0x37, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 0x38, 0x33, 0x33, 0x3d, 0x2e, 0x2e, 0x37, 0x2e, 
	0x2e, 0x36, 0x2e, 0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x36, 0x36, 0x3f, 0x2e, 0x2e, 
	0x37, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x2d, 0x2d, 0x36, 0x31, 0x31, 0x3a, 
	0x34, 0x34, 0x3f, 0x2c, 0x2c, 0x35, 0x2e, 0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x2c, 
	0x2c, 0x35, 0x3e, 0x3e, 0x4b, 0x3f, 0x3f, 0x4b, 0x37, 0x37, 0x42, 0x40, 0x40, 0x4c, 0x47, 0x47, 
	0x54, 0x3a, 0x3a, 0x44, 0x33, 0x33, 0x3d, 0x42, 0x42, 0x4f, 0x31, 0x31, 0x3a, 0x2d, 0x2d, 0x36, 
	0x2e, 0x2e, 0x37, 0x2d, 0x2d, 0x36, 0x30, 0x30, 0x39, 0x32, 0x32, 0x3b, 0x2d, 0x2d, 0x36, 0x2e, 
	0x2e, 0x37, 0x2e, 0x2e, 0x37, 0x2c, 0x2c, 0x35, 0x2f, 0x2f, 0x39, 0x35, 0x35, 0x3f, 0x2e, 0x2e, 
	0x37, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 0x37, 0x2d, 0x2d, 0x36, 0x2e, 0x2e, 0x38, 0x3e, 0x3e, 0x47, 
	0x1e, 0x1e, 0x21, 0x0f, 0x0f, 0x12, 0x0e, 0x0e, 0x10, 0x0c, 0x0c, 0x0f, 0x14, 0x14, 0x18, 0x21, 
	0x21, 0x28, 0x1f, 0x1f, 0x25, 0x0f, 0x0f, 0x12, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x10, 0x15, 0x15, 0x19, 0x16, 0x16, 0x1a, 0x11, 0x11, 0x15, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x0d, 0x0d, 0x10, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x10, 0x10, 0x13, 0x1a, 0x1a, 0x1f, 0x0d, 0x0d, 
	0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x19, 0x19, 0x1d, 
	0x11, 0x11, 0x15, 0x0b, 0x0b, 0x0e, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 
	0x0d, 0x10, 0x1f, 0x1f, 0x25, 0x16, 0x16, 0x1a, 0x10, 0x10, 0x13, 0x12, 0x12, 0x16, 0x14, 0x14, 
	0x18, 0x10, 0x10, 0x13, 0x18, 0x18, 0x1c, 0x1e, 0x1e, 0x23, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 
	0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x0f, 0x0f, 0x12, 0x1e, 0x1e, 0x24, 0x0f, 0x0f, 0x12, 0x0e, 
	0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x0d, 0x0d, 0x10, 0x1b, 0x1b, 0x20, 0x18, 0x18, 
	0x1c, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x1d, 0x1d, 0x20, 
	0x29, 0x29, 0x2e, 0x1c, 0x1c, 0x21, 0x1a, 0x1a, 0x20, 0x22, 0x22, 0x29, 0x2e, 0x2e, 0x37, 0x1f, 
	0x1f, 0x25, 0x19, 0x19, 0x1e, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 0x20, 0x24, 0x24, 
	0x2b, 0x25, 0x25, 0x2c, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 
	0x1b, 0x1b, 0x20, 0x24, 0x24, 0x2b, 0x24, 0x24, 0x2b, 0x1a, 0x1a, 0x20, 0x1c, 0x1c, 0x22, 0x1c, 
	0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 0x20, 0x23, 0x23, 0x2a, 0x23, 0x23, 0x2a, 0x1b, 0x1b, 
	0x21, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x1b, 0x1b, 0x20, 0x23, 0x23, 0x2a, 
	0x1d, 0x1d, 0x22, 0x21, 0x21, 0x25, 0x1a, 0x1a, 0x1e, 0x19, 0x19, 0x1f, 0x1b, 0x1b, 0x21, 0x1b, 
	0x1b, 0x20, 0x28, 0x28, 0x31, 0x1f, 0x1f, 0x26, 0x1b, 0x1b, 0x21, 0x1c, 0x1c, 0x21, 0x1b, 0x1b, 
	0x21, 0x1b, 0x1b, 0x21, 0x1f, 0x1f, 0x25, 0x2a, 0x2a, 0x33, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x22, 
	0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x1a, 0x1a, 0x20, 0x2a, 0x2a, 0x32, 0x26, 0x26, 0x2d, 0x1b, 
	0x1b, 0x21, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x19, 0x19, 0x1e, 0x29, 0x29, 
	0x31, 0x22, 0x22, 0x29, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x22, 0x29, 0x29, 0x2f, 
	0x2f, 0x2f, 0x35, 0x22, 0x22, 0x28, 0x33, 0x33, 0x3d, 0x29, 0x29, 0x32, 0x21, 0x21, 0x27, 0x21, 
	0x21, 0x28, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x21, 0x21, 0x27, 0x26, 0x26, 0x2e, 0x2a, 0x2a, 
	0x31, 0x21, 0x21, 0x28, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x28, 
	0x25, 0x25, 0x2c, 0x2a, 0x2a, 0x31, 0x21, 0x21, 0x28, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 
	0x22, 0x29, 0x22, 0x22, 0x29, 0x26, 0x26, 0x2d, 0x2f, 0x2f, 0x39, 0x21, 0x21, 0x28, 0x22, 0x22, 
	0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x26, 0x26, 0x2d, 0x30, 0x30, 0x39, 
	0x60, 0x60, 0x62, 0x62, 0x62, 0x62, 0x45, 0x45, 0x45, 0x38, 0x38, 0x3d, 0x21, 0x21, 0x27, 0x1c, 
	0x1c, 0x23, 0x2a, 0x2a, 0x33, 0x24, 0x24, 0x2b, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x21, 0x21, 
	0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x28, 0x28, 0x2f, 0x23, 0x23, 0x2a, 0x22, 0x22, 0x28, 
	0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x21, 0x21, 0x27, 0x2a, 0x2a, 0x32, 0x23, 
	0x23, 0x2a, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x22, 0x22, 0x29, 0x21, 0x21, 
	0x27, 0x2f, 0x2f, 0x38, 0x27, 0x27, 0x2f, 0x22, 0x22, 0x28, 0x22, 0x22, 0x29, 0x2f, 0x2f, 0x36, 
	0x24, 0x24, 0x27, 0x1c, 0x1c, 0x22, 0x17, 0x17, 0x1c, 0x11, 0x11, 0x14, 0x10, 0x10, 0x14, 0x11, 
	0x11, 0x14, 0x10, 0x10, 0x14, 0x10, 0x10, 0x13, 0x1e, 0x1e, 0x24, 0x1c, 0x1c, 0x22, 0x13, 0x13, 
	0x17, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x13, 0x13, 0x17, 
	0x22, 0x22, 0x28, 0x13, 0x13, 0x17, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 
	0x10, 0x14, 0x10, 0x10, 0x13, 0x1c, 0x1c, 0x21, 0x15, 0x15, 0x19, 0x10, 0x10, 0x14, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x15, 0x15, 0x1a, 0x2b, 0x2b, 0x2f, 
	0x89, 0x89, 0x88, 0xad, 0xad, 0xad, 0x9d, 0x9d, 0x9c, 0x66, 0x66, 0x65, 0x57, 0x57, 0x57, 0x3e, 
	0x3e, 0x3e, 0x22, 0x22, 0x28, 0x15, 0x15, 0x1a, 0x10, 0x10, 0x13, 0x10, 0x10, 0x14, 0x16, 0x16, 
	0x1b, 0x18, 0x18, 0x1d, 0x11, 0x11, 0x15, 0x1d, 0x1d, 0x22, 0x1a, 0x1a, 0x1f, 0x14, 0x14, 0x18, 
	0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x14, 0x18, 0x18, 0x1c, 0x1e, 
	0x1e, 0x23, 0x10, 0x10, 0x13, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 
	0x14, 0x15, 0x15, 0x19, 0x1e, 0x1e, 0x23, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x1f, 0x1f, 0x22, 
	0x25, 0x25, 0x28, 0x15, 0x15, 0x19, 0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0f, 0x0f, 0x12, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x17, 0x1e, 0x1e, 0x24, 
	0x11, 0x11, 0x14, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x18, 0x18, 0x1d, 0x17, 0x17, 0x1c, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1a, 0x1a, 0x1f, 0x69, 0x69, 0x69, 
	0xa5, 0xa5, 0xa5, 0xb7, 0xb7, 0xb7, 0xcc, 0xcc, 0xcc, 0xc6, 0xc6, 0xc6, 0x99, 0x99, 0x99, 0x82, 
	0x82, 0x81, 0x7e, 0x7e, 0x7f, 0x4b, 0x4b, 0x52, 0x0d, 0x0d, 0x11, 0x0b, 0x0b, 0x0e, 0x1c, 0x1c, 
	0x21, 0x20, 0x20, 0x26, 0x12, 0x12, 0x16, 0x19, 0x19, 0x1e, 0x32, 0x32, 0x3b, 0x17, 0x17, 0x1c, 
	0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1b, 
	0x1b, 0x21, 0x13, 0x13, 0x17, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0d, 0x0d, 0x10, 0x11, 0x11, 0x15, 0x17, 0x17, 0x1c, 0x12, 0x12, 0x16, 0x1c, 0x1c, 0x1f, 
	0x25, 0x25, 0x29, 0x18, 0x18, 0x1c, 0x19, 0x19, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x1a, 
	0x1a, 0x1e, 0x22, 0x22, 0x29, 0x20, 0x20, 0x26, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 
	0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x19, 0x19, 0x1e, 0x24, 0x24, 0x2b, 0x1d, 0x1d, 0x22, 
	0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x19, 0x19, 0x1d, 0x1a, 
	0x1a, 0x1f, 0x25, 0x25, 0x2b, 0x1a, 0x1a, 0x1f, 0x19, 0x19, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 
	0x1d, 0x18, 0x18, 0x1d, 0x19, 0x19, 0x1d, 0x15, 0x15, 0x1b, 0x44, 0x44, 0x48, 0x9b, 0x9b, 0x9b, 
	0xb0, 0xb0, 0xb0, 0xc0, 0xc0, 0xc0, 0xca, 0xca, 0xca, 0xd8, 0xd8, 0xd8, 0xd6, 0xd6, 0xd6, 0xb4, 
	0xb4, 0xb4, 0x99, 0x99, 0x99, 0x89, 0x89, 0x88, 0x60, 0x60, 0x60, 0x20, 0x20, 0x25, 0x12, 0x12, 
	0x17, 0x16, 0x16, 0x1b, 0x18, 0x18, 0x1d, 0x1c, 0x1c, 0x21, 0x26, 0x26, 0x2e, 0x18, 0x18, 0x1c, 
	0x19, 0x19, 0x1e, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x1e, 
	0x1e, 0x24, 0x21, 0x21, 0x28, 0x18, 0x18, 0x1d, 0x19, 0x19, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 
	0x1d, 0x19, 0x19, 0x1d, 0x18, 0x18, 0x1d, 0x1e, 0x1e, 0x24, 0x22, 0x22, 0x28, 0x27, 0x27, 0x2a, 
	0x24, 0x24, 0x27, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1b, 0x1d, 
	0x1d, 0x22, 0x20, 0x20, 0x27, 0x15, 0x15, 0x19, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 
	0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1b, 0x20, 0x20, 0x26, 0x20, 0x20, 0x26, 0x15, 0x15, 0x19, 
	0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x15, 0x15, 0x19, 0x1c, 
	0x1c, 0x22, 0x27, 0x27, 0x2e, 0x15, 0x15, 0x19, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 
	0x1a, 0x16, 0x16, 0x1a, 0x15, 0x15, 0x19, 0x1a, 0x1a, 0x1d, 0x79, 0x79, 0x7a, 0xa9, 0xa9, 0xa9, 
	0xbb, 0xbb, 0xbb, 0xc6, 0xc6, 0xc6, 0xd0, 0xd0, 0xd0, 0xd4, 0xd4, 0xd4, 0xd0, 0xd0, 0xd0, 0xce, 
	0xce, 0xce, 0xb8, 0xb8, 0xb8, 0x99, 0x99, 0x99, 0x9d, 0x9d, 0x9d, 0x76, 0x76, 0x77, 0x2a, 0x2a, 
	0x2d, 0x13, 0x13, 0x18, 0x16, 0x16, 0x1a, 0x15, 0x15, 0x19, 0x1e, 0x1e, 0x24, 0x18, 0x18, 0x1e, 
	0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x14, 
	0x14, 0x19, 0x1e, 0x1e, 0x24, 0x1c, 0x1c, 0x21, 0x15, 0x15, 0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 
	0x1a, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1a, 0x15, 0x15, 0x19, 0x1d, 0x1d, 0x22, 0x28, 0x28, 0x2c, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x0f, 0x0f, 0x12, 0x1d, 0x1d, 0x22, 0x10, 
	0x10, 0x14, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0c, 0x0c, 0x0f, 0x16, 0x16, 0x1a, 0x21, 0x21, 0x27, 0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0f, 0x0f, 0x12, 0x1d, 
	0x1d, 0x22, 0x10, 0x10, 0x13, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0a, 0x0a, 0x0d, 0x39, 0x39, 0x39, 0x9d, 0x9d, 0x9d, 0xb4, 0xb4, 0xb4, 
	0xc3, 0xc3, 0xc3, 0xcc, 0xcc, 0xcc, 0xd5, 0xd5, 0xd5, 0xd1, 0xd1, 0xd1, 0xcb, 0xcb, 0xcb, 0xc3, 
	0xc3, 0xc3, 0xb0, 0xb0, 0xb0, 0x99, 0x99, 0x99, 0x84, 0x84, 0x84, 0x82, 0x82, 0x82, 0x79, 0x79, 
	0x79, 0x29, 0x29, 0x29, 0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x10, 0x19, 0x19, 0x1e, 0x10, 0x10, 0x14, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x13, 0x13, 0x16, 0x22, 0x22, 0x28, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x29, 0x29, 0x2e, 
	0x28, 0x28, 0x2d, 0x19, 0x19, 0x1e, 0x21, 0x21, 0x27, 0x23, 0x23, 0x29, 0x1b, 0x1b, 0x20, 0x1a, 
	0x1a, 0x1f, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x19, 0x19, 
	0x1e, 0x26, 0x26, 0x2e, 0x26, 0x26, 0x2d, 0x1a, 0x1a, 0x1f, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 
	0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x23, 0x23, 0x28, 0x1e, 
	0x1e, 0x24, 0x1a, 0x1a, 0x1f, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 
	0x20, 0x18, 0x18, 0x1d, 0x29, 0x29, 0x2e, 0x86, 0x86, 0x86, 0xac, 0xac, 0xac, 0xbe, 0xbe, 0xbe, 
	0xc9, 0xc9, 0xc9, 0xd2, 0xd2, 0xd2, 0xd4, 0xd4, 0xd4, 0xcd, 0xcd, 0xcd, 0xc7, 0xc7, 0xc7, 0xb8, 
	0xb8, 0xb8, 0xa1, 0xa1, 0xa1, 0x8a, 0x8a, 0x8a, 0x73, 0x73, 0x73, 0x5a, 0x5a, 0x5a, 0x3e, 0x3e, 
	0x3e, 0x2e, 0x2e, 0x2e, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x20, 0x20, 0x26, 0x2b, 0x2b, 0x33, 
	0x19, 0x19, 0x1f, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 
	0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x23, 0x23, 0x29, 0x20, 0x20, 0x27, 0x1a, 0x1a, 0x1e, 0x1b, 0x1b, 
	0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x28, 0x28, 0x2d, 
	0x1d, 0x1d, 0x20, 0x13, 0x13, 0x16, 0x1f, 0x1f, 0x24, 0x14, 0x14, 0x18, 0x0f, 0x0f, 0x12, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x14, 0x14, 
	0x17, 0x20, 0x20, 0x25, 0x14, 0x14, 0x18, 0x0f, 0x0f, 0x12, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x11, 0x11, 0x15, 0x20, 0x20, 0x26, 0x11, 
	0x11, 0x15, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x57, 0x57, 0x59, 0xa4, 0xa4, 0xa4, 0xb7, 0xb7, 0xb7, 0xc6, 0xc6, 0xc6, 
	0xcf, 0xcf, 0xcf, 0xd4, 0xd4, 0xd4, 0xcf, 0xcf, 0xcf, 0xc9, 0xc9, 0xc9, 0xbf, 0xbf, 0xbf, 0xaa, 
	0xaa, 0xaa, 0x93, 0x93, 0x93, 0x7a, 0x7a, 0x7a, 0x63, 0x63, 0x63, 0x48, 0x48, 0x48, 0x22, 0x22, 
	0x22, 0x10, 0x10, 0x13, 0x0f, 0x0f, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x16, 0x21, 0x21, 0x27, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x0f, 0x0f, 0x13, 0x13, 0x13, 0x17, 0x21, 0x21, 0x28, 0x12, 0x12, 0x15, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x1e, 0x1e, 0x21, 
	0x1c, 0x1c, 0x1f, 0x1f, 0x1f, 0x24, 0x0b, 0x0b, 0x0d, 0x08, 0x08, 0x09, 0x08, 0x08, 0x0a, 0x08, 
	0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x09, 0x09, 0x0b, 0x1d, 0x1d, 
	0x22, 0x0b, 0x0b, 0x0d, 0x08, 0x08, 0x09, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 
	0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x09, 0x1d, 0x1d, 0x22, 0x19, 0x19, 0x1d, 0x08, 
	0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x07, 0x07, 
	0x09, 0x37, 0x37, 0x38, 0x95, 0x95, 0x95, 0xad, 0xad, 0xad, 0xc0, 0xc0, 0xc0, 0xcb, 0xcb, 0xcb, 
	0xd4, 0xd4, 0xd4, 0xd3, 0xd3, 0xd3, 0xcb, 0xcb, 0xcb, 0xc4, 0xc4, 0xc4, 0xb4, 0xb4, 0xb4, 0x9d, 
	0x9d, 0x9d, 0x85, 0x85, 0x85, 0x6c, 0x6c, 0x6c, 0x54, 0x54, 0x54, 0x34, 0x34, 0x34, 0x17, 0x17, 
	0x18, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x17, 0x11, 0x11, 0x15, 0x0e, 0x0e, 0x11, 0x15, 0x15, 0x19, 
	0x1a, 0x1a, 0x1e, 0x15, 0x15, 0x19, 0x09, 0x09, 0x0b, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 
	0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x09, 0x0d, 0x0d, 0x10, 0x16, 0x16, 0x1a, 0x08, 0x08, 
	0x09, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x17, 0x17, 0x18, 
	0x24, 0x24, 0x27, 0x15, 0x15, 0x19, 0x10, 0x10, 0x13, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 
	0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x14, 0x1f, 0x1f, 0x25, 0x17, 0x17, 
	0x1b, 0x10, 0x10, 0x13, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 
	0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x14, 0x14, 0x18, 0x20, 0x20, 0x26, 0x11, 0x11, 0x15, 0x10, 
	0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x12, 0x12, 
	0x16, 0x71, 0x71, 0x72, 0xa5, 0xa5, 0xa4, 0xb8, 0xb8, 0xb8, 0xc6, 0xc6, 0xc6, 0xd1, 0xd1, 0xd1, 
	0xd4, 0xd4, 0xd4, 0xcf, 0xcf, 0xcf, 0xc8, 0xc8, 0xc8, 0xb9, 0xb9, 0xb9, 0xa6, 0xa6, 0xa6, 0x90, 
	0x90, 0x90, 0x77, 0x77, 0x77, 0x5d, 0x5d, 0x5d, 0x3f, 0x3f, 0x3f, 0x21, 0x21, 0x21, 0x16, 0x16, 
	0x19, 0x1a, 0x1a, 0x1f, 0x1e, 0x1e, 0x25, 0x1c, 0x1c, 0x22, 0x1a, 0x1a, 0x1f, 0x1d, 0x1d, 0x22, 
	0x27, 0x27, 0x2e, 0x22, 0x22, 0x28, 0x12, 0x12, 0x16, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 
	0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x15, 0x24, 0x24, 0x2a, 0x16, 0x16, 
	0x1a, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x1f, 0x1f, 0x22, 
	0x17, 0x17, 0x19, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 
	0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x08, 0x08, 0x09, 0x19, 0x19, 0x1e, 0x15, 0x15, 0x1a, 0x09, 0x09, 
	0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 
	0x0a, 0x0a, 0x0c, 0x08, 0x08, 0x0a, 0x1c, 0x1c, 0x22, 0x12, 0x12, 0x15, 0x09, 0x09, 0x0b, 0x0a, 
	0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x05, 0x05, 0x07, 0x34, 0x34, 
	0x34, 0x9b, 0x9b, 0x9b, 0xb0, 0xb0, 0xb0, 0xc2, 0xc2, 0xc2, 0xcd, 0xcd, 0xcd, 0xd4, 0xd4, 0xd4, 
	0xd0, 0xd0, 0xd0, 0xca, 0xca, 0xca, 0xc0, 0xc0, 0xc0, 0xab, 0xab, 0xab, 0x97, 0x97, 0x97, 0x80, 
	0x80, 0x80, 0x67, 0x67, 0x67, 0x49, 0x49, 0x49, 0x29, 0x29, 0x29, 0x18, 0x18, 0x1a, 0x09, 0x09, 
	0x0b, 0x09, 0x09, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x13, 0x13, 0x16, 
	0x1a, 0x1a, 0x1e, 0x06, 0x06, 0x07, 0x09, 0x09, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 
	0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x09, 0x09, 0x0c, 0x14, 0x14, 0x18, 0x15, 0x15, 
	0x19, 0x08, 0x08, 0x0a, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x18, 0x18, 0x1a, 
	0x1b, 0x1b, 0x1e, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x0d, 0x0d, 0x0f, 0x17, 0x17, 0x1c, 0x18, 0x18, 0x1c, 0x0c, 0x0c, 0x0f, 0x0d, 0x0d, 
	0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 
	0x0d, 0x0d, 0x10, 0x10, 0x10, 0x13, 0x1a, 0x1a, 0x1e, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x0f, 0x16, 0x16, 0x19, 0x7f, 0x7f, 
	0x80, 0xab, 0xab, 0xab, 0xbb, 0xbb, 0xbb, 0xc7, 0xc7, 0xc7, 0xd2, 0xd2, 0xd2, 0xd3, 0xd3, 0xd3, 
	0xcc, 0xcc, 0xcc, 0xc6, 0xc6, 0xc6, 0xb4, 0xb4, 0xb4, 0x9d, 0x9d, 0x9d, 0x89, 0x89, 0x89, 0x70, 
	0x70, 0x70, 0x56, 0x56, 0x56, 0x32, 0x32, 0x32, 0x14, 0x14, 0x13, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 
	0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0f, 0x0f, 0x11, 
	0x26, 0x26, 0x2d, 0x11, 0x11, 0x14, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x1d, 0x1d, 
	0x22, 0x13, 0x13, 0x17, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x1b, 0x1b, 0x1e, 
	0x18, 0x18, 0x19, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x09, 
	0x09, 0x0b, 0x0c, 0x0c, 0x0f, 0x1f, 0x1f, 0x25, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 
	0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 
	0x08, 0x08, 0x0a, 0x19, 0x19, 0x1e, 0x14, 0x14, 0x17, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 
	0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x09, 0x09, 0x0b, 0x07, 0x07, 0x09, 0x3f, 0x3f, 0x40, 0x9e, 0x9e, 
	0x9e, 0xb4, 0xb4, 0xb4, 0xc2, 0xc2, 0xc2, 0xcd, 0xcd, 0xcd, 0xd4, 0xd4, 0xd4, 0xcf, 0xcf, 0xcf, 
	0xc9, 0xc9, 0xc9, 0xbe, 0xbe, 0xbe, 0xa7, 0xa7, 0xa7, 0x90, 0x90, 0x90, 0x7b, 0x7b, 0x7b, 0x62, 
	0x62, 0x62, 0x43, 0x43, 0x43, 0x23, 0x23, 0x24, 0x0c, 0x0c, 0x0e, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 
	0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x09, 0x09, 0x0b, 
	0x1e, 0x1e, 0x23, 0x13, 0x13, 0x16, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 
	0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x09, 0x09, 0x0b, 0x10, 0x10, 
	0x13, 0x17, 0x17, 0x1b, 0x08, 0x08, 0x0a, 0x0a, 0x0a, 0x0b, 0x0a, 0x0a, 0x0b, 0x18, 0x18, 0x19, 
	0x25, 0x25, 0x29, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1b, 0x1a, 
	0x1a, 0x20, 0x23, 0x23, 0x2a, 0x19, 0x19, 0x1d, 0x17, 0x17, 0x1b, 0x17, 0x17, 0x1c, 0x17, 0x17, 
	0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1b, 
	0x1e, 0x1e, 0x24, 0x22, 0x22, 0x28, 0x17, 0x17, 0x1b, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 
	0x17, 0x1c, 0x17, 0x17, 0x1c, 0x16, 0x16, 0x1b, 0x1d, 0x1d, 0x21, 0x7e, 0x7e, 0x7f, 0xab, 0xab, 
	0xab, 0xbe, 0xbe, 0xbe, 0xc9, 0xc9, 0xc9, 0xd3, 0xd3, 0xd3, 0xd2, 0xd2, 0xd2, 0xcb, 0xcb, 0xcb, 
	0xc3, 0xc3, 0xc3, 0xb1, 0xb1, 0xb1, 0x99, 0x99, 0x99, 0x81, 0x81, 0x81, 0x6b, 0x6b, 0x6b, 0x50, 
	0x50, 0x50, 0x2a, 0x2a, 0x2a, 0x15, 0x15, 0x17, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 
	0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 
	0x19, 0x19, 0x1e, 0x21, 0x21, 0x27, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 
	0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x16, 0x16, 
	0x1a, 0x28, 0x28, 0x2f, 0x21, 0x21, 0x27, 0x17, 0x17, 0x1b, 0x17, 0x17, 0x1c, 0x25, 0x25, 0x29, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 
	0x12, 0x15, 0x05, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 
	0x1e, 0x1e, 0x24, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x47, 0x47, 0x46, 0xa0, 0xa0, 0xa0, 0xb6, 0xb6, 
	0xb6, 0xc5, 0xc5, 0xc5, 0xcf, 0xcf, 0xcf, 0xd4, 0xd4, 0xd4, 0xce, 0xce, 0xce, 0xc7, 0xc7, 0xc7, 
	0xb8, 0xb8, 0xb8, 0xa2, 0xa2, 0xa2, 0x8a, 0x8a, 0x8a, 0x70, 0x70, 0x70, 0x5b, 0x5b, 0x5b, 0x39, 
	0x39, 0x39, 0x23, 0x23, 0x25, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x04, 0x04, 0x05, 0x0e, 0x0e, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x08, 0x08, 0x09, 0x19, 0x19, 0x1d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x24, 0x24, 0x28, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1a, 0x21, 0x21, 0x27, 0x21, 
	0x21, 0x27, 0x15, 0x15, 0x19, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 
	0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x19, 0x1e, 0x1e, 0x24, 
	0x21, 0x21, 0x28, 0x15, 0x15, 0x19, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 
	0x16, 0x1b, 0x15, 0x15, 0x19, 0x27, 0x27, 0x2a, 0x8a, 0x8a, 0x8a, 0xab, 0xab, 0xab, 0xbf, 0xbf, 
	0xbf, 0xcb, 0xcb, 0xcb, 0xd3, 0xd3, 0xd3, 0xd0, 0xd0, 0xd0, 0xc9, 0xc9, 0xc9, 0xbd, 0xbd, 0xbd, 
	0xa8, 0xa8, 0xa8, 0x92, 0x92, 0x92, 0x79, 0x79, 0x79, 0x5f, 0x5f, 0x5f, 0x44, 0x44, 0x44, 0x22, 
	0x22, 0x22, 0x2c, 0x2c, 0x34, 0x1a, 0x1a, 0x1f, 0x16, 0x16, 0x1a, 0x16, 0x16, 0x1b, 0x16, 0x16, 
	0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 
	0x17, 0x17, 0x1c, 0x25, 0x25, 0x2c, 0x19, 0x19, 0x1d, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 
	0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 0x1b, 0x16, 0x16, 
	0x1b, 0x15, 0x15, 0x1a, 0x1e, 0x1e, 0x23, 0x22, 0x22, 0x28, 0x16, 0x16, 0x1a, 0x24, 0x24, 0x28, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x11, 0x19, 0x19, 0x1e, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x02, 0x13, 0x13, 0x16, 
	0x01, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x49, 0x49, 0x49, 0xa3, 0xa3, 0xa3, 0xb5, 0xb5, 0xb5, 0xc5, 0xc5, 
	0xc5, 0xd0, 0xd0, 0xd0, 0xd3, 0xd3, 0xd3, 0xcc, 0xcc, 0xcc, 0xc5, 0xc5, 0xc5, 0xb1, 0xb1, 0xb1, 
	0x9b, 0x9b, 0x9b, 0x84, 0x84, 0x84, 0x6a, 0x6a, 0x6a, 0x4e, 0x4e, 0x4e, 0x2e, 0x2e, 0x2d, 0x0c, 
	0x0c, 0x0b, 0x11, 0x11, 0x15, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x02, 0x15, 0x15, 0x1a, 0x02, 0x02, 0x03, 0x0f, 0x0f, 0x0f, 
	0x1e, 0x1e, 0x21, 0x0f, 0x0f, 0x12, 0x12, 0x12, 0x15, 0x1f, 0x1f, 0x24, 0x12, 0x12, 0x15, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x0f, 0x0f, 0x12, 0x19, 0x19, 0x1d, 0x19, 0x19, 0x1d, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x0e, 
	0x0e, 0x11, 0x1b, 0x1b, 0x1d, 0x82, 0x82, 0x81, 0xab, 0xab, 0xab, 0xbc, 0xbc, 0xbc, 0xc9, 0xc9, 
	0xc9, 0xd2, 0xd2, 0xd2, 0xcf, 0xcf, 0xcf, 0xc9, 0xc9, 0xc9, 0xbc, 0xbc, 0xbc, 0xa2, 0xa2, 0xa2, 
	0x8b, 0x8b, 0x8b, 0x74, 0x74, 0x74, 0x59, 0x59, 0x59, 0x36, 0x36, 0x36, 0x20, 0x20, 0x21, 0x10, 
	0x10, 0x13, 0x1f, 0x1f, 0x26, 0x15, 0x15, 0x19, 0x0f, 0x0f, 0x12, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x12, 0x12, 0x16, 0x1a, 0x1a, 0x20, 0x0f, 0x0f, 0x12, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x0f, 0x0f, 0x12, 0x1b, 0x1b, 0x20, 0x14, 0x14, 0x17, 0x1d, 0x1d, 0x20, 
	0x1c, 0x1c, 0x1e, 0x13, 0x13, 0x16, 0x1a, 0x1a, 0x1f, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x17, 0x22, 0x22, 0x29, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 
	0x0d, 0x10, 0x3c, 0x3c, 0x3e, 0x9e, 0x9e, 0x9e, 0xb5, 0xb5, 0xb4, 0xc3, 0xc3, 0xc3, 0xd0, 0xd0, 
	0xd0, 0xd0, 0xd0, 0xd0, 0xca, 0xca, 0xca, 0xc2, 0xc2, 0xc2, 0xad, 0xad, 0xad, 0x92, 0x92, 0x92, 
	0x7a, 0x7a, 0x7a, 0x62, 0x62, 0x62, 0x42, 0x42, 0x42, 0x20, 0x20, 0x1f, 0x0f, 0x0f, 0x10, 0x0e, 
	0x0e, 0x10, 0x1f, 0x1f, 0x25, 0x13, 0x13, 0x17, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x12, 0x12, 0x14, 0x1a, 0x1a, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x10, 0x10, 0x12, 0x1e, 0x1e, 0x23, 0x1d, 0x1d, 0x20, 
	0x0f, 0x0f, 0x0f, 0x0e, 0x0e, 0x10, 0x01, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0e, 0x07, 0x07, 0x08, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x6c, 0x6c, 0x6c, 0xab, 0xab, 0xab, 0xbb, 0xbb, 0xbb, 0xc8, 0xc8, 0xc8, 0xd1, 0xd1, 
	0xd1, 0xcd, 0xcd, 0xcd, 0xc7, 0xc7, 0xc7, 0xb8, 0xb8, 0xb8, 0x9d, 0x9d, 0x9d, 0x82, 0x82, 0x82, 
	0x69, 0x69, 0x69, 0x4e, 0x4e, 0x4e, 0x29, 0x29, 0x28, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x10, 0x10, 0x14, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17, 0x17, 0x1c, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0a, 0x0a, 0x0c, 0x17, 0x17, 0x18, 
	0x2a, 0x2a, 0x2f, 0x1f, 0x1f, 0x25, 0x16, 0x16, 0x1b, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 
	0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 
	0x1c, 0x17, 0x17, 0x1c, 0x18, 0x18, 0x1d, 0x1e, 0x1e, 0x24, 0x19, 0x19, 0x1e, 0x17, 0x17, 0x1c, 
	0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x16, 0x16, 0x1b, 0x25, 
	0x25, 0x29, 0x8c, 0x8c, 0x8c, 0xb0, 0xb0, 0xb0, 0xbf, 0xbf, 0xbf, 0xcd, 0xcd, 0xcd, 0xcf, 0xcf, 
	0xcf, 0xca, 0xca, 0xca, 0xc2, 0xc2, 0xc2, 0xaa, 0xaa, 0xaa, 0x8f, 0x8f, 0x8f, 0x73, 0x73, 0x73, 
	0x5a, 0x5a, 0x5a, 0x39, 0x39, 0x39, 0x25, 0x25, 0x26, 0x19, 0x19, 0x1e, 0x17, 0x17, 0x1c, 0x17, 
	0x17, 0x1c, 0x26, 0x26, 0x2e, 0x1c, 0x1c, 0x21, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 
	0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 
	0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x20, 0x20, 0x27, 0x1f, 0x1f, 0x25, 0x17, 0x17, 0x1c, 0x17, 
	0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 
	0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x17, 0x17, 0x1c, 0x19, 0x19, 0x1e, 0x33, 0x33, 0x39, 
	0x1c, 0x1c, 0x1f, 0x06, 0x06, 0x07, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 
	0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 
	0x08, 0x06, 0x06, 0x07, 0x16, 0x16, 0x1a, 0x0a, 0x0a, 0x0c, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 
	0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x06, 0x06, 0x08, 0x37, 
	0x37, 0x38, 0x9e, 0x9e, 0x9e, 0xb7, 0xb7, 0xb7, 0xc5, 0xc5, 0xc5, 0xd0, 0xd0, 0xd0, 0xcc, 0xcc, 
	0xcc, 0xc6, 0xc6, 0xc6, 0xb8, 0xb8, 0xb8, 0x9b, 0x9b, 0x9b, 0x7f, 0x7f, 0x7f, 0x62, 0x62, 0x62, 
	0x46, 0x46, 0x46, 0x23, 0x23, 0x22, 0x0a, 0x0a, 0x0b, 0x07, 0x07, 0x09, 0x07, 0x07, 0x08, 0x07, 
	0x07, 0x08, 0x19, 0x19, 0x1e, 0x0c, 0x0c, 0x0f, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 
	0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 
	0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x0a, 0x0a, 0x0c, 0x14, 0x14, 0x18, 0x07, 0x07, 0x08, 0x07, 
	0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 
	0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x07, 0x07, 0x08, 0x06, 0x06, 0x08, 0x1d, 0x1d, 0x20, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x08, 0x08, 0x09, 0x14, 0x14, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 
	0x64, 0x63, 0xad, 0xad, 0xac, 0xc2, 0xc2, 0xc2, 0xc9, 0xc9, 0xc9, 0xbc, 0xbc, 0xbf, 0xd2, 0xd2, 
	0xd2, 0xc4, 0xc4, 0xc4, 0xad, 0xad, 0xad, 0x8f, 0x8f, 0x8f, 0x72, 0x72, 0x71, 0x50, 0x50, 0x50, 
	0x2e, 0x2e, 0x2e, 0x0d, 0x0d, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x12, 0x12, 0x16, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x10, 0x10, 0x13, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1f, 0x1f, 0x22, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 
	0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 
	0x13, 0x25, 0x25, 0x2b, 0x17, 0x17, 0x1c, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 
	0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x17, 0x17, 0x1a, 0x73, 
	0x73, 0x76, 0x96, 0x96, 0x99, 0xa6, 0xa6, 0xa9, 0xaa, 0xaa, 0xad, 0x94, 0x94, 0x99, 0xae, 0xae, 
	0xb0, 0x9c, 0x9c, 0x9f, 0x88, 0x88, 0x8b, 0x72, 0x72, 0x74, 0x59, 0x59, 0x5c, 0x3f, 0x3f, 0x42, 
	0x27, 0x27, 0x2a, 0x14, 0x14, 0x17, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 
	0x10, 0x13, 0x1f, 0x1f, 0x26, 0x15, 0x15, 0x19, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 
	0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x24, 0x24, 0x2a, 0x1a, 0x1a, 0x1f, 0x10, 
	0x10, 0x13, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x1f, 0x1f, 0x22, 
	0x14, 0x14, 0x15, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 
	0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x04, 0x04, 0x05, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 
	0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x1f, 0x1f, 0x20, 0x8b, 
	0x8b, 0x8b, 0xae, 0xae, 0xae, 0xc5, 0xc5, 0xc5, 0x97, 0x97, 0x9c, 0xae, 0xae, 0xb0, 0xc0, 0xc0, 
	0xc0, 0xa6, 0xa6, 0xa6, 0x8b, 0x8b, 0x8c, 0x6f, 0x6f, 0x6f, 0x4f, 0x4f, 0x50, 0x31, 0x31, 0x32, 
	0x17, 0x17, 0x17, 0x04, 0x04, 0x05, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 
	0x05, 0x06, 0x17, 0x17, 0x1c, 0x0a, 0x0a, 0x0c, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 
	0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 
	0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x0c, 0x0c, 0x0e, 0x20, 0x20, 0x26, 0x05, 
	0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 
	0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x14, 0x14, 0x15, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x16, 0x16, 
	0x1a, 0x03, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1f, 0x1e, 0x94, 
	0x94, 0x94, 0xb8, 0xb8, 0xb7, 0xbf, 0xbf, 0xbf, 0x8a, 0x8a, 0x90, 0xc8, 0xc8, 0xc8, 0xbe, 0xbe, 
	0xbd, 0x9f, 0x9f, 0x9f, 0x84, 0x84, 0x84, 0x66, 0x66, 0x66, 0x45, 0x45, 0x45, 0x24, 0x24, 0x24, 
	0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x17, 0x05, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x02, 0x14, 0x14, 0x18, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x10, 0x10, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 
	0x11, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x29, 0x29, 0x29, 0x98, 
	0x98, 0x98, 0xb7, 0xb7, 0xb7, 0xad, 0xad, 0xb0, 0x9a, 0x9a, 0x9e, 0xcb, 0xcb, 0xcb, 0xb2, 0xb2, 
	0xb3, 0x93, 0x93, 0x93, 0x78, 0x78, 0x78, 0x59, 0x59, 0x59, 0x3c, 0x3c, 0x3c, 0x26, 0x26, 0x26, 
	0x02, 0x02, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x15, 0x15, 0x19, 0x07, 0x07, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0f, 0x0f, 0x12, 0x05, 
	0x05, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x10, 0x10, 
	0x26, 0x26, 0x2a, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 
	0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1c, 0x18, 0x18, 0x1c, 0x27, 0x27, 0x2e, 0x1a, 0x1a, 
	0x20, 0x18, 0x18, 0x1c, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 
	0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x16, 0x16, 0x1a, 0x3a, 0x3a, 0x3f, 0x7e, 
	0x7e, 0x82, 0x91, 0x91, 0x95, 0x85, 0x85, 0x8a, 0x95, 0x95, 0x9a, 0x98, 0x98, 0x9c, 0x88, 0x88, 
	0x8c, 0x74, 0x74, 0x77, 0x61, 0x61, 0x65, 0x4c, 0x4c, 0x50, 0x3c, 0x3c, 0x40, 0x24, 0x24, 0x29, 
	0x18, 0x18, 0x1c, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x17, 
	0x17, 0x1c, 0x25, 0x25, 0x2b, 0x1c, 0x1c, 0x20, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 
	0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 
	0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x17, 0x17, 0x1c, 0x20, 0x20, 0x25, 0x22, 
	0x22, 0x28, 0x17, 0x17, 0x1c, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 
	0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x18, 0x18, 0x1d, 0x26, 0x26, 0x2a, 
	0x12, 0x12, 0x13, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 
	0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x03, 0x13, 0x13, 0x17, 0x0e, 0x0e, 0x10, 0x02, 0x02, 
	0x03, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 
	0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x03, 0x38, 0x38, 0x39, 0x9c, 
	0x9c, 0x9c, 0xb8, 0xb8, 0xb8, 0xa7, 0xa7, 0xaa, 0xbc, 0xbc, 0xbd, 0xc0, 0xc0, 0xc1, 0xa6, 0xa6, 
	0xa6, 0x87, 0x87, 0x87, 0x6b, 0x6b, 0x6c, 0x4d, 0x4d, 0x4d, 0x3b, 0x3b, 0x3b, 0x05, 0x05, 0x06, 
	0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 
	0x03, 0x04, 0x16, 0x16, 0x1a, 0x09, 0x09, 0x0a, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 
	0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 
	0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x0b, 0x0b, 0x0d, 0x20, 
	0x20, 0x26, 0x03, 0x03, 0x03, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 
	0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x12, 0x12, 0x13, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x08, 0x1a, 0x1a, 0x1e, 0x00, 0x00, 0x01, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2d, 0x2d, 0x2d, 0x9c, 
	0x9c, 0x9c, 0xaa, 0xaa, 0xab, 0x8e, 0x8e, 0x93, 0xc9, 0xc9, 0xc9, 0xc1, 0xc1, 0xc0, 0xa5, 0xa5, 
	0xa5, 0x85, 0x85, 0x85, 0x69, 0x69, 0x69, 0x4c, 0x4c, 0x4c, 0x44, 0x44, 0x44, 0x06, 0x06, 0x05, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x05, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x12, 
	0x12, 0x15, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x13, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x1f, 0x97, 
	0x97, 0x96, 0xa0, 0xa0, 0xa2, 0x9d, 0x9d, 0xa1, 0xcd, 0xcd, 0xcc, 0xc1, 0xc1, 0xc1, 0xa5, 0xa5, 
	0xa5, 0x86, 0x86, 0x86, 0x69, 0x69, 0x69, 0x4f, 0x4f, 0x4f, 0x4f, 0x4f, 0x4f, 0x03, 0x03, 0x03, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x16, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 
	0x0e, 0x10, 0x03, 0x03, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x13, 0x13, 0x16, 0x22, 0x22, 0x28, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x26, 0x26, 0x28, 0x80, 
	0x80, 0x82, 0x84, 0x84, 0x88, 0xa2, 0xa2, 0xa6, 0xa9, 0xa9, 0xab, 0xa5, 0xa5, 0xa7, 0x8f, 0x8f, 
	0x92, 0x77, 0x77, 0x79, 0x60, 0x60, 0x63, 0x4f, 0x4f, 0x52, 0x57, 0x57, 0x59, 0x21, 0x21, 0x23, 
	0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x10, 0x1f, 0x1f, 0x25, 0x13, 0x13, 0x17, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x19, 
	0x19, 0x1d, 0x17, 0x17, 0x1b, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x1d, 0x1d, 0x20, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 
	0x0f, 0x11, 0x1d, 0x1d, 0x22, 0x13, 0x13, 0x16, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 
	0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 
	0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x11, 0x13, 0x13, 0x16, 0x75, 
	0x75, 0x77, 0x82, 0x82, 0x86, 0x9e, 0x9e, 0xa1, 0xa8, 0xa8, 0xaa, 0xa6, 0xa6, 0xa8, 0x92, 0x92, 
	0x95, 0x7a, 0x7a, 0x7d, 0x66, 0x66, 0x69, 0x58, 0x58, 0x5a, 0x67, 0x67, 0x6a, 0x41, 0x41, 0x44, 
	0x0c, 0x0c, 0x0e, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0e, 
	0x0e, 0x11, 0x20, 0x20, 0x26, 0x14, 0x14, 0x18, 0x0f, 0x0f, 0x11, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 
	0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 
	0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x11, 0x15, 
	0x15, 0x19, 0x26, 0x26, 0x2e, 0x0e, 0x0e, 0x11, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 
	0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x1d, 0x1d, 0x20, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 
	0x08, 0x09, 0x12, 0x12, 0x15, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x58, 
	0x58, 0x59, 0x83, 0x83, 0x88, 0xc0, 0xc0, 0xbf, 0xc6, 0xc6, 0xc5, 0xc5, 0xc5, 0xc5, 0xaf, 0xaf, 
	0xae, 0x90, 0x90, 0x90, 0x7a, 0x7a, 0x7a, 0x68, 0x68, 0x67, 0x72, 0x72, 0x71, 0x5e, 0x5e, 0x5e, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x16, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x10, 0x10, 0x13, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x13, 
	0x13, 0x16, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x25, 
	0x25, 0x27, 0x8b, 0x8b, 0x8d, 0xb3, 0xb3, 0xb3, 0xbc, 0xbc, 0xbd, 0xc1, 0xc1, 0xc1, 0xb1, 0xb1, 
	0xb1, 0x96, 0x96, 0x96, 0x81, 0x81, 0x81, 0x70, 0x70, 0x70, 0x70, 0x70, 0x70, 0x87, 0x87, 0x87, 
	0x4b, 0x4b, 0x4b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0f, 0x0f, 0x12, 0x04, 0x04, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x09, 0x09, 0x0b, 0x12, 
	0x12, 0x15, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 
	0x12, 0x14, 0x89, 0x89, 0x89, 0xad, 0xad, 0xac, 0xbc, 0xbc, 0xbb, 0xc3, 0xc3, 0xc2, 0xbb, 0xbb, 
	0xbb, 0xa1, 0xa1, 0xa1, 0x8d, 0x8d, 0x8c, 0x7d, 0x7d, 0x7d, 0x78, 0x78, 0x77, 0x86, 0x86, 0x86, 
	0x9d, 0x9d, 0x9d, 0x33, 0x33, 0x33, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x17, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0a, 0x0a, 0x0b, 0x13, 0x13, 0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x0d, 0x0d, 0x0f, 0x1c, 0x1c, 0x21, 0x10, 
	0x10, 0x14, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x1d, 
	0x1d, 0x21, 0x6f, 0x6f, 0x71, 0x90, 0x90, 0x93, 0x9e, 0x9e, 0xa1, 0xa6, 0xa6, 0xa8, 0xa5, 0xa5, 
	0xa7, 0x92, 0x92, 0x94, 0x82, 0x82, 0x84, 0x75, 0x75, 0x78, 0x6f, 0x6f, 0x71, 0x76, 0x76, 0x78, 
	0x88, 0x88, 0x8a, 0x71, 0x71, 0x74, 0x10, 0x10, 0x12, 0x0c, 0x0c, 0x0f, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x10, 0x1e, 0x1e, 0x23, 0x13, 0x13, 0x16, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x14, 0x14, 0x18, 0x25, 0x25, 0x2c, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x1a, 0x1a, 0x1c, 0x0c, 0x0c, 0x0e, 0x0b, 0x0b, 0x0e, 0x1b, 0x1b, 0x20, 0x1b, 0x1b, 0x20, 0x0a, 
	0x0a, 0x0d, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 
	0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0b, 0x0b, 0x0e, 0x13, 0x13, 0x17, 0x1d, 
	0x1d, 0x24, 0x2d, 0x2d, 0x2f, 0x85, 0x85, 0x88, 0x98, 0x98, 0x9a, 0xa4, 0xa4, 0xa6, 0xab, 0xab, 
	0xad, 0xa0, 0xa0, 0xa2, 0x8e, 0x8e, 0x90, 0x80, 0x80, 0x82, 0x79, 0x79, 0x7b, 0x7a, 0x7a, 0x7c, 
	0x84, 0x84, 0x86, 0x92, 0x92, 0x94, 0x56, 0x56, 0x58, 0x09, 0x09, 0x0b, 0x0c, 0x0c, 0x0e, 0x0b, 
	0x0b, 0x0d, 0x1c, 0x1c, 0x22, 0x11, 0x11, 0x14, 0x0b, 0x0b, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 
	0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 
	0x0c, 0x0e, 0x0b, 0x0b, 0x0d, 0x18, 0x18, 0x1c, 0x0f, 0x0f, 0x12, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x1a, 0x1a, 0x1c, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x12, 0x12, 0x16, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x0d, 0x0f, 0x06, 
	0x06, 0x07, 0x05, 0x05, 0x05, 0x71, 0x71, 0x71, 0xa4, 0xa4, 0xa4, 0xb4, 0xb4, 0xb4, 0xc1, 0xc1, 
	0xc1, 0xbf, 0xbf, 0xbf, 0xac, 0xac, 0xab, 0x98, 0x98, 0x97, 0x8e, 0x8e, 0x8d, 0x8a, 0x8a, 0x89, 
	0x91, 0x91, 0x90, 0x9f, 0x9f, 0x9f, 0x96, 0x96, 0x96, 0x29, 0x29, 0x29, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x17, 0x05, 0x05, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x10, 0x04, 0x04, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x01, 0x01, 0x01, 0x0d, 0x0d, 0x10, 0x0a, 0x0a, 0x0c, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x11, 0x04, 
	0x04, 0x05, 0x00, 0x00, 0x00, 0x2e, 0x2e, 0x2e, 0x8f, 0x8f, 0x8f, 0xa7, 0xa7, 0xa7, 0xb7, 0xb7, 
	0xb7, 0xc0, 0xc0, 0xc0, 0xb7, 0xb7, 0xb7, 0xa2, 0xa2, 0xa2, 0x93, 0x93, 0x93, 0x8d, 0x8d, 0x8d, 
	0x8e, 0x8e, 0x8e, 0x99, 0x99, 0x99, 0xa3, 0xa3, 0xa3, 0x86, 0x86, 0x86, 0x13, 0x13, 0x13, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x0a, 0x0a, 0x0c, 0x19, 0x19, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x04, 0x04, 0x05, 0x11, 0x11, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x03, 0x15, 0x15, 0x18, 0x02, 
	0x02, 0x03, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x6d, 0x6d, 0x6d, 0x99, 0x99, 0x99, 0xae, 0xae, 
	0xae, 0xbb, 0xbb, 0xbb, 0xc0, 0xc0, 0xc0, 0xae, 0xae, 0xae, 0x9c, 0x9c, 0x9c, 0x94, 0x94, 0x94, 
	0x90, 0x90, 0x90, 0x94, 0x94, 0x94, 0xa0, 0xa0, 0xa0, 0xa9, 0xa9, 0xa9, 0x73, 0x73, 0x73, 0x00, 
	0x00, 0x00, 0x11, 0x11, 0x15, 0x07, 0x07, 0x08, 0x03, 0x03, 0x04, 0x02, 0x02, 0x02, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x01, 0x01, 0x01, 0x02, 0x03, 0x03, 0x03, 0x1f, 0x1f, 0x24, 0x04, 0x04, 0x04, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x16, 0x16, 0x19, 0x0a, 0x0a, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0a, 0x0a, 0x0c, 0x1d, 0x1d, 0x22, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x19, 0x19, 0x19, 0x7e, 0x7e, 0x7e, 0xa4, 0xa4, 
	0xa4, 0xb3, 0xb3, 0xb3, 0xbf, 0xbf, 0xbf, 0xbc, 0xbc, 0xbc, 0xa8, 0xa8, 0xa8, 0x9a, 0x9a, 0x9a, 
	0x96, 0x96, 0x96, 0x94, 0x94, 0x94, 0x9a, 0x9a, 0x9a, 0xa6, 0xa6, 0xa6, 0xa9, 0xa9, 0xa9, 0x45, 
	0x45, 0x44, 0x0e, 0x0e, 0x11, 0x05, 0x06, 0x07, 0x1f, 0x1f, 0x25, 0x29, 0x29, 0x31, 0x04, 0x04, 
	0x05, 0x08, 0x08, 0x0a, 0x00, 0x00, 0x00, 0x13, 0x13, 0x17, 0x0a, 0x0a, 0x0c, 0x35, 0x35, 0x40, 
	0x2a, 0x2a, 0x33, 0x2d, 0x2d, 0x36, 0x42, 0x42, 0x4f, 0x12, 0x12, 0x16, 0x00, 0x00, 0x00, 0x01, 
	0x01, 0x01, 0x2c, 0x2c, 0x35, 0x3a, 0x3a, 0x46, 0x21, 0x21, 0x28, 0x3d, 0x3d, 0x49, 0x18, 0x18, 
	0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x17, 0x17, 0x18, 0x16, 0x16, 0x1a, 0x06, 0x06, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 
	0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 
	0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 
	0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x06, 0x1e, 0x1e, 0x23, 0x10, 0x10, 0x13, 0x05, 
	0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x06, 0x02, 0x02, 0x03, 0x42, 0x42, 0x43, 0x8f, 0x8f, 
	0x90, 0xa4, 0xa4, 0xa4, 0xb1, 0xb1, 0xb2, 0xbc, 0xbc, 0xbd, 0xb3, 0xb3, 0xb4, 0xa2, 0xa2, 0xa2, 
	0x99, 0x99, 0x9a, 0x96, 0x96, 0x97, 0x95, 0x95, 0x96, 0x9c, 0x9c, 0x9c, 0xa6, 0xa6, 0xa6, 0xa7, 
	0xa6, 0xa7, 0x4c, 0x4a, 0x52, 0x04, 0x03, 0x05, 0x1d, 0x1c, 0x22, 0x16, 0x16, 0x1a, 0x0e, 0x0e, 
	0x11, 0x0c, 0x0c, 0x0e, 0x05, 0x05, 0x07, 0x11, 0x11, 0x15, 0x0c, 0x0c, 0x0e, 0x24, 0x24, 0x2b, 
	0x20, 0x20, 0x26, 0x1f, 0x1f, 0x26, 0x2c, 0x2c, 0x35, 0x14, 0x14, 0x18, 0x09, 0x09, 0x0a, 0x05, 
	0x05, 0x06, 0x18, 0x18, 0x1d, 0x28, 0x28, 0x30, 0x0e, 0x0e, 0x11, 0x2c, 0x2c, 0x35, 0x15, 0x15, 
	0x19, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x05, 0x05, 0x07, 0x14, 0x14, 0x16, 
	0x36, 0x36, 0x3d, 0x18, 0x18, 0x1d, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 
	0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 
	0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 
	0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x15, 0x15, 0x19, 0x1d, 0x1d, 0x24, 0x12, 0x12, 0x16, 0x12, 
	0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x15, 0x14, 0x14, 0x18, 0x57, 0x57, 
	0x5b, 0x87, 0x87, 0x89, 0x92, 0x92, 0x95, 0x9f, 0x9f, 0xa3, 0xa7, 0xa7, 0xab, 0x9e, 0x9e, 0xa2, 
	0x93, 0x93, 0x96, 0x8c, 0x8c, 0x90, 0x8a, 0x8a, 0x8d, 0x88, 0x88, 0x8c, 0x8e, 0x8e, 0x92, 0x9d, 
	0xa8, 0xa4, 0x8f, 0x9b, 0x8b, 0x4a, 0x3d, 0x44, 0x13, 0x12, 0x15, 0x10, 0x10, 0x13, 0x11, 0x11, 
	0x15, 0x12, 0x12, 0x15, 0x12, 0x12, 0x16, 0x11, 0x11, 0x14, 0x12, 0x12, 0x15, 0x0e, 0x0e, 0x11, 
	0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0d, 0x0d, 0x10, 0x11, 0x11, 0x14, 0x12, 0x12, 0x15, 0x12, 
	0x12, 0x16, 0x10, 0x10, 0x13, 0x0e, 0x0e, 0x11, 0x15, 0x15, 0x19, 0x18, 0x18, 0x1d, 0x0f, 0x0f, 
	0x12, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x20, 0x20, 0x24, 
	0x20, 0x20, 0x23, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x0d, 0x0d, 0x0f, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1e, 0x1e, 
	0x1e, 0x7c, 0x7c, 0x7b, 0x9d, 0x9d, 0x9d, 0xaf, 0xaf, 0xaf, 0xbf, 0xbf, 0xbe, 0xc2, 0xc2, 0xc1, 
	0xb4, 0xb4, 0xb3, 0xa7, 0xa7, 0xa6, 0xa6, 0xa6, 0xa6, 0xa9, 0xa9, 0xa9, 0x96, 0x90, 0x90, 0xb5, 
	0xa6, 0xa3, 0xbf, 0xc9, 0xbd, 0x4a, 0x46, 0x4a, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x15, 0x15, 0x18, 0x02, 0x02, 
	0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0d, 0x0d, 0x0d, 0x10, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x31, 0x31, 0x31, 0x7e, 0x7e, 0x7e, 0xa0, 0xa0, 0xa0, 0xb3, 0xb3, 0xb3, 0xc3, 0xc3, 0xc3, 
	0xc6, 0xc6, 0xc6, 0xc2, 0xc2, 0xc2, 0x94, 0x94, 0x94, 0x5c, 0x5e, 0x5e, 0x29, 0x1c, 0x1c, 0x25, 
	0x18, 0x19, 0x67, 0x6d, 0x6d, 0x25, 0x38, 0x27, 0x01, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17, 0x17, 0x1c, 0x0a, 0x0a, 
	0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x1c, 0x21, 0x08, 0x08, 0x0a, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x28, 0x28, 0x28, 0x68, 0x68, 0x68, 0x7a, 0x7a, 0x7a, 0x88, 0x88, 0x88, 
	0x91, 0x91, 0x91, 0x6a, 0x6a, 0x6a, 0x10, 0x10, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0b, 0x0b, 0x0f, 0x0c, 0x13, 0x0d, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x05, 0x05, 0x0f, 0x0f, 
	0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x04, 0x04, 0x04, 0x10, 0x10, 0x13, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x05, 0x04, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x04, 0x04, 0x10, 0x10, 
	0x12, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x0e, 0x0e, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x05, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x1c, 
	0x22, 0x07, 0x07, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x11, 0x11, 0x15, 0x10, 0x10, 0x12, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x17, 0x17, 0x1b, 0x0a, 0x0a, 0x0b, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x15, 0x15, 
	0x19, 0x0f, 0x0f, 0x11, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x13, 0x13, 0x14, 
	0x1e, 0x1e, 0x21, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x0f, 0x0f, 0x12, 0x26, 0x26, 0x2d, 0x16, 0x16, 0x1a, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x0f, 
	0x0f, 0x12, 0x1f, 0x1f, 0x25, 0x15, 0x15, 0x18, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x12, 0x12, 
	0x15, 0x1b, 0x1b, 0x20, 0x0f, 0x0f, 0x12, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x1e, 0x1e, 0x21, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x02, 0x02, 0x03, 0x0f, 0x0f, 0x12, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x13, 0x13, 0x17, 0x05, 0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 
	0x04, 0x0e, 0x0e, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x02, 0x02, 0x03, 0x0e, 0x0e, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x1d, 0x1d, 0x22, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x10, 0x10, 0x13, 0x0c, 0x0c, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x14, 0x14, 0x17, 0x14, 0x14, 0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0b, 0x0b, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 
	0x1e, 0x1e, 0x23, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x14, 0x14, 0x18, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x03, 0x03, 0x03, 0x17, 0x17, 0x1b, 0x02, 0x02, 0x03, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0b, 
	0x1e, 0x1e, 0x1e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x17, 0x17, 0x18, 
	0x29, 0x29, 0x2e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x22, 0x22, 0x26, 0x15, 0x15, 0x16, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x1b, 0x1b, 0x1e, 0x13, 0x13, 0x14, 0x0f, 0x0f, 0x0f, 0x1e, 0x1e, 0x1e
};



        // Thumbnail for the second take.
        // 64x64 RGB pixels
        readonly byte[] TakeTwoThumbnail = {
	0x3e, 0x3e, 0x43, 0x31, 0x31, 0x37, 0x30, 0x30, 0x37, 0x31, 0x31, 0x37, 0x40, 0x40, 0x4a, 0x35, 
	0x35, 0x3b, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x30, 0x30, 0x36, 0x39, 0x39, 
	0x42, 0x37, 0x37, 0x3e, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x30, 0x30, 0x37, 
	0x34, 0x34, 0x3a, 0x3d, 0x3d, 0x46, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 
	0x31, 0x37, 0x30, 0x30, 0x37, 0x37, 0x37, 0x3e, 0x33, 0x33, 0x39, 0x31, 0x31, 0x37, 0x31, 0x31, 
	0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x37, 0x37, 0x3f, 0x33, 0x33, 0x39, 0x31, 0x31, 0x37, 
	0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x32, 0x32, 0x38, 0x3d, 0x3d, 0x46, 0x32, 
	0x32, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x3e, 0x3e, 
	0x47, 0x34, 0x34, 0x3a, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 
	0x36, 0x36, 0x3d, 0x3b, 0x3b, 0x44, 0x30, 0x30, 0x37, 0x31, 0x31, 0x37, 0x31, 0x31, 0x37, 0x31, 
	0x31, 0x37, 0x33, 0x33, 0x39, 0x3c, 0x3c, 0x45, 0x32, 0x32, 0x38, 0x33, 0x33, 0x39, 0x33, 0x33, 
	0x39, 0x33, 0x33, 0x39, 0x33, 0x33, 0x39, 0x39, 0x39, 0x41, 0x34, 0x34, 0x3a, 0x3f, 0x3f, 0x45, 
	0x17, 0x17, 0x1a, 0x0d, 0x0d, 0x0f, 0x10, 0x10, 0x13, 0x1a, 0x1a, 0x1f, 0x19, 0x19, 0x1f, 0x0e, 
	0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0b, 0x0b, 0x0d, 0x15, 0x15, 0x19, 0x1c, 0x1c, 
	0x20, 0x10, 0x10, 0x13, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x0f, 0x0e, 0x0e, 0x10, 
	0x19, 0x19, 0x1e, 0x15, 0x15, 0x19, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x11, 0x11, 0x14, 0x1f, 0x1f, 0x25, 0x0f, 0x0f, 0x12, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 
	0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x1a, 0x1a, 0x1f, 0x11, 0x11, 0x14, 0x0d, 0x0d, 0x10, 
	0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x10, 0x10, 0x13, 0x25, 0x25, 0x2c, 0x0f, 
	0x0f, 0x12, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x19, 0x19, 
	0x1e, 0x16, 0x16, 0x1b, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 
	0x12, 0x12, 0x15, 0x1f, 0x1f, 0x25, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x0e, 0x0e, 0x10, 0x1a, 0x1a, 0x20, 0x1c, 0x1c, 0x21, 0x14, 0x14, 0x17, 0x14, 0x14, 
	0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x18, 0x18, 0x1d, 0x1d, 0x1d, 0x22, 0x22, 0x22, 0x27, 
	0x1b, 0x1b, 0x1d, 0x15, 0x15, 0x19, 0x1d, 0x1d, 0x23, 0x12, 0x12, 0x16, 0x0e, 0x0e, 0x11, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x0f, 0x0f, 0x12, 0x24, 0x24, 0x2b, 0x1e, 0x1e, 0x23, 0x11, 0x11, 
	0x14, 0x10, 0x10, 0x12, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x1f, 0x1f, 0x25, 
	0x15, 0x15, 0x19, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x12, 0x1d, 0x1d, 0x22, 0x15, 0x15, 0x1a, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x11, 0x11, 0x15, 0x20, 0x20, 0x27, 0x12, 0x12, 0x15, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x12, 0x12, 0x16, 0x25, 0x25, 0x2c, 0x0d, 
	0x0d, 0x0f, 0x0f, 0x0f, 0x12, 0x10, 0x10, 0x13, 0x0b, 0x0b, 0x0d, 0x0d, 0x0d, 0x10, 0x11, 0x11, 
	0x15, 0x18, 0x18, 0x1d, 0x0b, 0x0b, 0x0d, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x1a, 0x1a, 0x1e, 0x20, 0x20, 0x26, 0x10, 0x10, 0x12, 0x10, 0x10, 0x13, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x11, 0x11, 0x14, 0x1b, 0x1b, 0x20, 0x17, 0x17, 0x1b, 0x0f, 0x0f, 
	0x11, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x11, 0x12, 0x12, 0x16, 0x2d, 0x2d, 0x33, 
	0x3e, 0x3e, 0x47, 0x18, 0x18, 0x1c, 0x12, 0x12, 0x16, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 
	0x11, 0x14, 0x10, 0x10, 0x13, 0x21, 0x21, 0x27, 0x1c, 0x1c, 0x21, 0x12, 0x12, 0x16, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x1d, 0x1d, 0x23, 0x1c, 0x1c, 0x21, 
	0x10, 0x10, 0x13, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x19, 
	0x19, 0x1e, 0x1a, 0x1a, 0x20, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 
	0x14, 0x11, 0x11, 0x14, 0x19, 0x19, 0x1e, 0x22, 0x22, 0x29, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 
	0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x13, 0x13, 0x17, 0x26, 0x26, 0x2d, 0x34, 
	0x34, 0x3e, 0x16, 0x16, 0x1a, 0x12, 0x12, 0x15, 0x37, 0x37, 0x42, 0x28, 0x28, 0x30, 0x17, 0x17, 
	0x1c, 0x2f, 0x2f, 0x38, 0x36, 0x36, 0x41, 0x13, 0x13, 0x16, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 
	0x11, 0x11, 0x14, 0x12, 0x12, 0x16, 0x20, 0x20, 0x26, 0x13, 0x13, 0x17, 0x11, 0x11, 0x14, 0x11, 
	0x11, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x13, 0x13, 0x17, 0x20, 0x20, 0x26, 0x14, 0x14, 
	0x18, 0x10, 0x10, 0x14, 0x11, 0x11, 0x14, 0x11, 0x11, 0x14, 0x10, 0x10, 0x14, 0x20, 0x20, 0x23, 
	0x29, 0x29, 0x2f, 0x13, 0x13, 0x16, 0x12, 0x12, 0x16, 0x13, 0x13, 0x17, 0x13, 0x13, 0x16, 0x11, 
	0x11, 0x15, 0x22, 0x22, 0x29, 0x1c, 0x1c, 0x21, 0x12, 0x12, 0x16, 0x12, 0x12, 0x16, 0x13, 0x13, 
	0x17, 0x13, 0x13, 0x17, 0x12, 0x12, 0x16, 0x1f, 0x1f, 0x24, 0x20, 0x20, 0x26, 0x13, 0x13, 0x17, 
	0x13, 0x13, 0x16, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x12, 0x12, 0x16, 0x1f, 
	0x1f, 0x26, 0x15, 0x15, 0x19, 0x13, 0x13, 0x16, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x13, 0x13, 
	0x17, 0x13, 0x13, 0x16, 0x1f, 0x1f, 0x25, 0x18, 0x18, 0x1d, 0x13, 0x13, 0x16, 0x13, 0x13, 0x17, 
	0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x15, 0x15, 0x1a, 0x28, 0x28, 0x30, 0x1e, 
	0x1e, 0x24, 0x14, 0x14, 0x18, 0x13, 0x13, 0x17, 0x1e, 0x1e, 0x24, 0x1a, 0x1a, 0x1f, 0x14, 0x14, 
	0x18, 0x2f, 0x2f, 0x37, 0x23, 0x23, 0x2a, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 
	0x13, 0x13, 0x17, 0x12, 0x12, 0x16, 0x1c, 0x1c, 0x21, 0x22, 0x22, 0x29, 0x12, 0x12, 0x16, 0x13, 
	0x13, 0x17, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x12, 0x12, 0x16, 0x16, 0x16, 0x1a, 0x21, 0x21, 
	0x28, 0x14, 0x14, 0x18, 0x13, 0x13, 0x16, 0x13, 0x13, 0x17, 0x13, 0x13, 0x17, 0x20, 0x20, 0x24, 
	0x1d, 0x1d, 0x20, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x12, 0x1f, 
	0x1f, 0x25, 0x1a, 0x1a, 0x1f, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x10, 0x10, 0x13, 0x13, 0x13, 0x16, 0x20, 0x20, 0x26, 0x14, 0x14, 0x17, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x18, 0x18, 0x1d, 0x19, 
	0x19, 0x1e, 0x11, 0x11, 0x14, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 
	0x13, 0x11, 0x11, 0x14, 0x1f, 0x1f, 0x24, 0x13, 0x13, 0x16, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x13, 0x13, 0x17, 0x27, 0x27, 0x2e, 0x11, 
	0x11, 0x14, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x0f, 0x0f, 0x11, 0x0f, 0x0f, 0x12, 0x10, 0x10, 
	0x13, 0x1b, 0x1b, 0x20, 0x17, 0x17, 0x1c, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 
	0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x12, 0x12, 0x15, 0x1f, 0x1f, 0x25, 0x13, 0x13, 0x16, 0x10, 
	0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x18, 0x18, 
	0x1d, 0x20, 0x20, 0x26, 0x0f, 0x0f, 0x11, 0x10, 0x10, 0x13, 0x10, 0x10, 0x13, 0x1e, 0x1e, 0x21, 
	0x17, 0x17, 0x19, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x0d, 0x0d, 0x10, 0x13, 0x13, 0x16, 0x10, 
	0x10, 0x13, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 
	0x0a, 0x0d, 0x0d, 0x0f, 0x18, 0x18, 0x1c, 0x0d, 0x0d, 0x0f, 0x08, 0x08, 0x0a, 0x09, 0x09, 0x0b, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x16, 0x16, 0x1b, 0x0b, 
	0x0b, 0x0e, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 
	0x0b, 0x0c, 0x0c, 0x0e, 0x1c, 0x1c, 0x21, 0x0a, 0x0a, 0x0d, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x0c, 0x0c, 0x0e, 0x21, 0x21, 0x27, 0x0b, 
	0x0b, 0x0c, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 
	0x0b, 0x0c, 0x0c, 0x0e, 0x16, 0x16, 0x1a, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x0f, 0x0f, 0x12, 0x1c, 0x1c, 0x20, 0x09, 
	0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 
	0x0b, 0x14, 0x14, 0x17, 0x19, 0x19, 0x1e, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x17, 0x17, 0x19, 
	0x27, 0x27, 0x2c, 0x19, 0x19, 0x1e, 0x1b, 0x1b, 0x21, 0x24, 0x24, 0x2b, 0x21, 0x21, 0x27, 0x19, 
	0x19, 0x1e, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 
	0x1f, 0x22, 0x22, 0x28, 0x22, 0x22, 0x28, 0x19, 0x19, 0x1e, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 
	0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x19, 0x19, 0x1e, 0x1f, 0x1f, 0x25, 0x23, 0x23, 0x2a, 0x1a, 
	0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 
	0x1f, 0x20, 0x20, 0x26, 0x27, 0x27, 0x2e, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 
	0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1c, 0x1c, 0x21, 0x2b, 0x2b, 0x32, 0x1b, 
	0x1b, 0x20, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 
	0x1f, 0x1c, 0x1c, 0x21, 0x24, 0x24, 0x2b, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 
	0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x27, 0x27, 0x2e, 0x1c, 
	0x1c, 0x22, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 0x1f, 0x1a, 0x1a, 
	0x1f, 0x1c, 0x1c, 0x21, 0x28, 0x28, 0x2f, 0x1d, 0x1d, 0x23, 0x1a, 0x1a, 0x1e, 0x27, 0x27, 0x2c, 
	0x1a, 0x1a, 0x1c, 0x0f, 0x0f, 0x12, 0x1c, 0x1c, 0x21, 0x11, 0x11, 0x14, 0x0c, 0x0c, 0x0e, 0x0d, 
	0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0c, 0x0c, 0x0e, 0x1b, 0x1b, 
	0x20, 0x18, 0x18, 0x1d, 0x0b, 0x0b, 0x0e, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 
	0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x10, 0x10, 0x13, 0x1f, 0x1f, 0x25, 0x0d, 0x0d, 0x10, 0x0c, 
	0x0c, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 
	0x0f, 0x1a, 0x1a, 0x1f, 0x0f, 0x0f, 0x12, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 
	0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0f, 0x0f, 0x12, 0x23, 0x23, 0x29, 0x0d, 
	0x0d, 0x10, 0x0c, 0x0c, 0x0f, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x25, 0x25, 0x2b, 0x13, 0x13, 0x17, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0f, 
	0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0c, 0x0c, 0x0f, 0x13, 0x13, 0x16, 0x20, 
	0x20, 0x26, 0x0c, 0x0c, 0x0e, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 
	0x0f, 0x0c, 0x0c, 0x0e, 0x0d, 0x0d, 0x0f, 0x1d, 0x1d, 0x23, 0x11, 0x11, 0x15, 0x1a, 0x1a, 0x1c, 
	0x1d, 0x1d, 0x20, 0x18, 0x18, 0x1c, 0x0c, 0x0c, 0x0e, 0x08, 0x08, 0x0a, 0x09, 0x09, 0x0b, 0x09, 
	0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x14, 0x14, 0x18, 0x15, 0x15, 
	0x19, 0x0a, 0x0a, 0x0c, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x15, 0x15, 0x19, 0x11, 0x11, 0x14, 0x09, 0x09, 0x0a, 0x09, 
	0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 
	0x0b, 0x1a, 0x1a, 0x1f, 0x0c, 0x0c, 0x0e, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x0c, 0x0c, 0x0e, 0x21, 0x21, 0x27, 0x10, 
	0x10, 0x12, 0x0b, 0x0b, 0x0d, 0x0c, 0x0c, 0x0f, 0x0c, 0x0c, 0x0e, 0x0e, 0x0e, 0x11, 0x0c, 0x0c, 
	0x0f, 0x0e, 0x0e, 0x10, 0x10, 0x10, 0x14, 0x19, 0x19, 0x1d, 0x0f, 0x0f, 0x12, 0x0a, 0x0a, 0x0c, 
	0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x16, 
	0x16, 0x1a, 0x15, 0x15, 0x19, 0x09, 0x09, 0x0a, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 
	0x0b, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 0x0b, 0x0b, 0x0e, 0x18, 0x18, 0x1d, 0x1b, 0x1b, 0x1d, 
	0x3a, 0x3a, 0x44, 0x24, 0x24, 0x2c, 0x1f, 0x1f, 0x25, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 
	0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x25, 0x22, 0x22, 0x29, 0x2c, 0x2c, 0x35, 0x21, 0x21, 
	0x28, 0x1f, 0x1f, 0x25, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 
	0x1f, 0x1f, 0x26, 0x20, 0x20, 0x27, 0x28, 0x28, 0x30, 0x21, 0x21, 0x28, 0x1f, 0x1f, 0x26, 0x1f, 
	0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x20, 0x20, 
	0x27, 0x2d, 0x2d, 0x37, 0x21, 0x21, 0x27, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 
	0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x21, 0x21, 0x28, 0x30, 0x30, 0x3a, 0x31, 
	0x31, 0x3c, 0x27, 0x27, 0x2f, 0x2b, 0x2b, 0x33, 0x2a, 0x2a, 0x33, 0x30, 0x30, 0x3a, 0x2b, 0x2b, 
	0x34, 0x2f, 0x2f, 0x38, 0x23, 0x23, 0x2b, 0x33, 0x33, 0x3d, 0x33, 0x33, 0x3e, 0x24, 0x24, 0x2c, 
	0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x24, 
	0x24, 0x2c, 0x2c, 0x2c, 0x36, 0x1f, 0x1f, 0x25, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 
	0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x26, 0x1f, 0x1f, 0x25, 0x25, 0x25, 0x2d, 0x36, 0x36, 0x3e, 
	0x1a, 0x1a, 0x1d, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x10, 0x10, 0x13, 0x1f, 0x1f, 0x25, 0x11, 0x11, 0x15, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x10, 0x1b, 0x1b, 0x20, 0x13, 0x13, 0x17, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1a, 0x1a, 
	0x1f, 0x1a, 0x1a, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x11, 0x11, 0x14, 0x25, 0x25, 0x2c, 0x0e, 
	0x0e, 0x10, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x0f, 0x0d, 0x0d, 0x0f, 0x0c, 0x0c, 0x0f, 0x0d, 0x0d, 
	0x0f, 0x0c, 0x0c, 0x0f, 0x0f, 0x0f, 0x13, 0x1f, 0x1f, 0x26, 0x0f, 0x0f, 0x12, 0x0e, 0x0e, 0x10, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 
	0x0d, 0x10, 0x1b, 0x1b, 0x20, 0x17, 0x17, 0x1b, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x22, 0x22, 0x26, 
	0x1a, 0x1a, 0x1c, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 
	0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x1e, 0x1e, 0x24, 0x13, 0x13, 0x17, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 
	0x10, 0x10, 0x13, 0x1e, 0x1e, 0x24, 0x0e, 0x0e, 0x10, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 
	0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x19, 0x19, 
	0x1e, 0x0f, 0x0f, 0x12, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 
	0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0f, 0x0f, 0x12, 0x23, 0x23, 0x29, 0x0e, 
	0x0e, 0x10, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x23, 0x23, 0x2a, 0x14, 0x14, 0x17, 0x0c, 0x0c, 0x0e, 
	0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 
	0x0c, 0x0e, 0x12, 0x12, 0x16, 0x1f, 0x1f, 0x24, 0x0c, 0x0c, 0x0f, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 
	0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x0c, 0x0c, 0x0e, 0x19, 0x19, 0x1b, 
	0x12, 0x12, 0x12, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x02, 0x02, 0x03, 0x02, 
	0x02, 0x02, 0x13, 0x13, 0x18, 0x0f, 0x0f, 0x12, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x01, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x0e, 0x0e, 0x12, 0x0a, 0x0a, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x02, 0x01, 
	0x01, 0x02, 0x01, 0x01, 0x02, 0x02, 0x02, 0x02, 0x03, 0x03, 0x03, 0x02, 0x02, 0x02, 0x13, 0x13, 
	0x17, 0x06, 0x06, 0x07, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x05, 0x05, 0x07, 0x1d, 0x1d, 0x23, 0x04, 
	0x04, 0x05, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x0b, 0x0b, 0x0e, 0x10, 0x10, 0x12, 0x02, 0x02, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x12, 0x12, 0x16, 0x0e, 0x0e, 0x11, 0x02, 0x02, 0x02, 0x03, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x12, 0x12, 0x12, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x15, 
	0x15, 0x19, 0x1e, 0x1e, 0x24, 0x0f, 0x0f, 0x12, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x11, 0x11, 0x14, 0x36, 0x36, 0x39, 0x17, 0x17, 0x19, 0x21, 0x21, 0x26, 
	0x22, 0x22, 0x26, 0x17, 0x17, 0x1a, 0x17, 0x17, 0x19, 0x14, 0x14, 0x17, 0x0d, 0x0d, 0x10, 0x0d, 
	0x0d, 0x10, 0x0e, 0x0e, 0x10, 0x0b, 0x0b, 0x0d, 0x04, 0x04, 0x07, 0x0a, 0x0a, 0x0e, 0x19, 0x19, 
	0x20, 0x0c, 0x0c, 0x0f, 0x0b, 0x0b, 0x0e, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x11, 0x11, 0x14, 0x24, 0x24, 0x2b, 0x10, 
	0x10, 0x12, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x12, 0x12, 0x15, 0x1b, 0x1b, 0x1f, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x10, 0x10, 0x13, 0x1e, 0x1e, 0x24, 0x10, 0x10, 0x13, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x03, 0x10, 
	0x10, 0x13, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x8f, 0x8f, 0x8f, 0x84, 0x84, 0x84, 0x86, 0x86, 0x87, 
	0x84, 0x84, 0x84, 0x82, 0x82, 0x82, 0x84, 0x84, 0x84, 0x7b, 0x7b, 0x7b, 0x64, 0x64, 0x63, 0x64, 
	0x64, 0x64, 0x67, 0x67, 0x67, 0x5c, 0x5c, 0x5c, 0x46, 0x46, 0x46, 0x51, 0x51, 0x52, 0x3d, 0x3d, 
	0x3e, 0x07, 0x07, 0x07, 0x08, 0x08, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1a, 0x1a, 0x1e, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17, 0x17, 0x1b, 0x02, 0x02, 0x03, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x0e, 0x0a, 0x0a, 0x0b, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x1b, 0x1b, 0x21, 0x17, 
	0x17, 0x1c, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0b, 0x0b, 0x0e, 0x74, 0x74, 0x75, 0xa6, 0xa6, 0xa6, 0x9d, 0x9d, 0x9d, 0x9c, 0x9c, 0x9c, 
	0x9b, 0x9b, 0x9b, 0x9a, 0x9a, 0x9a, 0x99, 0x99, 0x99, 0x99, 0x99, 0x99, 0x9b, 0x9b, 0x9b, 0x9a, 
	0x9a, 0x9a, 0x99, 0x99, 0x99, 0x9a, 0x9a, 0x9a, 0x9c, 0x9c, 0x9c, 0x9d, 0x9d, 0x9c, 0x96, 0x96, 
	0x96, 0x8b, 0x8b, 0x8b, 0x8d, 0x8d, 0x8d, 0x61, 0x61, 0x61, 0x4c, 0x4c, 0x4c, 0x36, 0x36, 0x36, 
	0x26, 0x26, 0x27, 0x17, 0x17, 0x19, 0x06, 0x06, 0x08, 0x08, 0x08, 0x0c, 0x24, 0x24, 0x2b, 0x10, 
	0x10, 0x13, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x23, 0x23, 0x2b, 0x15, 0x15, 0x19, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x16, 0x23, 0x23, 0x29, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x12, 0x12, 0x13, 0x03, 0x03, 0x04, 0x03, 0x03, 0x03, 0x15, 0x15, 0x19, 0x0e, 0x0e, 0x10, 0x03, 
	0x03, 0x03, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 
	0x04, 0x22, 0x22, 0x23, 0xab, 0xab, 0xab, 0xb5, 0xb5, 0xb5, 0xb1, 0xb1, 0xb1, 0xb0, 0xb0, 0xb0, 
	0xaf, 0xaf, 0xaf, 0xae, 0xae, 0xae, 0xad, 0xad, 0xad, 0xab, 0xab, 0xab, 0xa9, 0xa9, 0xa9, 0xa7, 
	0xa7, 0xa7, 0xa4, 0xa4, 0xa4, 0xa3, 0xa3, 0xa3, 0xa1, 0xa1, 0xa1, 0x9e, 0x9e, 0x9e, 0x9e, 0x9e, 
	0x9e, 0x9d, 0x9d, 0x9d, 0x9c, 0x9c, 0x9c, 0xa2, 0xa2, 0xa2, 0xa5, 0xa5, 0xa5, 0x9f, 0x9f, 0x9f, 
	0x9b, 0x9b, 0x9b, 0x98, 0x98, 0x99, 0x5f, 0x5f, 0x5f, 0x39, 0x39, 0x3a, 0x22, 0x22, 0x27, 0x00, 
	0x00, 0x00, 0x01, 0x01, 0x02, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 
	0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x0a, 0x0a, 0x0c, 0x11, 0x11, 0x15, 
	0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 
	0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x03, 0x0f, 0x0f, 0x13, 0x14, 0x14, 
	0x18, 0x03, 0x03, 0x03, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x03, 0x03, 0x04, 0x12, 0x12, 0x13, 
	0x17, 0x17, 0x17, 0x07, 0x07, 0x08, 0x11, 0x11, 0x14, 0x17, 0x17, 0x1b, 0x08, 0x08, 0x09, 0x08, 
	0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x07, 0x07, 
	0x08, 0x3e, 0x3e, 0x3f, 0xcb, 0xcb, 0xcb, 0xc4, 0xc4, 0xc4, 0xc2, 0xc2, 0xc2, 0xc1, 0xc1, 0xc1, 
	0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xbf, 0xbf, 0xbf, 0xbd, 0xbd, 0xbd, 0xbb, 0xbb, 0xbb, 0xba, 
	0xba, 0xba, 0xb8, 0xb8, 0xb8, 0xb6, 0xb6, 0xb6, 0xb3, 0xb3, 0xb3, 0xb0, 0xb0, 0xb0, 0xae, 0xae, 
	0xae, 0xab, 0xab, 0xab, 0xa7, 0xa7, 0xa7, 0xa5, 0xa5, 0xa5, 0xa2, 0xa2, 0xa2, 0xa1, 0xa1, 0xa1, 
	0xa1, 0xa1, 0xa1, 0xa2, 0xa2, 0xa2, 0xaa, 0xaa, 0xaa, 0xab, 0xab, 0xab, 0x84, 0x84, 0x85, 0x3c, 
	0x3c, 0x3b, 0x08, 0x08, 0x08, 0x00, 0x00, 0x02, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 
	0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x0c, 0x0c, 0x0e, 0x14, 0x14, 0x18, 
	0x07, 0x07, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 
	0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x0b, 0x0b, 0x0d, 0x1c, 0x1c, 
	0x22, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x08, 0x08, 0x09, 0x17, 0x17, 0x17, 
	0x28, 0x28, 0x2e, 0x22, 0x22, 0x28, 0x25, 0x25, 0x2d, 0x1c, 0x1c, 0x22, 0x1c, 0x1c, 0x21, 0x1c, 
	0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x19, 0x19, 
	0x1e, 0x78, 0x78, 0x7c, 0xdb, 0xdb, 0xdb, 0xd2, 0xd2, 0xd2, 0xd1, 0xd1, 0xd1, 0xd0, 0xd0, 0xd0, 
	0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcb, 0xcb, 0xcb, 0xca, 
	0xca, 0xca, 0xc9, 0xc9, 0xc9, 0xc7, 0xc7, 0xc7, 0xc5, 0xc5, 0xc5, 0xc2, 0xc2, 0xc2, 0xc0, 0xc0, 
	0xc0, 0xbc, 0xbc, 0xbc, 0xb8, 0xb8, 0xb8, 0xb3, 0xb3, 0xb3, 0xaf, 0xaf, 0xaf, 0xa9, 0xa9, 0xa9, 
	0xa6, 0xa6, 0xa6, 0xa3, 0xa3, 0xa3, 0xa0, 0xa0, 0xa0, 0xa4, 0xa4, 0xa4, 0xac, 0xac, 0xac, 0xb1, 
	0xb1, 0xb0, 0x90, 0x90, 0x93, 0x45, 0x45, 0x4b, 0x14, 0x14, 0x1a, 0x1b, 0x1b, 0x20, 0x1c, 0x1c, 
	0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x22, 0x2d, 0x2d, 0x35, 
	0x1e, 0x1e, 0x24, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 
	0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x22, 0x22, 
	0x29, 0x28, 0x28, 0x2f, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x29, 0x29, 0x2e, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x8e, 0x8e, 0x8d, 0xe3, 0xe3, 0xe3, 0xd3, 0xd3, 0xd3, 0xd4, 0xd4, 0xd4, 0xd4, 0xd4, 0xd4, 
	0xd4, 0xd4, 0xd4, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 0xd5, 
	0xd5, 0xd5, 0xd4, 0xd4, 0xd4, 0xd4, 0xd4, 0xd4, 0xd3, 0xd3, 0xd3, 0xd1, 0xd1, 0xd1, 0xd0, 0xd0, 
	0xd0, 0xcd, 0xcd, 0xcd, 0xca, 0xca, 0xca, 0xc6, 0xc6, 0xc6, 0xc2, 0xc2, 0xc2, 0xbc, 0xbc, 0xbc, 
	0xb6, 0xb6, 0xb6, 0xae, 0xae, 0xae, 0xa5, 0xa5, 0xa5, 0xa1, 0xa1, 0xa1, 0x9f, 0x9f, 0x9f, 0xa5, 
	0xa5, 0xa5, 0xb2, 0xb2, 0xb1, 0xa9, 0xa9, 0xa9, 0x46, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x12, 0x15, 
	0x0c, 0x0c, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x12, 0x12, 0x15, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x31, 0x31, 0x37, 0x1a, 0x1a, 0x1f, 0x13, 0x13, 0x17, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 
	0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x12, 0x12, 
	0x17, 0x8c, 0x8c, 0x8d, 0xd9, 0xd9, 0xd9, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 
	0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd1, 0xd1, 0xd1, 0xd1, 0xd1, 0xd1, 0xd2, 
	0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd3, 0xd3, 0xd3, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 
	0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd2, 0xd1, 0xd1, 0xd1, 0xd0, 0xd0, 0xd0, 0xcd, 0xcd, 0xcd, 
	0xc9, 0xc9, 0xc9, 0xc2, 0xc2, 0xc2, 0xb9, 0xb9, 0xb9, 0xad, 0xad, 0xad, 0x9f, 0x9f, 0x9f, 0x9a, 
	0x9a, 0x9a, 0xa1, 0xa1, 0xa1, 0xac, 0xac, 0xac, 0xb5, 0xb5, 0xb5, 0x51, 0x51, 0x53, 0x0e, 0x0e, 
	0x12, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x19, 0x19, 0x1f, 
	0x26, 0x26, 0x2e, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 
	0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x13, 0x13, 
	0x17, 0x1b, 0x1b, 0x21, 0x1d, 0x1d, 0x23, 0x14, 0x14, 0x18, 0x14, 0x14, 0x18, 0x22, 0x22, 0x26, 
	0x14, 0x14, 0x15, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 
	0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x05, 0x05, 
	0x07, 0x5c, 0x5c, 0x60, 0xbd, 0xbd, 0xbf, 0xc5, 0xc5, 0xc5, 0xc5, 0xc5, 0xc5, 0xc5, 0xc5, 0xc5, 
	0xc6, 0xc6, 0xc6, 0xc7, 0xc7, 0xc7, 0xc8, 0xc8, 0xc8, 0xc9, 0xc9, 0xc9, 0xca, 0xca, 0xca, 0xca, 
	0xca, 0xca, 0xcc, 0xcc, 0xcc, 0xc3, 0xc3, 0xc4, 0xcb, 0xcb, 0xcb, 0xce, 0xce, 0xce, 0xce, 0xce, 
	0xce, 0xce, 0xce, 0xce, 0xce, 0xce, 0xce, 0xcf, 0xcf, 0xcf, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 0xd0, 
	0xd0, 0xd0, 0xd0, 0xce, 0xce, 0xce, 0xca, 0xca, 0xca, 0xc1, 0xc1, 0xc1, 0xb4, 0xb4, 0xb4, 0xa1, 
	0xa1, 0xa1, 0x96, 0x96, 0x96, 0x9a, 0x9a, 0x9a, 0xa9, 0xa9, 0xa9, 0xa8, 0xa8, 0xa8, 0x30, 0x30, 
	0x32, 0x05, 0x05, 0x06, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 
	0x16, 0x16, 0x1a, 0x09, 0x09, 0x0b, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 
	0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 0x07, 0x06, 0x06, 
	0x07, 0x05, 0x05, 0x06, 0x16, 0x16, 0x19, 0x0a, 0x0a, 0x0b, 0x06, 0x06, 0x07, 0x15, 0x15, 0x16, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x41, 0x41, 0x44, 0xad, 0xad, 0xac, 0xab, 0xab, 0xaa, 0xac, 0xac, 0xab, 0xad, 0xad, 0xac, 
	0xae, 0xae, 0xae, 0xaf, 0xaf, 0xaf, 0xb1, 0xb1, 0xb0, 0xb3, 0xb3, 0xb2, 0xb4, 0xb4, 0xb4, 0xb5, 
	0xb5, 0xb4, 0xb9, 0xb9, 0xb9, 0x9d, 0x9d, 0xa0, 0xb3, 0xb3, 0xb3, 0xbb, 0xbb, 0xbb, 0xbb, 0xbb, 
	0xbb, 0xbd, 0xbd, 0xbd, 0xc1, 0xc1, 0xc0, 0xc5, 0xc5, 0xc4, 0xc8, 0xc8, 0xc8, 0xcc, 0xcc, 0xcc, 
	0xcf, 0xcf, 0xcf, 0xd1, 0xd1, 0xd1, 0xd1, 0xd1, 0xd0, 0xcd, 0xcd, 0xcc, 0xb7, 0xb7, 0xba, 0xb8, 
	0xb8, 0xb8, 0xa3, 0xa3, 0xa2, 0x91, 0x91, 0x91, 0x9b, 0x9b, 0x9b, 0xa8, 0xa8, 0xa8, 0x99, 0x99, 
	0x98, 0x0f, 0x0f, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x0e, 0x0e, 0x10, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x09, 0x09, 0x0a, 0x0f, 0x0f, 0x13, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x18, 0x18, 
	0x1e, 0x36, 0x36, 0x3a, 0x7d, 0x7d, 0x7f, 0x7f, 0x7f, 0x81, 0x80, 0x80, 0x82, 0x80, 0x80, 0x83, 
	0x81, 0x81, 0x84, 0x82, 0x82, 0x84, 0x83, 0x83, 0x85, 0x84, 0x84, 0x87, 0x86, 0x86, 0x89, 0x86, 
	0x86, 0x89, 0x88, 0x88, 0x8a, 0x77, 0x77, 0x7c, 0x83, 0x83, 0x86, 0x88, 0x88, 0x8b, 0x88, 0x88, 
	0x8b, 0x89, 0x89, 0x8b, 0x8b, 0x8b, 0x8d, 0x8d, 0x8d, 0x90, 0x92, 0x92, 0x95, 0x97, 0x97, 0x9a, 
	0x9f, 0x9f, 0xa1, 0xa6, 0xa6, 0xa9, 0xad, 0xad, 0xaf, 0xab, 0xab, 0xae, 0x91, 0x91, 0x98, 0xa4, 
	0xa4, 0xa7, 0x98, 0x98, 0x9b, 0x84, 0x84, 0x86, 0x7e, 0x7e, 0x81, 0x88, 0x88, 0x8b, 0x8b, 0x8b, 
	0x8d, 0x3c, 0x3c, 0x3e, 0x0d, 0x0d, 0x10, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x17, 0x17, 0x1b, 0x1f, 0x1f, 0x26, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x22, 0x13, 0x13, 0x17, 0x1c, 0x1c, 0x1e, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x0a, 0x0a, 0x0b, 0x16, 0x16, 
	0x1a, 0x08, 0x08, 0x09, 0x4b, 0x4b, 0x4c, 0x65, 0x65, 0x66, 0x65, 0x65, 0x66, 0x67, 0x67, 0x68, 
	0x68, 0x68, 0x68, 0x6a, 0x6a, 0x6b, 0x6c, 0x6c, 0x6c, 0x6d, 0x6d, 0x6e, 0x6f, 0x6f, 0x70, 0x70, 
	0x70, 0x70, 0x6c, 0x6c, 0x6e, 0x62, 0x62, 0x67, 0x6f, 0x6f, 0x70, 0x70, 0x70, 0x70, 0x6f, 0x6f, 
	0x70, 0x6f, 0x6f, 0x70, 0x70, 0x70, 0x71, 0x73, 0x73, 0x74, 0x77, 0x77, 0x77, 0x7d, 0x7d, 0x7e, 
	0x85, 0x85, 0x86, 0x8f, 0x8f, 0x8f, 0xa0, 0xa0, 0xa1, 0xaf, 0xaf, 0xb0, 0xa2, 0xa2, 0xa9, 0xbf, 
	0xbf, 0xc0, 0xb8, 0xb8, 0xb9, 0xa3, 0xa3, 0xa4, 0x8e, 0x8e, 0x8f, 0x8a, 0x8a, 0x8b, 0x8f, 0x8f, 
	0x90, 0x85, 0x85, 0x86, 0x12, 0x12, 0x13, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x06, 0x06, 0x07, 0x18, 0x18, 0x1d, 0x06, 0x06, 0x08, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x09, 0x09, 0x0b, 0x1c, 0x1c, 0x21, 0x14, 0x14, 0x15, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x15, 0x15, 0x19, 0x08, 0x08, 
	0x0a, 0x00, 0x00, 0x00, 0x0a, 0x0a, 0x0a, 0x1d, 0x1d, 0x1d, 0x1f, 0x1f, 0x1f, 0x24, 0x24, 0x24, 
	0x2c, 0x2c, 0x2c, 0x2f, 0x2f, 0x2f, 0x31, 0x31, 0x31, 0x32, 0x32, 0x32, 0x34, 0x34, 0x34, 0x35, 
	0x35, 0x35, 0x38, 0x38, 0x3a, 0x32, 0x32, 0x35, 0x36, 0x36, 0x35, 0x34, 0x34, 0x34, 0x33, 0x33, 
	0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x35, 0x35, 0x35, 0x39, 0x39, 0x39, 0x46, 0x46, 0x45, 
	0x51, 0x51, 0x51, 0x63, 0x63, 0x63, 0x80, 0x80, 0x80, 0x93, 0x93, 0x94, 0x99, 0x99, 0x9e, 0xc8, 
	0xc8, 0xc8, 0xc8, 0xc8, 0xc7, 0xb6, 0xb6, 0xb6, 0xa1, 0xa1, 0xa1, 0x8c, 0x8c, 0x8c, 0x86, 0x86, 
	0x86, 0x8f, 0x8f, 0x8e, 0x2f, 0x2f, 0x2f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x12, 0x12, 0x16, 0x02, 0x02, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x11, 0x15, 0x15, 0x16, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x08, 0x10, 0x10, 0x13, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0c, 0x0c, 0x0e, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x30, 0x30, 0x2f, 0x5d, 0x5d, 0x5d, 0x7f, 0x7f, 0x84, 0xb0, 
	0xb0, 0xb0, 0xcc, 0xcc, 0xcb, 0xc2, 0xc2, 0xc2, 0xac, 0xac, 0xab, 0x8f, 0x8f, 0x8f, 0x7b, 0x7b, 
	0x7b, 0x87, 0x87, 0x87, 0x5e, 0x5e, 0x5e, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0e, 0x0e, 0x10, 0x0a, 0x0a, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x04, 0x05, 0x26, 0x26, 0x29, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0d, 0x0d, 0x10, 0x1d, 0x1d, 0x23, 0x16, 0x16, 0x1a, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 
	0x0e, 0x11, 0x1b, 0x1b, 0x20, 0x10, 0x10, 0x14, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 
	0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0e, 0x0e, 0x10, 0x0d, 0x0d, 0x10, 
	0x0d, 0x0d, 0x0f, 0x0b, 0x0b, 0x0e, 0x0a, 0x0a, 0x0d, 0x1a, 0x1a, 0x1d, 0x53, 0x53, 0x5a, 0x81, 
	0x81, 0x84, 0xa0, 0xa0, 0xa2, 0xa8, 0xa8, 0xaa, 0x97, 0x97, 0x9a, 0x81, 0x81, 0x83, 0x68, 0x68, 
	0x6b, 0x6c, 0x6c, 0x6f, 0x6c, 0x6c, 0x6f, 0x1c, 0x1c, 0x1e, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x15, 0x15, 0x18, 0x20, 0x20, 0x26, 0x0f, 0x0f, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x24, 0x24, 0x28, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x0f, 0x0f, 0x12, 0x13, 0x13, 0x17, 0x04, 0x04, 0x04, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x0d, 
	0x0d, 0x0f, 0x1d, 0x1d, 0x23, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x03, 0x03, 0x03, 0x23, 0x23, 0x29, 0x65, 
	0x65, 0x66, 0x9c, 0x9c, 0x9d, 0xbd, 0xbd, 0xbe, 0xb0, 0xb0, 0xb1, 0x93, 0x93, 0x94, 0x72, 0x72, 
	0x73, 0x6d, 0x6d, 0x6d, 0x70, 0x70, 0x71, 0x22, 0x22, 0x23, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x05, 0x05, 0x06, 0x18, 0x18, 0x1c, 0x06, 0x06, 0x07, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x12, 0x12, 0x12, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x1d, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 
	0x0d, 0x10, 0x05, 0x05, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x1e, 0x1e, 0x23, 0x33, 
	0x33, 0x33, 0x93, 0x93, 0x93, 0xbe, 0xbe, 0xbe, 0xc0, 0xc0, 0xc0, 0x9f, 0x9f, 0x9f, 0x7b, 0x7b, 
	0x7b, 0x6a, 0x6a, 0x6a, 0x70, 0x70, 0x70, 0x25, 0x25, 0x25, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x14, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0a, 0x0a, 0x0b, 0x0e, 0x0e, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 
	0x0e, 0x10, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x38, 0x38, 0x3d, 0x0a, 
	0x0a, 0x0a, 0x76, 0x76, 0x76, 0xb3, 0xb3, 0xb3, 0xc5, 0xc5, 0xc5, 0xa6, 0xa6, 0xa5, 0x81, 0x81, 
	0x81, 0x68, 0x68, 0x68, 0x6f, 0x6f, 0x6f, 0x34, 0x34, 0x34, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x11, 0x0d, 0x0d, 0x10, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x1d, 0x1d, 0x24, 0x0a, 0x0a, 0x0c, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x12, 
	0x12, 0x15, 0x08, 0x08, 0x0a, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x11, 0x11, 0x12, 0x3b, 0x3b, 0x41, 0x02, 
	0x02, 0x03, 0x64, 0x64, 0x65, 0xa6, 0xa6, 0xa7, 0xbc, 0xbc, 0xbd, 0xa3, 0xa3, 0xa4, 0x7f, 0x7f, 
	0x7f, 0x64, 0x64, 0x65, 0x67, 0x67, 0x67, 0x31, 0x31, 0x32, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x0b, 0x0b, 0x0d, 0x1b, 0x1b, 0x20, 0x05, 0x05, 0x06, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x13, 0x13, 0x14, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 0x18, 
	0x18, 0x1d, 0x1a, 0x1a, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x14, 0x14, 0x18, 0x22, 
	0x22, 0x28, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x19, 0x19, 0x1c, 0x3e, 0x3e, 0x44, 0x0f, 
	0x0f, 0x11, 0x51, 0x51, 0x54, 0x91, 0x91, 0x94, 0xa7, 0xa7, 0xaa, 0x95, 0x95, 0x98, 0x76, 0x76, 
	0x79, 0x5e, 0x5e, 0x60, 0x5c, 0x5c, 0x5f, 0x32, 0x32, 0x35, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x10, 0x10, 0x12, 0x20, 0x20, 0x26, 0x0f, 0x0f, 0x12, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x14, 
	0x14, 0x17, 0x02, 0x02, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x0f, 0x05, 
	0x05, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x0d, 0x0c, 0x3b, 0x3b, 0x3f, 0x00, 
	0x00, 0x00, 0x50, 0x50, 0x50, 0xaa, 0xaa, 0xaa, 0xc8, 0xc8, 0xc8, 0xb1, 0xb1, 0xb1, 0x89, 0x89, 
	0x89, 0x67, 0x67, 0x67, 0x62, 0x62, 0x62, 0x2c, 0x2c, 0x2c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x12, 0x03, 0x03, 0x04, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0d, 0x0e, 
	0x0e, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x10, 0x04, 
	0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x0e, 0x3a, 0x3a, 0x3f, 0x00, 
	0x00, 0x00, 0x63, 0x63, 0x63, 0xa7, 0xa7, 0xa7, 0xc5, 0xc5, 0xc5, 0xb1, 0xb1, 0xb1, 0x88, 0x88, 
	0x88, 0x66, 0x66, 0x66, 0x5f, 0x5f, 0x5f, 0x2a, 0x2a, 0x2a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x10, 0x11, 0x11, 0x14, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x13, 0x13, 0x16, 0x01, 
	0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x14, 0x14, 0x17, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0b, 0x39, 0x39, 0x3e, 0x00, 
	0x00, 0x00, 0x6c, 0x6c, 0x6b, 0xae, 0xae, 0xad, 0xcc, 0xcc, 0xcb, 0xb5, 0xb5, 0xb4, 0x8b, 0x8b, 
	0x8a, 0x67, 0x67, 0x66, 0x5d, 0x5d, 0x5c, 0x27, 0x27, 0x27, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x16, 0x16, 0x1a, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x29, 0x29, 0x2e, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x22, 0x22, 0x29, 0x23, 0x23, 0x2a, 0x1b, 
	0x1b, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 
	0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 
	0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x22, 0x22, 0x28, 0x2b, 0x2b, 0x33, 0x1c, 
	0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 
	0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 
	0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x24, 0x24, 0x29, 0x3c, 0x3c, 0x44, 0x1e, 
	0x1e, 0x23, 0x5b, 0x5b, 0x60, 0x7d, 0x7d, 0x82, 0x8b, 0x8b, 0x90, 0x7d, 0x7d, 0x82, 0x67, 0x67, 
	0x6d, 0x55, 0x55, 0x5a, 0x4e, 0x4e, 0x53, 0x33, 0x33, 0x38, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 
	0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1d, 0x1d, 0x23, 0x29, 0x29, 0x31, 0x1d, 
	0x1d, 0x22, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 
	0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x1c, 0x1c, 0x21, 0x29, 0x29, 0x2e, 
	0x17, 0x17, 0x18, 0x08, 0x08, 0x0a, 0x0a, 0x0a, 0x0c, 0x1a, 0x1a, 0x1e, 0x09, 0x09, 0x0b, 0x08, 
	0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 
	0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 
	0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x15, 0x15, 0x19, 0x0c, 0x0c, 0x0e, 0x08, 
	0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 
	0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 
	0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x0f, 0x0f, 0x11, 0x5c, 0x5c, 0x62, 0x3c, 
	0x3c, 0x3e, 0x80, 0x80, 0x82, 0xa5, 0xa5, 0xa6, 0xb4, 0xb4, 0xb6, 0x9c, 0x9c, 0x9d, 0x7b, 0x7b, 
	0x7d, 0x5e, 0x5e, 0x5f, 0x52, 0x52, 0x54, 0x2b, 0x2b, 0x2c, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 
	0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x17, 0x17, 0x1b, 0x0e, 
	0x0e, 0x10, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 
	0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x08, 0x08, 0x0a, 0x17, 0x17, 0x18, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x11, 0x11, 0x14, 0x0a, 0x0a, 0x0c, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0e, 0x0e, 0x11, 0x03, 0x03, 0x04, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x05, 0x05, 0x6d, 0x6d, 0x73, 0x56, 
	0x56, 0x56, 0x93, 0x93, 0x93, 0xb9, 0xb9, 0xb9, 0xc7, 0xc7, 0xc7, 0xa7, 0xa7, 0xa7, 0x81, 0x81, 
	0x80, 0x62, 0x62, 0x62, 0x50, 0x50, 0x50, 0x18, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x0e, 0x12, 
	0x12, 0x15, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x04, 0x04, 0x05, 0x14, 0x14, 0x18, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x07, 0x19, 0x19, 0x1e, 0x01, 0x01, 0x02, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x6d, 0x6d, 0x72, 0x73, 
	0x73, 0x73, 0x9e, 0x9e, 0x9d, 0xbf, 0xbf, 0xbf, 0xc2, 0xc2, 0xc2, 0xa1, 0xa1, 0xa1, 0x81, 0x81, 
	0x81, 0x5d, 0x5d, 0x5d, 0x24, 0x24, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x19, 
	0x19, 0x1d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x16, 0x16, 0x1a, 0x09, 0x09, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x0d, 0x10, 0x0d, 0x0d, 0x10, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x08, 0x60, 0x60, 0x65, 0x83, 
	0x83, 0x83, 0xa7, 0xa7, 0xa8, 0xc2, 0xc2, 0xc3, 0xc7, 0xc7, 0xc7, 0x9f, 0x9f, 0x9f, 0x5b, 0x5b, 
	0x5b, 0x17, 0x17, 0x17, 0x02, 0x02, 0x02, 0x05, 0x05, 0x07, 0x03, 0x03, 0x04, 0x07, 0x07, 0x08, 
	0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x05, 0x06, 0x08, 0x08, 0x0a, 0x15, 
	0x15, 0x1a, 0x08, 0x08, 0x0a, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x11, 0x11, 0x11, 0x0d, 0x0d, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0e, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x68, 0x68, 0x6c, 0x85, 
	0x85, 0x83, 0x63, 0x63, 0x6f, 0x92, 0x92, 0x98, 0x66, 0x66, 0x68, 0x26, 0x26, 0x26, 0x06, 0x06, 
	0x07, 0x11, 0x11, 0x15, 0x21, 0x21, 0x27, 0x4d, 0x4d, 0x5d, 0x24, 0x24, 0x2d, 0x46, 0x46, 0x54, 
	0x3a, 0x3a, 0x45, 0x08, 0x08, 0x09, 0x00, 0x00, 0x00, 0x21, 0x21, 0x27, 0x4d, 0x4d, 0x5c, 0x22, 
	0x22, 0x29, 0x45, 0x45, 0x53, 0x2b, 0x2b, 0x33, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x37, 0x37, 0x40, 0x26, 0x26, 0x2e, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 
	0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 
	0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 
	0x21, 0x21, 0x28, 0x22, 0x22, 0x29, 0x2a, 0x2a, 0x33, 0x23, 0x23, 0x2a, 0x21, 0x21, 0x28, 0x21, 
	0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 
	0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 
	0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x19, 0x1d, 0x24, 0x23, 0x29, 0x31, 0x67, 0x6a, 0x74, 0x32, 
	0x35, 0x3c, 0x25, 0x25, 0x2e, 0x2b, 0x2b, 0x33, 0x1b, 0x1b, 0x22, 0x1b, 0x1b, 0x22, 0x21, 0x21, 
	0x28, 0x24, 0x24, 0x2c, 0x25, 0x25, 0x2d, 0x2a, 0x2a, 0x33, 0x26, 0x26, 0x2e, 0x28, 0x28, 0x30, 
	0x28, 0x28, 0x31, 0x25, 0x25, 0x2d, 0x21, 0x21, 0x28, 0x23, 0x23, 0x2b, 0x29, 0x29, 0x31, 0x2a, 
	0x2a, 0x32, 0x30, 0x30, 0x3a, 0x26, 0x26, 0x2e, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 
	0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x21, 0x21, 0x28, 0x2e, 0x2e, 0x35, 
	0x24, 0x24, 0x28, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 
	0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 
	0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 
	0x0a, 0x0a, 0x0c, 0x12, 0x12, 0x15, 0x23, 0x23, 0x29, 0x09, 0x09, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 
	0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 
	0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 
	0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x52, 0x30, 0x32, 0xae, 0x7b, 0x7d, 0xb7, 0x9b, 0xa1, 0x67, 
	0x4e, 0x50, 0x0a, 0x09, 0x0b, 0x07, 0x07, 0x09, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 
	0x0c, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0b, 0x08, 0x08, 0x0a, 0x09, 0x09, 0x0b, 0x09, 0x09, 0x0a, 
	0x09, 0x09, 0x0a, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x09, 0x09, 0x0b, 0x08, 0x08, 0x0a, 0x0c, 
	0x0c, 0x0f, 0x1e, 0x1e, 0x24, 0x09, 0x09, 0x0b, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 
	0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x0a, 0x0a, 0x0c, 0x18, 0x18, 0x1a, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0d, 0x0d, 0x0f, 0x09, 0x09, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x14, 0x0a, 0x0a, 0x5c, 0x4d, 0x4d, 0x99, 0x93, 0x97, 0x48, 
	0x37, 0x37, 0x06, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x11, 0x11, 0x14, 0x03, 0x03, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x0e, 0x0e, 0x11, 0x06, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x06, 0x07, 0x27, 0x28, 0x2d, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x0f, 0x0f, 0x11, 0x0d, 0x0d, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x03, 0x03, 0x04, 0x17, 0x17, 0x1b, 0x02, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x19, 0x19, 0x1e, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x08, 0x08, 0x09, 0x19, 0x19, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x0a, 0x0a, 0x0c, 0x18, 0x18, 0x1d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x03, 0x03, 0x04, 0x19, 0x19, 0x1d, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x0d, 0x0d, 0x10, 0x07, 0x07, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x1b, 0x1b, 0x20, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x11, 0x04, 0x04, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x10, 
	0x1a, 0x1a, 0x1f, 0x13, 0x13, 0x16, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x11, 0x11, 0x14, 0x24, 0x24, 0x2b, 0x10, 
	0x10, 0x12, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x19, 0x19, 0x1e, 0x1a, 0x1a, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x06, 0x06, 0x07, 
	0x1a, 0x1a, 0x1f, 0x06, 0x06, 0x07, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x07, 0x07, 0x09, 0x1e, 0x1e, 0x24, 0x06, 
	0x06, 0x07, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x0a, 0x0a, 0x0c, 0x1e, 0x1e, 0x24, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x13, 0x13, 0x14, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x14, 
	0x17, 0x17, 0x1a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x1c, 0x1c, 0x21, 0x01, 
	0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x16, 0x16, 0x1a, 0x02, 0x02, 0x02, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x1a, 0x1a, 0x1f, 
	0x0a, 0x0a, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x13, 0x07, 0x07, 0x08, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x03, 0x14, 0x14, 0x19, 
	0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x0e, 0x12, 0x12, 0x14, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x04, 0x04, 0x10, 0x10, 0x14, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x1b, 0x1b, 0x20, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x12, 0x0f, 0x0f, 0x11, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x14, 0x04, 0x04, 
	0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x18, 0x18, 0x1d, 0x07, 0x07, 0x08, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x13, 0x07, 0x07, 
	0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x12, 0x12, 0x16, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x02, 0x1b, 0x1b, 0x20, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x09, 0x09, 0x0b, 0x14, 0x14, 
	0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x1c, 0x1c, 0x1f, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x17, 0x1b, 0x1b, 0x20, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x11, 0x11, 0x14, 0x26, 0x26, 0x2c, 0x10, 
	0x10, 0x13, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 
	0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 
	0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 
	0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x13, 0x13, 0x16, 0x24, 0x24, 
	0x2b, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x0e, 0x0e, 0x11, 0x1c, 0x1c, 0x1f, 
	0x13, 0x13, 0x14, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x1a, 0x1a, 0x1e, 0x0f, 0x0f, 0x11, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x07, 0x07, 0x09, 0x1f, 0x1f, 0x24, 0x06, 
	0x06, 0x07, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 
	0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 
	0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 
	0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x13, 0x13, 
	0x17, 0x09, 0x09, 0x0b, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x04, 0x04, 0x05, 0x13, 0x13, 0x14, 
	0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x16, 0x16, 0x1b, 0x02, 0x02, 0x03, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x02, 0x03, 0x1c, 0x1c, 0x21, 0x01, 
	0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x0d, 
	0x10, 0x0a, 0x0a, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 
	0x0b, 0x0b, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x03, 0x13, 0x13, 0x17, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0x04, 0x1c, 0x1c, 0x21, 0x02, 
	0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 
	0x0a, 0x1d, 0x1d, 0x22, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0b, 0x0b, 0x0b, 
	0x1e, 0x1e, 0x1e, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x18, 0x18, 0x19, 0x1d, 0x1d, 0x1f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x12, 0x12, 0x13, 0x29, 0x29, 0x2e, 0x11, 
	0x11, 0x11, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 
	0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x12, 0x12, 
	0x13, 0x24, 0x24, 0x27, 0x12, 0x12, 0x12, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x1e, 0x1e, 0x1e
};
        #endregion
    }



}
